#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/serial_8250.h>
#include <linux/clk.h>
#include <linux/dma-mapping.h>
#include <linux/pda_power.h>
#include <linux/io.h>
#include <linux/i2c.h>
#include <linux/gpio.h>
#include <linux/leds.h>
#include <linux/leds_pwm.h>
#include <linux/switch.h>
#include <linux/input/matrix_keypad.h>
#include <linux/mmc/host.h>

#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/time.h>
#include <asm/setup.h>
#include <mach/hardware.h>
#include <mach/irqs.h>
#include <mach/platform.h>
#include <mach/i2c.h>
#include <mach/gs_plat.h>
#include <linux/init-input.h>
#include <sound/tlv320aic3x.h>
#include "cpu-freq/cpu-freq.h"
#include "../../../drivers/usb/host/sw_hci_sun7i.h"
#include "../../../drivers/video/sun7i/disp/drv_disp_i.h"
#include "../../../drivers/mmc/host/sunxi-mci.h"

static char *gxv_pn;
#define SW_EHCI_NAME		"sw-ehci"
#define SW_OHCI_NAME		"sw-ohci"
#define	SW_USB_NAME			"sw-usb-host"

static __initdata struct gpio_config gpio_set_default[] = {
	/* i2c0 */
	{GPIOB(0), 2, GPIO_PULL_UP, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOB(1), 2, GPIO_PULL_UP, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOB(2), 2, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},/* pwm0 */
    /* I2S Port */
	{GPIOB(5), 2, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOB(6), 2, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOB(7), 2, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOB(8), 2, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOB(12), 2, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	/* i2c1 */
	{GPIOB(18), 2, GPIO_PULL_UP, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOB(19), 2, GPIO_PULL_UP, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	/* i2c2 */
	{GPIOB(20), 2, GPIO_PULL_UP, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOB(21), 2, GPIO_PULL_UP, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	/*CSI0*/
	{GPIOE(0), 3, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOE(1), 3, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOE(2), 3, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOE(3), 3, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOE(4), 3, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOE(5), 3, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOE(6), 3, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOE(7), 3, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOE(8), 3, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOE(9), 3, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOE(10), 3, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOE(11), 3, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOG(0), GPIO_CFG_OUTPUT, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},/* codec reset */
	{GPIOG(2), GPIO_CFG_OUTPUT, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},/* usb0 enable */
	{GPIOG(3), GPIO_CFG_OUTPUT, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},/* usb1 enable */
	{GPIOG(4), GPIO_CFG_OUTPUT, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, 0},/* led_red */
	{GPIOG(5), GPIO_CFG_OUTPUT, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, 0},/* led_green */
	{GPIOG(8), GPIO_CFG_OUTPUT, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},/* Mic switch */
	{GPIOG(9), GPIO_CFG_OUTPUT, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},/* Speak switch */
	{GPIOG(10), GPIO_CFG_OUTPUT, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, 0},/* key led red */
	{GPIOG(11), GPIO_CFG_OUTPUT, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, 0},/* key led green */
	{GPIOH(3), GPIO_CFG_EINT, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},/* touchscreen intr */
	/* keyboard row */
	{GPIOH(8), GPIO_CFG_INPUT, GPIO_PULL_UP, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOH(9), GPIO_CFG_INPUT, GPIO_PULL_UP, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOH(10), GPIO_CFG_INPUT, GPIO_PULL_UP, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOH(11), GPIO_CFG_INPUT, GPIO_PULL_UP, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	{GPIOH(14), GPIO_CFG_INPUT, GPIO_PULL_UP, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	/* keyboard col */
	{GPIOH(18), GPIO_CFG_OUTPUT, GPIO_PULL_UP, 2, GPIO_DATA_DEFAULT},
	{GPIOH(19), GPIO_CFG_OUTPUT, GPIO_PULL_UP, 2, GPIO_DATA_DEFAULT},
	{GPIOH(22), GPIO_CFG_OUTPUT, GPIO_PULL_UP, 2, GPIO_DATA_DEFAULT},
	{GPIOH(23), GPIO_CFG_OUTPUT, GPIO_PULL_UP, 2, GPIO_DATA_DEFAULT},
	{GPIOH(24), GPIO_CFG_OUTPUT, GPIO_PULL_UP, 2, GPIO_DATA_DEFAULT},
	{GPIOH(25), GPIO_CFG_OUTPUT, GPIO_PULL_UP, 2, GPIO_DATA_DEFAULT},
	/* ext */
	{GPIOH(20), GPIO_CFG_OUTPUT, GPIO_PULL_DOWN, 2, GPIO_DATA_DEFAULT},
	{GPIOH(21), GPIO_CFG_INPUT, GPIO_PULL_UP, 2, GPIO_DATA_DEFAULT},
	/* i2c3 */
	{GPIOI(0), 3, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},/* pwm1 */
	{GPIOI(1), 3, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},/* pwm1 */
	{GPIOI(3), 2, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},/* pwm1 */
};

#ifdef CONFIG_CPU_FREQ_DVFS
struct cpufreq_dvfs cpufreq_dvfs_table[] = {
	{.freq = 1008000000, .volt = 1450},
	{.freq = 912000000, .volt = 1400},
	{.freq = 864000000, .volt = 1300},
	{.freq = 720000000, .volt = 1200},
	{.freq = 528000000, .volt = 1100},
	{.freq = 312000000, .volt = 1000},
	{.freq = 144000000, .volt = 1000},
	{.freq = 0, .volt = 1000},
};
#endif

static struct sw_hci_hcd sw_ehci0 = {
	.used					= 1,
	.usbc_no				= 1,
	.usbc_type				= SW_USB_EHCI,
	.irq_no					= AW_IRQ_USB1,
	.drv_vbus_gpio_valid	= 1,
	.drv_vbus_gpio_set		= GPIOG(2),
	.host_init_state		= 1
};

static struct sw_hci_hcd sw_ohci0 = {
	.used					= 1,
	.usbc_no				= 1,
	.usbc_type				= SW_USB_OHCI,
	.irq_no					= AW_IRQ_USB3,
	.drv_vbus_gpio_valid	= 1,
	.drv_vbus_gpio_set		= GPIOG(2),
	.host_init_state		= 1
};

static struct sw_hci_hcd sw_ehci1 = {
		.used					= 1,
		.usbc_no				= 2,
		.usbc_type				= SW_USB_EHCI,
		.irq_no					= AW_IRQ_USB2,
		.drv_vbus_gpio_valid	= 1,
		.drv_vbus_gpio_set		= GPIOG(3),
		.host_init_state		= 1
};

static struct sw_hci_hcd sw_ohci1 = {
		.used					= 1,
		.usbc_no				= 2,
		.usbc_type				= SW_USB_OHCI,
		.irq_no					= AW_IRQ_USB4,
		.drv_vbus_gpio_valid	= 1,
		.drv_vbus_gpio_set		= GPIOG(3),
		.host_init_state		= 1
};

static u64 sw_ehci_dmamask = DMA_BIT_MASK(32);
static u64 sw_ohci_dmamask = DMA_BIT_MASK(32);


static struct platform_device sw_usb_device[] = {
	/* usb host 1 */
#ifdef  CONFIG_USB_SW_SUN7I_EHCI0
	 {
		.name		= SW_EHCI_NAME,
		.id			= 1,
		.dev		= {
			.dma_mask			= &sw_ehci_dmamask,
			.coherent_dma_mask	= DMA_BIT_MASK(32),
			.platform_data		= &sw_ehci0,
		},
	},
#endif
#ifdef  CONFIG_USB_SW_SUN7I_OHCI0
	{
		.name		= SW_OHCI_NAME,
		.id			= 1,
		.dev		= {
			.dma_mask			= &sw_ohci_dmamask,
			.coherent_dma_mask	= DMA_BIT_MASK(32),
			.platform_data		= &sw_ohci0,
		},
	},
#endif

#ifdef  CONFIG_USB_SW_SUN7I_EHCI1
	/* usb host 2 */
	{
		.name		= SW_EHCI_NAME,
		.id			= 2,
		.dev		= {
			.dma_mask			= &sw_ehci_dmamask,
			.coherent_dma_mask	= DMA_BIT_MASK(32),
			.platform_data		= &sw_ehci1,
		},
	},
#endif
#ifdef  CONFIG_USB_SW_SUN7I_OHCI1
	{
		.name		= SW_OHCI_NAME,
		.id			= 2,
		.dev		= {
			.dma_mask			= &sw_ohci_dmamask,
			.coherent_dma_mask	= DMA_BIT_MASK(32),
			.platform_data		= &sw_ohci1,
		},
	},
#endif
};

struct resource usb_probe_res[] = {
	[0] = {
		.start	= ARRAY_SIZE(sw_usb_device),
		.end	= ARRAY_SIZE(sw_usb_device),
		.flags	= IORESOURCE_IO,
	}
};

static struct platform_device sw_usb_probe_device = {
	.name			= SW_USB_NAME,
	.id				= 0,
	.resource		= usb_probe_res,
	.num_resources	= ARRAY_SIZE(usb_probe_res),
	.dev			= {
		.platform_data	= sw_usb_device,
	},

};

static struct resource hdmi_resource[1] = {
	[0] = {
		.start = 0x01c16000,
		.end   = 0x01c165ff,
		.flags = IORESOURCE_MEM,
	},
};

/* A10/A10S initialization sequence:  disp mode come first, then lcd and hdmi mode */
static struct resource disp_resource[] =
{
	[DISP_IO_SCALER0] = {
		.start = 0x01e00000,
		.end   = 0x01e0077f,
		.flags = IORESOURCE_MEM,
	},
	[DISP_IO_SCALER1] = {
		.start = 0x01e20000,
		.end   = 0x01e2077f,
		.flags = IORESOURCE_MEM,
	},
	[DISP_IO_IMAGE0] = {
		.start = 0x01e60000,
		.end   = 0x01e657ff,
		.flags = IORESOURCE_MEM,
	},
	[DISP_IO_IMAGE1] = {
		.start = 0x01e40000,
		.end   = 0x01e457ff,
		.flags = IORESOURCE_MEM,
	},
	[DISP_IO_LCDC0] = {
		.start = 0x01c0c000,
		.end   = 0x01c0cfff,
		.flags = IORESOURCE_MEM,
	},
	[DISP_IO_LCDC1] = {
		.start = 0x01c0d000,
		.end   = 0x01c0dfff,
		.flags = IORESOURCE_MEM,
	},
	[DISP_IO_TVEC0] = {
		.start = 0x01c0a000,
		.end   = 0x01c0afff,
		.flags = IORESOURCE_MEM,
	},
	[DISP_IO_TVEC1] = {
		.start = 0x01c1b000,
		.end   = 0x01c1bfff,
		.flags = IORESOURCE_MEM,
	},
};

static __panel_para_t lcd0_para = {

		.lcd_x				= 480,
		.lcd_y				= 272,
		.lcd_dclk_freq		= 9,	//lcd_dclk_freq * 10*1000 HZ//MHZ
		.lcd_pwm_not_used	= 0,
		.lcd_pwm_ch			= 0,
		.lcd_pwm_freq		= 17000,	 //Hz
		.lcd_pwm_pol		= 0,
		.lcd_if				= 0,		//0:hv(sync+de); 1:8080; 2:ttl; 3:lvds
		.lcd_hbp			= 40,		//hsync back porch
		.lcd_ht				= 525,	//hsync total cycle
		.lcd_hspw		= 38,		 //hsync plus width
		.lcd_vbp			= 10,		//vsync back porch
		.lcd_vt				= 2 * 286,	//vysnc total cycle *2
		.lcd_vspw		= 8,		//vysnc plus width
		.lcd_hv_if			= 0,		//0:hv parallel 1:hv serial
		.lcd_hv_smode		= 0,		//0:RGB888 1:CCIR656
		.lcd_hv_s888_if		= 0,		//serial RGB format
		.lcd_hv_syuv_if		= 0,		//serial YUV format
		.lcd_cpu_if			= 0,		//0:18bit 4:16bit
		.lcd_frm			= 0,		//0: disable; 1: enable rgb666 dither; 2:enable rgb656 dither
		.lcd_lvds_ch		= 0,		//0:single channel; 1:dual channel
		.lcd_lvds_mode		= 0,		//0:NS mode; 1:JEIDA mode
		.lcd_lvds_bitwidth	= 0,	//0:24bit; 1:18bit
		.lcd_lvds_io_cross	= 0,		//0:normal; 1:pn cross
		.lcd_io_cfg0		= 0x10000000,	 //lcd_dclk_freq>40 should be 0x0 or 0x04000000
		.lcd_gamma_correction_en  = 0,
};

static struct lcd_sys_cfg lcd0_cfg = {
	.used			= 1,
	.lcd_bl_used	= 1,
	.lcd_bl_en      = GPIO_PORT("bl_en", GPIOI(21), 1, 0, -1, 1),
	.lcd_pwm_used	= 1,
	.lcd_pwm		= GPIO_PORT("pwm_pin", GPIOB(2), 2, -1, -1, 0),
	.lcd_power_used	= 0,
	.lcd_power_en	= GPIO_PORT("power en", GPIOI(21), 1, 0, -1, 1),
	.init_bri		= 197,
	.bright			= 50,
	.contrast		= 50,
	.saturation		= 57,
	.hue			= 50,
	.lcd_para		= &lcd0_para,
	.lcd_pin		= {
		GPIO_PORT("lcdd0", GPIOD(0), 2, 0, -1, 0),
		GPIO_PORT("lcdd1", GPIOD(1), 2, 0, -1, 0),
		GPIO_PORT("lcdd2", GPIOD(2), 2, 0, -1, 0),
		GPIO_PORT("lcdd3", GPIOD(3), 2, 0, -1, 0),
		GPIO_PORT("lcdd4", GPIOD(4), 2, 0, -1, 0),
		GPIO_PORT("lcdd5", GPIOD(5), 2, 0, -1, 0),
		GPIO_PORT("lcdd6", GPIOD(6), 2, 0, -1, 0),
		GPIO_PORT("lcdd7", GPIOD(7), 2, 0, -1, 0),
		GPIO_PORT("lcdd8", GPIOD(8), 2, 0, -1, 0),
		GPIO_PORT("lcdd9", GPIOD(9), 2, 0, -1, 0),
		GPIO_PORT("lcdd10", GPIOD(10), 2, 0, -1, 0),
		GPIO_PORT("lcdd11", GPIOD(11), 2, 0, -1, 0),
		GPIO_PORT("lcdd12", GPIOD(12), 2, 0, -1, 0),
		GPIO_PORT("lcdd13", GPIOD(13), 2, 0, -1, 0),
		GPIO_PORT("lcdd14", GPIOD(14), 2, 0, -1, 0),
		GPIO_PORT("lcdd15", GPIOD(15), 2, 0, -1, 0),
		GPIO_PORT("lcdd16", GPIOD(16), 2, 0, -1, 0),
		GPIO_PORT("lcdd17", GPIOD(17), 2, 0, -1, 0),
		GPIO_PORT("lcdd18", GPIOD(18), 2, 0, -1, 0),
		GPIO_PORT("lcdd19", GPIOD(19), 2, 0, -1, 0),
		GPIO_PORT("lcdd20", GPIOD(20), 2, 0, -1, 0),
		GPIO_PORT("lcdd21", GPIOD(21), 2, 0, -1, 0),
		GPIO_PORT("lcdd22", GPIOD(22), 2, 0, -1, 0),
		GPIO_PORT("lcdd23", GPIOD(23), 2, 0, -1, 0),
		GPIO_PORT("lcdclk", GPIOD(24), 2, 0, -1, 0),
		GPIO_PORT("lcdde ", GPIOD(25), 2, 0, -1, 0),
		GPIO_PORT("lcdhsync", GPIOD(26), 2, 0, -1, 0),
		GPIO_PORT("lcdvsync", GPIOD(27), 2, 0, -1, 0),
	},
};

struct disp_cfg disp_conf = {
	.disp_init_en	= 1,
	.disp_mode		= 0,
	.scn_type		= {
		1, 1
	},
	.scn_mode		= {
		4, 4
	},
	.fb_fb_num		= {
		2, 2
	},
	.fb_format		= {
		10, 10
	},
	.fb_pix_seq		= {
		0, 0
	},
	.fb_sca_mode	= {
		0, 0
	},
	.lcd_cfg		= {
		&lcd0_cfg,
		NULL
	}
};

struct platform_device disp_device = {
	.name           = "disp",
	.id		        = -1,
	.num_resources  = ARRAY_SIZE(disp_resource),
	.resource	    = disp_resource,
	.dev            = {
		.platform_data	= &disp_conf,
	}
};


/* This device is for A10&A10s embendded HDMI */
struct platform_device hdmi_device = {
	.name           = "hdmi",
	.id		        = -1,
	.num_resources  = ARRAY_SIZE(hdmi_resource),
	.resource	    = hdmi_resource,
	.dev            = {}
};

static struct resource csi0_resource[] = {
	[0] = {
		.start	= SW_PA_CSI0_IO_BASE,
		.end	= SW_PA_CSI0_IO_BASE + 0x1000 - 1,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= AW_IRQ_CSI0,
		.end	= AW_IRQ_CSI0,
		.flags	= IORESOURCE_IRQ,
	},
};

struct csi_dev_cfg csi0_cam0_cfg = {
	.dev_no		= 0,
	.twi_id		= 2,
	.twi_addr	= 0x54,
	.csi_if		= 0,
	.vflip		= 0,
	.hflip		= 1,
	.flash_pol	= 1,
	.cam_name	= "nt99141",
	.csi_reset  = {
		GPIOI(20), GPIO_CFG_OUTPUT, GPIO_PULL_DEFAULT,
		GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},/* csi0 reset */
	.csi_stby	= {
		.gpio	= GPIO_INDEX_INVALID,
	},
	.csi_power	= {
		.gpio	= GPIO_INDEX_INVALID,
	},
	.csi_flash	= {
		.gpio	= GPIO_INDEX_INVALID,
	},
	.csi_af	= {
		.gpio	= GPIO_INDEX_INVALID,
	},
};

struct csi_plat csi0_plat = {
	.dev_num	= 1,
	.stby_mode	= 1,
	.cfg		= {
		&csi0_cam0_cfg,
		NULL
	}
};

void csi_dev_release(struct device *dev)
{
}

static struct platform_device csi0_device[] = {
	[0] = {
		.name			= "sunxi_csi0",
		.id				= 0,
		.num_resources	= ARRAY_SIZE(csi0_resource),
		.resource		= csi0_resource,
		.dev			= {
			.release		= csi_dev_release,
			.platform_data	= &csi0_plat,
		},
	},
};

/* SDCard device */
static struct sunxi_mmc_platform_data sw_mci_pdata[] = {
	[0] = {
		.ocr_avail = MMC_VDD_28_29 | MMC_VDD_29_30 | MMC_VDD_30_31 | MMC_VDD_31_32
				| MMC_VDD_32_33 | MMC_VDD_33_34,
		.caps = MMC_CAP_4_BIT_DATA | MMC_CAP_MMC_HIGHSPEED | MMC_CAP_SD_HIGHSPEED
			| MMC_CAP_SDIO_IRQ
			| MMC_CAP_SET_XPC_330 | MMC_CAP_DRIVER_TYPE_A,
		.f_min = 400000,
		.f_max = 50000000,
		.f_ddr_max = 47000000,
		.dma_tl= 0x20070008,
		.regulator=NULL,
		.used = 1,
		.cdmode = 1,
		.width = 4,
		.wpmode = 1,
		.mmcio = {
			[0] = {
				.gpio = GPIOF(2),/* sdc_clk */
				.mul_sel = 2,
				.pull = 1,
				.drv_level = 2,
			},
			[1] = {
				.gpio = GPIOF(3),/* sdc_cmd */
				.mul_sel = 2,
				.pull = 1,
				.drv_level = 2,
			},
			[2] = {
				.gpio = GPIOF(1),/* sdc_d0 */
				.mul_sel = 2,
				.pull = 1,
				.drv_level = 2,
			},
			[3] = {
				.gpio = GPIOF(0),/* sdc_d1 */
				.mul_sel = 2,
				.pull = 1,
				.drv_level = 2,
			},
			[4] = {
				.gpio = GPIOF(5),/* sdc_d2 */
				.mul_sel = 2,
				.pull = 1,
				.drv_level = 2,
			},
			[5] = {
				.gpio = GPIOF(4),/* sdc_d3 */
				.mul_sel = 2,
				.pull = 1,
				.drv_level = 2,
			},
		},
		.cd = {
			.gpio = GPIOH(1),
			.mul_sel = 0,
			.pull = 1,
			.drv_level = 2,
		},
		.wp = {
			.gpio = GPIOH(2),
			.mul_sel = 0,
			.pull = 1,
			.drv_level = 2,
		},
	},
	[1] = {
		.ocr_avail = MMC_VDD_28_29 | MMC_VDD_29_30 | MMC_VDD_30_31 | MMC_VDD_31_32
				| MMC_VDD_32_33 | MMC_VDD_33_34,
		.caps = MMC_CAP_4_BIT_DATA | MMC_CAP_MMC_HIGHSPEED | MMC_CAP_SD_HIGHSPEED
			| MMC_CAP_SDIO_IRQ,
		.f_min = 400000,
		.f_max = 50000000,
		.dma_tl= 0x20070008,
		.regulator=NULL,
		.used = 1,
		.cdmode = 4,
		.width = 4,
		.wpmode = 0,
		.isiodev = 1,
		.mmcio = {
			[0] = {
				.gpio = GPIOI(5),/* sdc_clk */
				.mul_sel = 2,
				.pull = 1,
				.drv_level = 2,
			},
			[1] = {
				.gpio = GPIOI(4),/* sdc_cmd */
				.mul_sel = 2,
				.pull = 1,
				.drv_level = 2,
			},
			[2] = {
				.gpio = GPIOI(6),/* sdc_d0 */
				.mul_sel = 2,
				.pull = 1,
				.drv_level = 2,
			},
			[3] = {
				.gpio = GPIOI(7),/* sdc_d1 */
				.mul_sel = 2,
				.pull = 1,
				.drv_level = 2,
			},
			[4] = {
				.gpio = GPIOI(8),/* sdc_d2 */
				.mul_sel = 2,
				.pull = 1,
				.drv_level = 2,
			},
			[5] = {
				.gpio = GPIOI(9),/* sdc_d3 */
				.mul_sel = 2,
				.pull = 1,
				.drv_level = 2,
			},
		},
	},
};

static struct platform_device sw_mci_device[] = {
	[0] = {
		.name = "sunxi-mmc",
		.id = 0,
		.dev = {
			.platform_data = &sw_mci_pdata[0],
		},
	},
	[1] = {
		.name = "sunxi-mmc",
		.id = 3,
		.dev = {
			.platform_data = &sw_mci_pdata[1],
		},
	},
};

static struct ctp_config_info ctp_pdata = {
	/* 480x272 */
	.ctp_used             = 1,
	.twi_id			= 1,
	.screen_max_x   = 480,
	.screen_max_y   = 272,
	.revert_x_flag  = 1,
	.revert_y_flag  = 0,
	.exchange_x_y_flag = 1,
	.int_number = GPIOH(3),
	.reset_number = GPIOG(1),
};

static struct platform_device ctp_device = {
	.name = "ctp_gt9xx",
	.dev = {
		.platform_data = &ctp_pdata,
	},
};

static struct resource sunxi_pcm_resource[] = {
	[0] = {
		.start	= SW_PA_IIS0_IO_BASE,
		.end	= SW_PA_IIS0_IO_BASE + 0x100 - 1,
		.flags	= IORESOURCE_MEM,
	},
};

static struct platform_device sunxi_i2s_device = {
	.name			= "sun7i-i2s",
	.num_resources	= ARRAY_SIZE(sunxi_pcm_resource),
	.resource		= sunxi_pcm_resource,
};

/* gxv3240 sound device */
static struct gs_board_snd_pdata gxv3275_board_snd_pdata = {
	.hf_spk_sw = GPIOG(9),
	.hs_hf_sw = GPIOG(8),
};

static struct platform_device gxv3275_snd_device = {
	.name = "snd-gxv3275-mach-drv",
	.id = -1,
	.dev = {
		.platform_data = &gxv3275_board_snd_pdata,
	},
};

static struct platform_device gxv_bt_pcm_dev = {
	.name = "rda5876",
	.id = -1,
};

static struct aic3x_pdata gxv3275_aic3x_pdata = {
	.gpio_reset = GPIOG(0),
};

static struct i2c_board_info __initdata i2c2_devices[] = {
	{
	I2C_BOARD_INFO("tlv320aic3x", 0x18),
		.platform_data = &gxv3275_aic3x_pdata,
	},
};

/* GXV3240 leds */
static struct gpio_led gpio_leds[] = {
	{
		.name            = "led_green",
		.default_trigger = "default-off",
		.gpio            = GPIOG(5),
		.active_low      = 1,
	},
	{
		.name            = "led_red",
		.default_trigger = "default-off",
		.gpio            = GPIOG(4),
		.active_low      = 1,
	},
	{
		.name            = "mute",
		.default_trigger = "default-off",
		.gpio            = GPIOG(10),
		.active_low      = 1,
	},
	{
		.name            = "headset",
		.default_trigger = "default-off",
		.gpio            = GPIOG(11),
		.active_low      = 1,
	},
};

static struct gpio_led_platform_data gpio_led_info = {
	.leds           = gpio_leds,
	.num_leds       = ARRAY_SIZE(gpio_leds),
};

static struct platform_device leds_device = {
	.name           = "leds-gpio",
	.id = -1,
	.dev = {
		.platform_data = &gpio_led_info,
	}
};

/* hook switch */
static struct gs_hook_pdata hook_pdata = {
	.gpio	= GPIOH(0),
};

static struct platform_device gxv3275_hksw = {
	.name	= "hook-switch",
	.id		= -1,
	.dev	= {
		.platform_data	= &hook_pdata,
	},
};

/* gxp2260 keyboard device */
static const uint32_t gxv3240_keymap[] = {
	KEY(0, 0, KEY_A), KEY(0, 1, KEY_B),
	KEY(0, 2, KEY_1), KEY(0, 3, KEY_4),
	KEY(0, 4, KEY_7), KEY(0, 5, KEY_C),
	KEY(1, 0, KEY_D), KEY(1, 1, KEY_E),
	KEY(1, 2, KEY_2), KEY(1, 3, KEY_5),
	KEY(1, 4, KEY_8), KEY(1, 5, KEY_0),
	KEY(2, 0, KEY_WWW), KEY(2, 1, KEY_G),
	KEY(2, 2, KEY_3), KEY(2, 3, KEY_6),
	KEY(2, 4, KEY_9), KEY(2, 5, KEY_H),
	KEY(3, 0, KEY_I), KEY(3, 1, KEY_K),
	KEY(3, 2, KEY_G), KEY(3, 3, KEY_L),
	KEY(3, 4, KEY_M), KEY(3, 5, KEY_VOLUMEDOWN),
	KEY(4, 0, KEY_BACK), KEY(4, 1, KEY_P),
	KEY(4, 2, KEY_Q), KEY(4, 3, KEY_R),
	KEY(4, 4, KEY_S), KEY(4, 5, KEY_VOLUMEUP),
	KEY(5, 0, KEY_VOLUMEUP)		/* hksw_button */
};

static const struct matrix_keymap_data gxv3240_keymap_data = {
	.keymap		= gxv3240_keymap,
	.keymap_size	= ARRAY_SIZE(gxv3240_keymap),
};

static unsigned int gxv3240_row_gpios[] = {
	GPIOH(8), GPIOH(9), GPIOH(10),
	GPIOH(11), GPIOH(14),
};

static unsigned int gxv3240_col_gpios[] = {
	GPIOH(18), GPIOH(19), GPIOH(22),
	GPIOH(23), GPIOH(24), GPIOH(25),
};

static struct matrix_keypad_platform_data gxv3240_kbd_pdata = {
	.keymap_data = &gxv3240_keymap_data,
	.col_gpios	= gxv3240_col_gpios,
	.row_gpios	= gxv3240_row_gpios,
	.num_col_gpios	= ARRAY_SIZE(gxv3240_col_gpios),
	.num_row_gpios	= ARRAY_SIZE(gxv3240_row_gpios),
	.col_scan_delay_us	= 10,
	.debounce_ms		= 10,
	.wakeup			= 1,
	.active_low		= 1,
};

static struct platform_device gxv3240_keypad = {
	.name		= "matrix-keypad",
	.id		= -1,
	.dev		= {
		.platform_data = &gxv3240_kbd_pdata,
	},
};


struct mmc_pm_plat sdio_wifi = {
	.sdio_card_used		= 1,
	.sdio_hostid		= 3,
	.module_sel			= 3,	/* 8723as */
	.wl_dis				= {GPIOG(7), 1, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
	.bt_dis				= {GPIOG(6), 1, GPIO_PULL_DEFAULT, GPIO_DRVLVL_DEFAULT, GPIO_DATA_DEFAULT},
};

static struct platform_device wifi_pm_dev = {
	.name           = "wifi_pm",
	.dev			= {
		.platform_data	= &sdio_wifi,
	},
};

/* TP/ADC device */
static struct resource adc_resource[] = {
	[0] = {
		.start	= SW_PA_TP_IO_BASE,
		.end	= SW_PA_TP_IO_BASE + 0x100 - 1,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= AW_IRQ_TOUCH_PANEL,
		.end	= AW_IRQ_TOUCH_PANEL,
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device sunxi_adc_device = {
	.name	= "sunxi-adc",
	.id		= -1,
	.num_resources	= ARRAY_SIZE(adc_resource),
	.resource		= adc_resource,
};


/* earphone/ehs detected device */
static struct gs_switch_platform_data switch_pdata = {
	.earphone_sw	= GPIOH(27),
	.headset_sw		= GPIOH(25),
};

static struct platform_device gs_switch = {
	.name	= "gs-switch",
	.id		= -1,
	.dev	= {
		.platform_data = &switch_pdata,
	},
};

static struct ext_con_platform_data ext_con_info = {
	.i2c_bus = 4,
	.ext_sel = GPIOH(20),
	.ext_int = GPIOH(21),
};

static struct platform_device ext_device = {
	.name          = "ext_connect",
	.dev = {
		.platform_data = &ext_con_info,
	}
};

static struct platform_device *board_pdevs[] __initdata = {
//	&wifi_pm_dev,
	&sw_usb_probe_device,
	&disp_device,
	//&hdmi_device,
	&csi0_device[0],
	&sw_mci_device[0],
	&sw_mci_device[1],
	&ctp_device,
	&sunxi_i2s_device,
	&gxv3275_snd_device,
	&gxv_bt_pcm_dev,
	&leds_device,
	&gxv3275_hksw,
	&gxv3240_keypad,
	&sunxi_adc_device,
	&gs_switch,
	&ext_device,
};

static int __init hw_version_setup(char *s)
{
	gxv_pn = s;

	return 1;
}

__setup("pn=", hw_version_setup);

int gxv_hw_version(void)
{
	int version = 0x12A;

	if (gxv_pn)
		sscanf(gxv_pn, "%x", &version);

	return version & 0xfff;
}
EXPORT_SYMBOL(gxv_hw_version);

int __init gs_board_init(void)
{
	printk(KERN_INFO "GXV3240 board init.\n");
#ifdef CONFIG_CPU_FREQ_DVFS
	sunxi_cpufreq_dvfs_tbl_register(cpufreq_dvfs_table,
			ARRAY_SIZE(cpufreq_dvfs_table));
#endif
	sunxi_cpufreq_set_range(1008000000, 312000000);
	sw_gpio_setall_range(gpio_set_default, ARRAY_SIZE(gpio_set_default));
	i2c_register_board_info(2, i2c2_devices, ARRAY_SIZE(i2c2_devices));
	platform_add_devices(board_pdevs, ARRAY_SIZE(board_pdevs));
	return 0;
}

arch_initcall(gs_board_init);
