/*
 *  arch/arm/mach-$chip/devices.c
 *
 *  Copyright (C) 2012 AllWinner Limited
 *  Benn Huang <benn@allwinnertech.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/serial_8250.h>
#include <linux/clk.h>
#include <linux/dma-mapping.h>
#include <linux/pda_power.h>
#include <linux/io.h>
#include <linux/i2c.h>
 
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/time.h>
#include <asm/setup.h>
#include <mach/includes.h>

/* uart */
static struct plat_serial8250_port debug_uart_platform_data[] = {
	{
		.membase        = (void __iomem *)SW_VA_UART0_IO_BASE,
		.mapbase        = (resource_size_t)SW_PA_UART0_IO_BASE,
		.irq            = 33,
		.flags          = UPF_BOOT_AUTOCONF|UPF_IOREMAP,
		.iotype         = UPIO_MEM32,
		.regshift       = 2,
		.uartclk        = 24000000,
	}, {
		.flags          = 0,
	}
 };

static struct platform_device debug_uart = {
	.name = "serial8250",
	.id = PLAT8250_DEV_PLATFORM,
	.dev = {
		.platform_data = &debug_uart_platform_data[0],
	},
};

/* dma */
static u64 sw_dmac_dmamask = DMA_BIT_MASK(32);

static struct resource sw_dmac_resources[] = {
	[0] = {
		.start 	= SW_PA_DMAC_IO_BASE,
		.end 	= SW_PA_DMAC_IO_BASE + 0x1000,
		.flags 	= IORESOURCE_MEM,
	},
	[1] = {
		.start 	= AW_IRQ_DMA,
		.end 	= AW_IRQ_DMA,
		.flags 	= IORESOURCE_IRQ
	}
};

static struct platform_device sw_dmac_device = {
	.name 		= "sw_dmac",	/* must be same as sw_dmac_driver's name */
	.id 		= 0, 		/* there is only one device for sw_dmac dirver, so id is 0 */
	.num_resources 	= ARRAY_SIZE(sw_dmac_resources),
	.resource 	= sw_dmac_resources,
	.dev 		= {
				.dma_mask = &sw_dmac_dmamask,
				.coherent_dma_mask = DMA_BIT_MASK(32),	/* validate dma_pool_alloc */
			  },
};

struct platform_device sw_pdev_nand =
{
	.name = "sw_nand",
	.id = -1,
};
static struct resource sun7i_twi0_resources[] = {
	{
		.start	= TWI0_BASE_ADDR_START,
		.end	= TWI0_BASE_ADDR_END,
		.flags	= IORESOURCE_MEM,
	},
	{
		.start	= AW_IRQ_TWI0,
		.end	= AW_IRQ_TWI0,
		.flags	= IORESOURCE_IRQ,
	},
};

static struct sun7i_i2c_platform_data sun7i_twi0_pdata[] = {
	{
		.bus_num   = 0,
		.frequency = TWI0_TRANSFER_SPEED,
	},
};

struct platform_device sun7i_twi0_device = {
	.name		= "sun7i-i2c",
	.id		= 0,
	.resource	= sun7i_twi0_resources,
	.num_resources	= ARRAY_SIZE(sun7i_twi0_resources),
	.dev = {
		.platform_data = sun7i_twi0_pdata,
	},
};

/* twi1 */
static struct resource sun7i_twi1_resources[] = {
	{
		.start	= TWI1_BASE_ADDR_START,
		.end	= TWI1_BASE_ADDR_END,
		.flags	= IORESOURCE_MEM,
	}, 
	{
		.start	= AW_IRQ_TWI1,
		.end	= AW_IRQ_TWI1,
		.flags	= IORESOURCE_IRQ,
	},
};

static struct sun7i_i2c_platform_data sun7i_twi1_pdata[] = {
	{
		.bus_num   = 1,
    	        .frequency = TWI1_TRANSFER_SPEED,
	},
};

struct platform_device sun7i_twi1_device = {
	.name		= "sun7i-i2c",
	.id             = 1,
	.resource	= sun7i_twi1_resources,
	.num_resources	= ARRAY_SIZE(sun7i_twi1_resources),
	.dev = {
		.platform_data = sun7i_twi1_pdata,
	},
};

/* twi2 */
static struct resource sun7i_twi2_resources[] = {
	{
		.start	= TWI2_BASE_ADDR_START,
		.end	= TWI2_BASE_ADDR_END,
		.flags	= IORESOURCE_MEM,
	},
        {
		 .start	= AW_IRQ_TWI2,
		 .end	= AW_IRQ_TWI2,
		 .flags	= IORESOURCE_IRQ,
	 },
};

static struct sun7i_i2c_platform_data sun7i_twi2_pdata[] = {
	{
		.bus_num   = 2,
    	        .frequency = TWI2_TRANSFER_SPEED,
	},
};

struct platform_device sun7i_twi2_device = {
	.name		= "sun7i-i2c",
	.id             = 2,
	.resource	= sun7i_twi2_resources,
	.num_resources	= ARRAY_SIZE(sun7i_twi2_resources),
	.dev = {
		.platform_data = sun7i_twi2_pdata,
	},
};

/* twi3 */
static struct resource sun7i_twi3_resources[] = {
	{
		.start	= TWI3_BASE_ADDR_START,
		.end	= TWI3_BASE_ADDR_END,
		.flags	= IORESOURCE_MEM,
	},
        {
		 .start	= AW_IRQ_TWI3,
		 .end	= AW_IRQ_TWI3,
		 .flags	= IORESOURCE_IRQ,
	 },
};

static struct sun7i_i2c_platform_data sun7i_twi3_pdata[] = {
	{
		.bus_num   = 3,
    	        .frequency = TWI3_TRANSFER_SPEED,
	},
};

struct platform_device sun7i_twi3_device = {
	.name		= "sun7i-i2c",
	.id             = 3,
	.resource	= sun7i_twi3_resources,
	.num_resources	= ARRAY_SIZE(sun7i_twi3_resources),
	.dev = {
		.platform_data = sun7i_twi3_pdata,
	},
};

#ifdef CONFIG_GS_GVR3550
struct platform_device gs_mem_device = {
	.name		= "gs_mem",
	.id             = 0,
	.dev = {
		.platform_data = NULL,
	},
};
#endif
static struct platform_device *sw_pdevs[] __initdata = {
	&debug_uart,
	&sw_dmac_device,
	&sw_pdev_nand,
	&sun7i_twi0_device,
	&sun7i_twi1_device,
	&sun7i_twi2_device,
	&sun7i_twi3_device,
#ifdef CONFIG_GS_GVR3550
	&gs_mem_device,
#endif
};

void __init sw_pdev_init(void)
{
	platform_add_devices(sw_pdevs, ARRAY_SIZE(sw_pdevs));
}

