/* for gs specified platform data, added by kehua @ 2013-4-8 */
#ifndef __GS_PLAT_H__
#define __GS_PLAT_H__

#include <mach/sys_config.h>
#include <mach/gpio.h>
#include <linux/drv_display.h>
/* this datastruct is mainly for sdio wifi and some bluetooth chip 	*
  * @sdio_card_used:  if any sdio wifi is used in this platform       	*
  * @sdio_hostid:   which sd host connect by sdio wifi 			*
  * @module_sel:   which wifi module is used 					*
  * @wl_wps:		gpio for wifi wps control 					*
  * @wl_dis:		gpio for wifi disable/enable					*
  * @bt_dis:		gpio for bluetooth dis/enable				*
  */
 
struct mmc_pm_plat {
	u8 sdio_card_used;
	u8 sdio_hostid;
	u8 module_sel;
	struct gpio_config wl_dis;
	struct gpio_config bt_dis;
};

/* this datastruct is mainly for csi camera interface 							*
  * dev_no:	camera no , 0 for first caemra, 1 for second 
  * stby_mode: standy by mode 
  * twi_id:  i2c host id 
  * twi_addr: i2c  client addr 
  * cam_name: camera module name
  * csi_if: this parameter is queit important, 
  *		0:hv_8bit 1:hv_16bit 2:hv_24bit 3:bt656 1ch 4:bt656 2ch 5:bt656 4ch	
  * vflip:	vertical flip
  * hflip:  horizonal flip
  * csi_reset: reset gpio
  */
#define MAX_CAMERA_NUM	2
struct csi_dev_cfg {
	u8 dev_no;
	u8 twi_id;
	u8 twi_addr;
	u8 csi_if;
	u8 vflip;
	u8 hflip;
	u8 flash_pol;
	char *cam_name;	
	char *csi_iovdd;
	char *csi_avdd;
	char *csi_dvdd;
	struct gpio_config csi_reset;
	struct gpio_config csi_stby;
	struct gpio_config csi_power;
	struct gpio_config csi_flash;
	struct gpio_config csi_af;
};

struct csi_plat {
	u8 dev_num;
	u8 stby_mode;
	struct csi_dev_cfg *cfg[MAX_CAMERA_NUM];
};

/* lcd_sys_cfg is used to configure lcd on A10 platform		*
  * @used:			is lcd used?
  * @lcd_bl_used:		Is lcd backlight used ?
  * @lcd_pw_used:		Is lcd power used?
  * @lcd_pwm_used:	using pwm to control backlight?
  * @lcd_bl_en:		en/disable backlight pin
  * @lcd_pw_en:		en/disable lcd power
  * @lcd_pwm:		pin to output backlight pwm
  * @lcd_gpio:			don't know what it is used for
  * @lcd_pin:			the physical pin connect to LCD
  * @init_bri:			this parameter is for bakelight's initial value, 0 ~255
  **/
struct disp_gpio_set
{
        char    gpio_name[32];
        int     port;
        int     port_num;
        int     mul_sel;
        int     pull;
        int     drv_level;
        int     data;
        int     gpio;
};

#define GPIO_PORT(gname, gport, gmux_sel, gport_pull, gport_drv, gport_dat) \
{ .gpio_name = gname, .gpio = gport, \
   .mul_sel = gmux_sel, .pull = gport_pull, .drv_level = gport_drv, .data = gport_dat}

struct lcd_sys_cfg {
	u8 used;
	u8 lcd_bl_used;
	u8 lcd_power_used;
	u8 lcd_pwm_used;
	u8 lcd_reset_used;
	struct disp_gpio_set lcd_bl_en;
	struct disp_gpio_set lcd_power_en;
	struct disp_gpio_set lcd_pwm;
	struct disp_gpio_set lcd_reset;
	struct disp_gpio_set lcd_gpio[4];
	struct disp_gpio_set lcd_pin[28];

	u8 lcd_if;
	u32 init_bri;

	/* the following three parameter is just for display enigne 
	 * and not for lcd, value: 0~100 */
	u32 bright;
	u32	contrast;
	u32 saturation;
	u32 hue;

	__panel_para_t *lcd_para;
};

/* disp configuration 
  * @ disp_init_en:		enable display engine initilaztion
  * @ disp_mode:	0:screen0<screen0,fb0> 1:screen1<screen1,fb0>
  *		2:two_diff_screen_diff_contents<screen0,screen1,fb0,fb1>
  *		3:two_same_screen_diff_contets<screen0,screen1,fb0> 
  *		4:two_diff_screen_same_contents<screen0,scree     n1,fb0>)
  * @scn0_type: 0:none; 1:lcd; 2:tv; 3:hdmi; 4:vga
  * @scn0_mode:	only for tv/hdmi output
  *		tv/hdmi output, 0:480i 1:576i 2:480p 3:576p 4:720p50 
  *		5:720p60 6:1080i50 7:1080i60 8:1080p
  *		9:1080p50 10:1080p60 11:pal 14:ntsc
  * @
  **/
#define MAX_DISP_NUM 2
struct disp_cfg {
	u8 disp_init_en;
	u8 disp_mode;
	u8 scn_type[MAX_DISP_NUM];
	u8 scn_mode[MAX_DISP_NUM];
	u8 fb_fb_num[MAX_DISP_NUM];
	u8 fb_format[MAX_DISP_NUM];
	u8 fb_pix_seq[MAX_DISP_NUM];
	u8 fb_sca_mode[MAX_DISP_NUM];
	struct lcd_sys_cfg *lcd_cfg[MAX_DISP_NUM];
};

#ifndef __MMC_DETECT_MODE__
#define __MMC_DETECT_MODE__
#define CARD_DETECT_BY_GPIO     (1)
#define CARD_DETECT_BY_DATA3    (2)        /* mmc detected by status of data3 */
#define CARD_ALWAYS_PRESENT     (3)        /* mmc always present, without detect pin */
#define CARD_DETECT_BY_FS		(4)		   /* mmc insert/remove by manual mode, from /proc/awsmc.x/insert node */
#endif
#define MAX_MMC_PIN	8

struct sunxi_mmc_plat {
	unsigned short sdc_no;
	unsigned short sdc_detmode;
	unsigned short data_wide;
	unsigned short sdc_use_wp;
	int	sdc_pin[MAX_MMC_PIN];
};

struct gs_board_snd_pdata {
	int hf_spk_sw;
	int hs_spk_sw;
	int hs_hf_sw;
	int ehs_amp_sw;
};

struct gs_hook_pdata {
	int gpio;
};

struct gs_switch_platform_data {
	int earphone_sw;
	int headset_sw;
};

struct ext_con_platform_data {
	int i2c_bus;
	int ext_sel;
	int ext_int;
	int irq;
};

#endif
