/*  sun7i_user_dma.c: Version 1.0 2013.06.14

        sun7i dma driver for user malloc.

        This program is free software; you can redistribute it and/or
        modify it under the terms of the GNU General Public License
        as published by the Free Software Foundation; either version 2
        of the License, or (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.


  (C)Copyright 2013 Grandstream. All Rights Reserved.
        xchen@grandstream.com

*/

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/fcntl.h>
#include <linux/gfp.h>
#include <linux/interrupt.h>
#include <linux/init.h>
#include <linux/ioport.h>
#include <linux/in.h>
#include <linux/string.h>
#include <linux/delay.h>
#include <linux/errno.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/device.h>
#include <linux/skbuff.h>
#include <linux/platform_device.h>
#include <linux/dma-mapping.h>
#include <linux/slab.h>
#include <linux/io.h>
#include <asm/pgtable.h>
#include <asm/dma.h>
#include <linux/kthread.h>
#include <linux/delay.h>
#include <asm/dma-mapping.h>
#include <linux/wait.h>
#include <linux/random.h>
#include <mach/dma.h>
#include "sun7i_user_dma.h"
#include "dma_common.h"
#include "dma_csp.h"
#include <asm/cacheflush.h>
#include <linux/cdev.h>
#include <asm/uaccess.h>
#include <linux/vmalloc.h>
#include <linux/mutex.h>
#include <asm/page.h>
#include <linux/highmem.h>

#define MAX_DMA_CHAL 16
extern u32 dma_hdl_irq_fd(dma_channel_t *pchan);

static struct cdev	*user_dma_cdev;
static struct class	*user_dma_class;
static dev_t		 devid;

struct dma_chan_list
{
    dma_hdl_t dma_hdl;
    char name[64];
    int start;
    //spinlock_t dma_lock;
    struct mutex dma_mutex;
#define DMA_CHAN_TYPE_OFFSET 0
#define DMA_WAIT_TYPE_OFFSET 4
#define DMA_BITS_TYPE_OFFSET 8
#define DMA_BUF_TYPE_OFFSET 12
#define DMA_USE_INT 0
#define DMA_USE_POLL 1
#define DMA_USER_BUF_USE_MALLOC 0
#define DMA_USER_BUF_USE_GSMALLOC 0x7
    int type; 
    /*
     * type
     * 31                            0
     * |----------------------------|
     * |                            |
     * |----------------------------|
     * 
     */
    atomic_t dma_down;
    wait_queue_head_t dma_wq;
};

static struct dma_chan_list user_dma_list[MAX_DMA_CHAL];

static int user_dma_wait_done(dma_channel_t *pchan, int chan_id)
{
	unsigned int timeout = 0x10000;
    unsigned int pend_bits = 0;
    int type = user_dma_list[chan_id].type;
    if((type >> DMA_WAIT_TYPE_OFFSET) & 0xf)
    {
        while (timeout-- > 0) {
            pend_bits = csp_dma_get_irqpend(pchan);
            if (pend_bits & CHAN_IRQ_HD)
            {
                user_dma_dbg("dma hd  timeount = %d\n", timeout);
                csp_dma_clear_irqpend(pchan, CHAN_IRQ_HD);  
            }
            if(pend_bits & CHAN_IRQ_FD)
            {
                user_dma_dbg("dma down timeount = %d\n", timeout);
                csp_dma_clear_irqpend(pchan, CHAN_IRQ_FD);  
                dma_hdl_irq_fd(pchan);
                return 0;
            }
        }
	}
    else
    {
        wait_event_interruptible_timeout(user_dma_list[chan_id].dma_wq, atomic_read(&user_dma_list[chan_id].dma_down) == 1, msecs_to_jiffies(500));
	    atomic_set(&user_dma_list[chan_id].dma_down, 0);
        return 0;
    }

	user_dma_dbg("dma%d: failed to stop?\n", pchan->id);
    return 1;
}

u32 virtaddr_to_phys(u32 virtp)
{
    unsigned long pa = 0;
    struct vm_area_struct *vma = NULL;
    struct mm_struct *mm = current->mm;
    pgd_t *pgd_tmp = NULL;
    pud_t *pud_tmp = NULL;
    pmd_t *pmd_tmp = NULL;
    pte_t *pte_tmp = NULL;

    vma = find_vma(mm, virtp);
    if(vma == NULL)
    {
        return virtp;
    }

    user_dma_dbg("vma = 0x%x\n", (unsigned long)vma);
    user_dma_dbg("vma = 0x%p start 0x%x end 0x%x\n", vma, vma->vm_start, vma->vm_end);
    user_dma_dbg("pgd = 0x%p\n", mm->pgd);
    pgd_tmp = pgd_offset(mm, virtp);
    user_dma_dbg("pgd_tmp = 0x%p\n",pgd_tmp);
    user_dma_dbg("pgd_val(*pgd_tmp) = 0x%lx\n",pgd_val(*pgd_tmp));
    if(pgd_none(*pgd_tmp)){
        printk("Not mapped in pgd.\n");        
        return 0;
    }
    pud_tmp = pud_offset(pgd_tmp, virtp);
    user_dma_dbg("pud_tmp = 0x%p\n",pud_tmp);
    user_dma_dbg("pud_val(*pud_tmp) = 0x%lx\n",pud_val(*pud_tmp));
    if(pud_none(*pud_tmp)){
            printk("Not mapped in pud.\n");
            return 0;
    }
    pmd_tmp = pmd_offset(pud_tmp, virtp);
    user_dma_dbg("pmd_tmp = 0x%p\n",pmd_tmp);
    user_dma_dbg("pmd_val(*pmd_tmp) = 0x%lx\n",pmd_val(*pmd_tmp));
    if(pmd_none(*pmd_tmp)){
            printk("Not mapped in pmd.\n");
            return 0;
    }
    //pte_tmp = pte_offset_kernel(pmd_tmp, virtp);
    pte_tmp = pte_offset_map(pmd_tmp, virtp);

    user_dma_dbg("pte_tmp = 0x%p\n",pte_tmp);
    user_dma_dbg("pte_val(*pte_tmp) = 0x%lx\n",pte_val(*pte_tmp));
    if(pte_none(*pte_tmp)){
            printk("Not mapped in pte.\n");
            pte_unmap(pte_tmp);
            return 0;
    }
    if(!pte_present(*pte_tmp)){
            printk("pte not in RAM.\n");
            pte_unmap(pte_tmp);
            return 0;
    }
    pa = (pte_val(*pte_tmp) & PAGE_MASK) | (virtp & ~PAGE_MASK);
    user_dma_dbg("virt_addr 0x%lx in RAM is 0x%lx .\n",virtp ,pa);
    pte_unmap(pte_tmp);
    //user_dma_dbg("contect in 0x%lx is 0x%lx\n",pa,
    //        *(unsigned long *)((char *)pa + PAGE_OFFSET));
    return pa;
}
EXPORT_SYMBOL(virtaddr_to_phys);

static int check_addr_and_flush(struct user_dma_info_ext *dma_info)
{
    int dma_buf_type = 0;
    dma_buf_type = ((dma_info->type) >> DMA_BUF_TYPE_OFFSET) & 0xf;
    if(DMA_USER_BUF_USE_MALLOC == dma_buf_type)
    {
        if(((dma_info->dst_virt & ~PAGE_MASK) + dma_info->len) > PAGE_SIZE)
        {
            printk("we do not support dst in two pages!!!\n");
            return -1;
        }
        else
        {
            dma_info->dst_phy[1] = 0;
        }

        dma_info->src_phy[0] = virtaddr_to_phys(dma_info->src_virt);
        dma_info->dst_phy[0] = virtaddr_to_phys(dma_info->dst_virt);
        if((0 == dma_info->src_phy[0]) || (0 == dma_info->dst_phy[0]))
        {
            return -1;
        }
        //we do not support dst in two pages,so we flush dst phy once
        dmac_flush_range((void *)dma_info->src_virt, (void *)dma_info->src_virt + dma_info->len);
        dmac_flush_range((void *)dma_info->dst_virt, (void *)dma_info->dst_virt + dma_info->len);
        outer_flush_range(dma_info->dst_phy[0], dma_info->dst_phy[0] + dma_info->len);
        if(((dma_info->src_virt & ~PAGE_MASK) + dma_info->len) > PAGE_SIZE)
        {
            dma_info->src_phy[1] = virtaddr_to_phys(PAGE_ALIGN(dma_info->src_virt));
            outer_flush_range(dma_info->src_phy[0], dma_info->src_phy[0] + PAGE_SIZE - (dma_info->src_virt & ~PAGE_MASK));
            outer_flush_range(dma_info->src_phy[1], dma_info->src_phy[1] + (dma_info->len - (PAGE_SIZE - (dma_info->src_virt & ~PAGE_MASK))));
        }
        else
        {
            dma_info->src_phy[1] = 0;
            outer_flush_range(dma_info->src_phy[0], dma_info->src_phy[0] + dma_info->len);
        }
    }
    else if(DMA_USER_BUF_USE_GSMALLOC == dma_buf_type)
    {
        dma_info->dst_phy[1] = 0;
        dma_info->src_phy[1] = 0;
        dma_info->src_phy[0] = virtaddr_to_phys(dma_info->src_virt);
        dma_info->dst_phy[0] = virtaddr_to_phys(dma_info->dst_virt);
        if((0 == dma_info->src_phy[0]) || (0 == dma_info->dst_phy[0]))
        {
            return -1;
        }
        dmac_flush_range((void *)dma_info->src_virt, (void *)dma_info->src_virt + dma_info->len);
        dmac_flush_range((void *)dma_info->dst_virt, (void *)dma_info->dst_virt + dma_info->len);
        outer_flush_range(dma_info->dst_phy[0], dma_info->dst_phy[0] + dma_info->len);
        outer_flush_range(dma_info->src_phy[0], dma_info->src_phy[0] + dma_info->len);
    }
    else
    {
        printk("Dma type err, please check you dma type!!!\n");
        return -1;
    }

    return 0;
}

static int dma_prepare(struct user_dma_info_ext *dma_info)
{
    int ret = -1;
    ret = check_addr_and_flush(dma_info);
    return ret;
}

void dma_fd_normal(dma_hdl_t dma_hdl, void *parg)
{
    int i = 0;
    int chan_id = 0;
    for(i = 0; i < MAX_DMA_CHAL; i++)
    {
        if(dma_hdl == user_dma_list[i].dma_hdl)
        {
            chan_id = i;
            break;
        }
    }
	atomic_set(&user_dma_list[chan_id].dma_down, 1);
	wake_up_interruptible(&user_dma_list[chan_id].dma_wq);
}

static dma_hdl_t user_dma_reuqest(char *name, int type)
{
    dma_cb_t done_cb;
    dma_hdl_t dma_hdl = (dma_hdl_t)NULL;
    dma_channel_t *pchan = NULL;
    dma_config_t dma_config;
    int dma_type = (type >> DMA_CHAN_TYPE_OFFSET) & 0xf;
    int dma_bit_type = (type >> DMA_BITS_TYPE_OFFSET) & 0xf;
    int dma_wait_type = (type >> DMA_WAIT_TYPE_OFFSET) & 0xf;
    unsigned int tmp;

    switch (dma_type)
    {
        case (int)CHAN_NORAML:
        case (int)CHAN_DEDICATE:
        {
            break;
        }
        default:
        {
            printk("type err ?, please check dma type, must be CHAN_NORAML or CHAN_DEDICATE\n");
            goto err_out;

        }
    }

    if(unlikely(NULL == name))
    {
        printk("name == NULL ?, please check dma type\n");
        goto err_out;
    }

	/* request dma channel */
	dma_hdl = sw_dma_request(name, dma_type);
    if(NULL == dma_hdl) 
    {
        printk("sw_dma_request user_m2m_dma err\n");
		goto err_out;
	}

    pchan = (dma_channel_t *)dma_hdl;

    if(DMA_USE_POLL == dma_wait_type)
    {
	    done_cb.func = NULL;
	    done_cb.parg = NULL;
    }
    else if(DMA_USE_INT == dma_wait_type)
    {
	    done_cb.func = dma_fd_normal;
	    done_cb.parg = NULL;
    }  
    else
    {
	    done_cb.func = NULL;
	    done_cb.parg = NULL;
    }

	if(0 != sw_dma_ctl(dma_hdl, DMA_OP_SET_FD_CB, (void *)&done_cb)) 
    {
        printk("sw_dma_ctl err\n");
		goto err_out;
	}

#if 1
	done_cb.func = NULL;
	done_cb.parg = NULL;
	if(0 != sw_dma_ctl(dma_hdl, DMA_OP_SET_HD_CB, (void *)&done_cb)) 
    {
        printk("sw_dma_ctl err\n");
		goto err_out;
	}
#endif

	memset(&dma_config, 0, sizeof(dma_config));
    dma_config.xfer_type.src_data_width 	= dma_bit_type;
    dma_config.xfer_type.src_bst_len 	= DATA_BRST_4;
    dma_config.xfer_type.dst_data_width 	= dma_bit_type;
    dma_config.xfer_type.dst_bst_len 	= DATA_BRST_4;

    if(dma_type == (int)CHAN_NORAML)
    {
        dma_config.address_type.src_addr_mode 	= NDMA_ADDR_INCREMENT;
        dma_config.address_type.dst_addr_mode 	= NDMA_ADDR_INCREMENT;
        dma_config.src_drq_type 	= N_SRC_SDRAM;
        dma_config.dst_drq_type 	= N_DST_SDRAM;
        dma_config.bconti_mode 		= false;
    }
    else if(dma_type == (int)CHAN_DEDICATE)
    {
        dma_config.address_type.src_addr_mode 	= DDMA_ADDR_LINEAR;
        dma_config.address_type.dst_addr_mode 	= DDMA_ADDR_LINEAR;
        dma_config.src_drq_type 	= D_SRC_SDRAM;
        dma_config.dst_drq_type 	= D_DST_SDRAM;
        dma_config.bconti_mode 		= false;
    }

    if(DMA_USE_POLL == dma_wait_type)
    {
        dma_config.irq_spt 		= CHAN_IRQ_NO;
    }
    else if(DMA_USE_INT == dma_wait_type)
    {
        dma_config.irq_spt 		= CHAN_IRQ_FD;
    }        
    else
    {
        dma_config.irq_spt 		= CHAN_IRQ_NO;
    }

	if(0 != sw_dma_config(dma_hdl, &dma_config)) 
    {
        printk("sw_dma_config err\n");
		goto err_out;
	}

    if(dma_type == (int)CHAN_DEDICATE)
    {
        /* set src/dst secu */
        tmp = SRC_SECU_DST_SECU;
        if(0 != sw_dma_ctl(dma_hdl, DMA_OP_SET_SECURITY, &tmp)) {
            goto err_out;
        }
        
		dma_para_t para;
		para.src_blk_sz 	= 0;
		para.src_wait_cyc 	= 0;
		para.dst_blk_sz 	= 0;
		para.dst_wait_cyc 	= 0;
		if(0 != sw_dma_ctl(dma_hdl, DMA_OP_SET_PARA_REG, &tmp)) {
			goto err_out;
		}
    }

    
	return dma_hdl;
err_out:
	/* stop and free dma channel, if need */
	if((dma_hdl_t)NULL != dma_hdl) 
    {
		if(0 != sw_dma_ctl(dma_hdl, DMA_OP_STOP, NULL))
        {
			printk("%s err, line %d!\n", __func__, __LINE__);
        }
		if(0 != sw_dma_release(dma_hdl))
        {
			printk("%s err, line %d!\n", __func__, __LINE__);
        }
	}
    printk("set dma config failed!\n");
    return NULL;
}

static int user_dma_do_start(int chan_id, unsigned int src, unsigned int dst, unsigned int len)
{
    dma_channel_t *pchan = NULL;
    dma_hdl_t dma_hdl = user_dma_list[chan_id].dma_hdl;
    pchan = (dma_channel_t *)dma_hdl;

    //spin_lock(&user_dma_list[chan_id].dma_lock);
	mutex_lock(&user_dma_list[chan_id].dma_mutex);
	if(0 != sw_dma_enqueue(dma_hdl, src, dst, len)) 
    {
        printk("sw_dma_enqueue err\n");
		goto err_out;
	}

    if(user_dma_list[chan_id].start == 0)
    {
        /* start dma */
        if(0 != sw_dma_ctl(dma_hdl, DMA_OP_START, NULL)) 
        {
            printk("sw_dma_start err\n");
            goto err_out;
        }
        user_dma_list[chan_id].start = 1;
    }

	/* wait dma done */
	user_dma_wait_done(pchan, chan_id);
    user_dma_dbg("user_dma_wait_done ==================\n");

    user_dma_dbg(" user_dma_start return ==================\n");
    //spin_unlock(&user_dma_list[chan_id].dma_lock);
	mutex_unlock(&user_dma_list[chan_id].dma_mutex);
	return 0;
err_out:
	/* stop and free dma channel, if need */
	if((dma_hdl_t)NULL != dma_hdl) 
    {
		if(0 != sw_dma_ctl(dma_hdl, DMA_OP_STOP, NULL))
        {
			printk("%s err, line %d!\n", __func__, __LINE__);
        }
		if(0 != sw_dma_release(dma_hdl))
        {
			printk("%s err, line %d!\n", __func__, __LINE__);
        }
	}
    printk("set dma config failed!\n");
    //spin_unlock(&user_dma_list[chan_id].dma_lock);
	mutex_unlock(&user_dma_list[chan_id].dma_mutex);
    return -1;
}

static int user_dma_stop(dma_hdl_t dma_hdl)
{
	/* stop and free dma channel, if need */
	if((dma_hdl_t)NULL != dma_hdl) 
    {
		if(0 != sw_dma_ctl(dma_hdl, DMA_OP_STOP, NULL))
        {
			printk("%s err, line %d!\n", __func__, __LINE__);
            return -1;
        }
		if(0 != sw_dma_release(dma_hdl))
        {
			printk("%s err, line %d!\n", __func__, __LINE__);
            return -1;
        }
	}
    return 0;
}

static int user_dma_start(struct user_dma_info_ext *dma_info)
{
    int ret = -1;
    unsigned int len0;
    unsigned int len1;
    unsigned int src;
    unsigned int dst;
    int chan_id = -1;

    chan_id = dma_info->vir_channel;
    ret = dma_prepare(dma_info);
    if(ret)
    {
        printk("dma_prepare faild\n");
        return -1;
    }

    if(dma_info->src_phy[1])
    {
        len0 = PAGE_SIZE - (dma_info->src_virt & ~PAGE_MASK);
        src = dma_info->src_phy[0];
        dst = dma_info->dst_phy[0];
        user_dma_dbg("src = 0x%x dst 0x%x len %d\n", src, dst, len0);

        ret = user_dma_do_start(chan_id, src, dst, len0);
        if(ret)
        {
            printk("user_dma_do_start faild\n");
        }

        len1 = dma_info->len - len0;
        src = dma_info->src_phy[1];
        dst = dma_info->dst_phy[0] + len0;
        user_dma_dbg("src = 0x%x dst 0x%x len %d\n", src, dst, len1);

        ret = user_dma_do_start(chan_id, src, dst, len1);
        if(ret)
        {
            printk("user_dma_do_start faild\n");
        }

    }
    else
    {
        len0 = dma_info->len;
        src = dma_info->src_phy[0];
        dst = dma_info->dst_phy[0];
        user_dma_dbg("src = 0x%x dst 0x%x len %d\n", src, dst, len0);

        ret = user_dma_do_start(chan_id, src, dst, len0);
        if(ret)
        {
            printk("user_dma_do_start faild\n");
        }

    }
    
    return 0;
}

static long user_dma_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
{
	int  ret = 0;
    struct user_dma_info dma_info;
    struct user_dma_info_ext dma_info_ext;
    int i = 0;
    dma_hdl_t hdl = NULL;
	switch (cmd) 
	{
        case USER_DMA_START:
        {
            if (copy_from_user(&dma_info, (void __user *)arg, sizeof(dma_info)))
            {
                printk("copy_from_user err\n");
                return -1;
            }
            dma_info_ext.src_virt = dma_info.src_virt;
            dma_info_ext.dst_virt = dma_info.dst_virt;
            dma_info_ext.len = dma_info.len;
            dma_info_ext.type = dma_info.type;
            dma_info_ext.vir_channel = dma_info.vir_channel;
            strncpy(dma_info_ext.name, dma_info.name, strlen(dma_info.name) + 1);
            ret = user_dma_start(&dma_info_ext);
            if(ret)
            {
                printk("user dma err %d\n", ret);
                return -1;
            }
            break;
        }
        case USER_DMA_REQUEST:
        {
            if (copy_from_user(&dma_info, (void __user *)arg, sizeof(dma_info)))
            {
                printk("copy_from_user err\n");
                return -1;
            }

            for(i = 0; i < MAX_DMA_CHAL; i++)
            {
                if(0 == strncmp(user_dma_list[i].name, dma_info.name, strlen(dma_info.name) + 1))
                {
                    hdl = user_dma_list[i].dma_hdl;
                    user_dma_list[i].dma_hdl = NULL;
                    strcpy(user_dma_list[i].name, "unuse");
                    user_dma_list[i].start = 0;
                    break;
                }
            }

            if(i < MAX_DMA_CHAL)
            {
                ret = user_dma_stop(hdl);
                if(ret)
                {
                    printk("user dma request failed %d\n", ret);
                    return -1;
                }
            }

            hdl = user_dma_reuqest(dma_info.name, dma_info.type);
            if(NULL == hdl)
            {
                printk("user dma request failed %d\n", ret);
                return -1;
            }

            for(i = 0; i < MAX_DMA_CHAL; i++)
            {
                if(user_dma_list[i].dma_hdl == NULL)
                {
                    strncpy(user_dma_list[i].name, dma_info.name, strlen(dma_info.name) + 1);
                    user_dma_list[i].dma_hdl = hdl;
                    user_dma_list[i].type = dma_info.type;
                    break;
                }
            }

            dma_info.vir_channel = i;
            if (copy_to_user((struct user_dma_info *)arg, &dma_info, sizeof(dma_info)))
            {
                printk("copy_to_user err\n");
                return -1;
            }
            break;

        }
        case USER_DMA_STOP:
        {
            if (copy_from_user(&dma_info, (void __user *)arg, sizeof(dma_info)))
            {
                printk("copy_from_user err\n");
                return -1;
            }

            for(i = 0; i < MAX_DMA_CHAL; i++)
            {
                if(0 == strncmp(user_dma_list[i].name, dma_info.name, strlen(dma_info.name) + 1))
                {
                    hdl = user_dma_list[i].dma_hdl;
                    user_dma_list[i].dma_hdl = NULL;
                    strcpy(user_dma_list[i].name, "unuse");
                    user_dma_list[i].start = 0;
                    break;
                }
            }

            ret = user_dma_stop(hdl);
            if(ret)
            {
                printk("user dma request failed %d\n", ret);
                return -1;
            }

            break;
        }

	}
	return ret;
}

static int user_dma_open(struct inode * inode , struct file *filp)
{
	return 0;
}

static struct file_operations user_dma_fops =
{
    .owner = THIS_MODULE,
	.open = user_dma_open,
	.unlocked_ioctl =  user_dma_ioctl,
};

int __init user_dma_init(void)
{
	int result;
	int err;
    int i = 0;

	result = 0;
	alloc_chrdev_region(&devid, 0, 1, "user_dma_dev");
	user_dma_cdev = cdev_alloc();
	cdev_init(user_dma_cdev, &user_dma_fops);
	user_dma_cdev->owner = THIS_MODULE;
	err = cdev_add(user_dma_cdev, devid, 1);
	if (err)
	{

        printk("cdev_addr err user_dma_dev\n");
		return -1;
	}

    user_dma_dbg("major number %d.\n", MAJOR(devid));
	user_dma_class = class_create(THIS_MODULE, "user_dma_class");
	if (IS_ERR(user_dma_class))
	{
		printk(KERN_ERR "create class error\n");
		return -1;
	}

	device_create(user_dma_class, NULL, devid, NULL, "user_dma");
    for(i = 0; i < MAX_DMA_CHAL; i++)
    {
        user_dma_list[i].dma_hdl = NULL;
        strcpy(user_dma_list[i].name, "unuse");
        user_dma_list[i].start = 0;
        init_waitqueue_head(&user_dma_list[i].dma_wq);
        atomic_set(&user_dma_list[i].dma_down, 0);
        //spin_lock_init(&user_dma_list[i].dma_lock);
        mutex_init(&user_dma_list[i].dma_mutex);
    }
	user_dma_dbg("user_dma init success\n");

	return result;

}


void user_dma_exit(void)
{
	device_destroy(user_dma_class,  devid);
	class_destroy(user_dma_class);
	cdev_del(user_dma_cdev);

	user_dma_dbg("user_dma exit success\n");
}


module_init(user_dma_init);
module_exit(user_dma_exit);

MODULE_AUTHOR("chenxi");
MODULE_DESCRIPTION("sun7i dma normal channel for user buf");
MODULE_LICENSE("GPL");
