/*
 * Copyright (c) 2012 Qualcomm Atheros, Inc..
 * All Rights Reserved.
 * Qualcomm Atheros Confidential and Proprietary.
 */

/*
 * Manage the atheros ethernet PHY.
 *
 * All definitions in this file are operating system independent!
 */

#include <qca/athrs17_phy.h>
#define MODULE_NAME "ATHRS17"

/* for PHY related debugging messages */
//#define S17_PHY_DEBUG
#ifdef S17_PHY_DEBUG
#define DPRINTF_PHY(_fmt,...) do {         \
                printf(MODULE_NAME": "_fmt, __VA_ARGS__);      \
} while (0)
#define DPRINTF_PHY2(_fmt,...)
#else
#define DPRINTF_PHY(_fmt,...)
#define DPRINTF_PHY2(_fmt,...)
#endif

#define S17_PHY_MAX 5
//#define CONFIG_1000M
static uint8_t athr17_init_flag = 0;
#if defined __KERNEL__
uint16_t phy_reg_read(struct mii_bus *bus, uint32_t phy_addr, uint8_t reg)
{
	return gmac_mdio_read(bus, phy_addr, reg);
}

int phy_reg_write(struct mii_bus *bus, uint32_t phy_addr, uint8_t reg, uint16_t data)
{
	return gmac_mdio_write(bus, phy_addr, reg, data);
}

#else
uint16_t phy_reg_read(int dev_id, uint32_t phy_addr, uint8_t reg)
{
	uint16_t data = 0;
	int ret;
	const char* devname = miiphy_get_current_dev();

	ret = miiphy_read(devname, (unsigned char)phy_addr, reg, &data);
	if (ret) {
		DPRINTF_PHY("miiphy_read returns error %d\n", ret);
		return 0;
	} else
		return data;
}

void phy_reg_write(int dev_id, uint32_t phy_addr, uint8_t reg, uint16_t data)
{
	int ret;
	const char* devname = miiphy_get_current_dev();

	ret = miiphy_write(devname, (unsigned char)phy_addr, reg, data);

	if (ret) {
		DPRINTF_PHY("miiphy_write returns error %d\n", ret);
	}
}
#endif

void athrs17_reg_init(struct mii_bus *bus)
{
    int phy_addr;

#ifdef CONFIG_ATHRS17_HNAT
    /* header len 4 with type 0xaaaa */
    athrs17_reg_write(bus, S17_HDRCTRL_REG, S17_HDRLENGTH_SEL | S17_HDR_VALUE);

    /*PORT0 TX_HEADER_MODE_0 all frame with header*/
    athrs17_reg_write(bus, S17_P0HDRCTRL_REG, S17_TXHDR_MODE_ALL);

    /*cpu port en*/
    athrs17_reg_write(bus, S17_GLOFW_CTRL0_REG, (athrs17_reg_read(bus, S17_GLOFW_CTRL0_REG)|S17_CPU_PORT_EN));

    /* enable L3 offload */
    athrs17_reg_write(bus, S17_MODULE_EN_REG, (athrs17_reg_read(bus, S17_MODULE_EN_REG) | 0x4));
#ifdef CONFIG_VLAN_8021Q
    /* set WAN port VLAN ID */
    if (S17_WAN_PORT == 5)
        athrs17_reg_write(bus, S17_ROUTER_VID2, (S17_WAN_VID << 16) | S17_LAN_VID);
    else if (S17_WAN_PORT == 1)
        athrs17_reg_write(bus, S17_ROUTER_VID0, (S17_WAN_VID << 16) | S17_LAN_VID);

    /* CPU port with VLAN tag, others w/o VLAN */
    athrs17_reg_write(bus, S17_ROUTER_EG_VLAN_MODE, \
        (S17_ROUTER_EG_MODE_DEFAULT & ~0x3) | S17_ROUTER_EG_WVLAN);

#endif
#endif
#ifdef CONFIG_ATHRS_QOS
        /* header len 2 byte */
        athrs17_reg_write(bus, S17_HDRCTRL_REG, 0x00);
        /*PORT0 RX & TX_HEADER_MODE_0 all frame with header */
        athrs17_reg_write(bus, S17_P0HDRCTRL_REG, (S17_TXHDR_MODE_ALL | S17_RXHDR_MODE_ALL));
#endif

       /* configure the RGMII */
       athrs17_reg_write(bus, S17_GLOFW_CTRL1_REG, \
                  S17_BROAD_DPALL | S17_MULTI_FLOOD_DPALL | S17_UNI_FLOOD_DPALL);
       athrs17_reg_write(bus, S17_PWS_REG, S17_PWS_CHIP_AR8327);
	   athrs17_reg_write(bus, S17_P0PAD_MODE_REG, S17_MAC0_RGMII_EN | S17_MAC0_RGMII_TXCLK_DELAY | \
                  S17_MAC0_RGMII_RXCLK_DELAY | (1 << S17_MAC0_RGMII_TXCLK_SHIFT) | \
                  (1 << S17_MAC0_RGMII_RXCLK_SHIFT));
	   
#if !HYBRID_SWITCH_PORT6_USED
       athrs17_reg_write(bus, S17_P6PAD_MODE_REG, S17_MAC6_RGMII_RXCLK_DELAY | \
                  (0 << S17_MAC6_RGMII_RXCLK_SHIFT));
#endif
       athrs17_reg_write(bus, S17_P0STATUS_REG, S17_PORT_STATUS_DEFAULT);

#if HYBRID_SWITCH_PORT6_USED
    /* configure AR8327 mac6 as RGMII mode */
       athrs17_reg_write(bus, S17_PWS_REG, S17_PWS_CHIP_AR8327);
       athrs17_reg_write(bus, S17_P6PAD_MODE_REG, S17_MAC6_RGMII_EN | S17_MAC6_RGMII_TXCLK_DELAY | \
                S17_MAC6_RGMII_RXCLK_DELAY |(2 << S17_MAC6_RGMII_RXCLK_SHIFT) | \
                (1 << S17_MAC6_RGMII_TXCLK_SHIFT));
       athrs17_reg_write(bus, S17_P6STATUS_REG, S17_PORT6_STATUS_DEFAULT);
#endif

    /* AR8327/AR8328 v1.0 fixup */
    if ((athrs17_reg_read(bus, S17_MASK_CTRL_REG) & 0xffff) == S17_CHIPID_V1_0)
    {
        for (phy_addr = 0x0; phy_addr <= S17_PHY_MAX; phy_addr++)
        {
          /* For 100M waveform */
          phy_reg_write(bus, phy_addr, S17_DEBUG_PORT_ADDRESS, 0x0);
          phy_reg_write(bus, phy_addr, S17_DEBUG_PORT_DATA, 0x02ea);
          /* Turn On Gigabit Clock */
          phy_reg_write(bus, phy_addr, S17_DEBUG_PORT_ADDRESS, 0x3d);
          phy_reg_write(bus, phy_addr, S17_DEBUG_PORT_DATA, 0x68a0);
        }
    }

    /* AR8337 v1.0 fixup */
    if ((athrs17_reg_read(bus, S17_MASK_CTRL_REG) & 0xffff) == S17C_CHIPID_V1_0)
    {
        for (phy_addr = 0x0; phy_addr <= S17_PHY_MAX; phy_addr++)
        {
            /* Turn On Gigabit Clock */
            phy_reg_write(bus, phy_addr, S17_DEBUG_PORT_ADDRESS, 0x3d);
            phy_reg_write(bus, phy_addr, S17_DEBUG_PORT_DATA, 0x6820);
        }
    }

    /* enable HOL by default */
    for (phy_addr = 0; phy_addr < S17_MAC_MAX; phy_addr++)
    {
       switch (phy_addr) {
       	case 0:
       	case 5:
       	case 6:
       		athrs17_reg_write(bus, S17_PORT0_HOL_CTRL0 + phy_addr * 8, \
            S17_HOL_CTRL0_WAN);
          break;

        default:
        	athrs17_reg_write(bus, S17_PORT0_HOL_CTRL0 + phy_addr * 8, \
            S17_HOL_CTRL0_LAN);
          break;
       }

        athrs17_reg_write(bus, S17_PORT0_HOL_CTRL1 + phy_addr * 8, \
            S17_HOL_CTRL1_DEFAULT);
    }


#if defined(S17_DUMP_MIB) && defined(CONFIG_S17_HNAT)
    /* register proc file system */
    athrs17_proc_init(&init_net);
#endif
    athrs17_reg_write(bus, 0x100, 0x1550);
    /* enable switch MIB */
    athrs17_reg_write(bus, 0x30, athrs17_reg_read(bus, 0x30)|0x1);
    athr17_init_flag = 1;

#if 0
	/* LED control */
    athrs17_reg_write(S17_LED_CTRL0_REG, 0xffb7ffb7);
    athrs17_reg_write(S17_LED_CTRL1_REG, 0xffb7ffb7);
    athrs17_reg_write(S17_LED_CTRL2_REG, 0xffb7ffb7);
#else
	// Turn all LEDs off.
	/* LED control */
    athrs17_reg_write(bus, S17_LED_CTRL0_REG, 0);
    athrs17_reg_write(bus, S17_LED_CTRL1_REG, 0);
    athrs17_reg_write(bus, S17_LED_CTRL2_REG, 0);
#endif

	athrs17_reg_write(bus, 0x0624, 0x007F7F7F);	// Global forward control register 1
	athrs17_reg_write(bus, S17_P0PAD_MODE_REG, 0x07600000);
#if defined CONFIG_1000M
	athrs17_reg_write(bus, 0x7C, 0x007E); // Port 0 status reg: full duplex, MAC, 1000MB/s
#else
	athrs17_reg_write(bus, 0x7C, 0x007d); // Port 0 status reg: full duplex, MAC, 100MB/s
#endif
}


unsigned int
athrs17_reg_read(struct mii_bus *bus, unsigned int reg_addr)
{
    uint32_t reg_word_addr;
    uint32_t phy_addr, tmp_val, reg_val;
    uint16_t phy_val;
    uint8_t phy_reg;

#if 1	// Original
    /* change reg_addr to 16-bit word address, 32-bit aligned */
    reg_word_addr = (reg_addr & 0xfffffffc) >> 1;

    /* configure register high address */
    phy_addr = 0x18;
    phy_reg = 0x0;
    phy_val = (uint16_t) ((reg_word_addr >> 8) & 0x1ff);  /* bit16-8 of reg address */
    phy_reg_write(bus, phy_addr, phy_reg, phy_val);

    /* For some registers such as MIBs, since it is read/clear, we should */
    /* read the lower 16-bit register then the higher one */

    /* read register in lower address */
    phy_addr = 0x10 | ((reg_word_addr >> 5) & 0x7); /* bit7-5 of reg address */
    phy_reg = (uint8_t) (reg_word_addr & 0x1f);   /* bit4-0 of reg address */
    reg_val = (uint32_t) phy_reg_read(bus, phy_addr, phy_reg);

    /* read register in higher address */
    reg_word_addr++;
    phy_addr = 0x10 | ((reg_word_addr >> 5) & 0x7); /* bit7-5 of reg address */
    phy_reg = (uint8_t) (reg_word_addr & 0x1f);   /* bit4-0 of reg address */
    tmp_val = (uint32_t) phy_reg_read(bus, phy_addr, phy_reg);
    reg_val |= (tmp_val << 16);

#else	// From DSPG;  This does not work!
	/* change reg_addr to 16-bit word address, 32-bit aligned */
	reg_word_addr = (reg_addr & 0xfffffffc) >> 1;
	
	/* configure register high address */
	phy_addr = 0x18;
	phy_reg = 0x0;
	phy_val = (uint16_t) ((reg_word_addr >> 8) & 0x1ff);  /* bit16-8 of reg address */
	phy_reg_write(0, phy_addr, phy_reg, phy_val);
	
	/* For some registers such as MIBs, since it is read/clear, we should */
	/* read the lower 16-bit register then the higher one */
	
	/* read register in lower address */
	phy_addr = 0x10 | ((reg_word_addr >> 5) & 0x7); /* bit7-5 of reg address */
	phy_reg = (uint8_t) (reg_word_addr & 0x1f);   /* bit4-0 of reg address */
	reg_val = (uint32_t) phy_reg_read(0, phy_addr, phy_reg);
	
	/* read register in higher address */
	reg_word_addr++;
	phy_addr = 0x10 | ((reg_word_addr >> 5) & 0x7); /* bit7-5 of reg address */
	phy_reg = (uint8_t) (reg_word_addr & 0x1f);   /* bit4-0 of reg address */
	tmp_val = (uint32_t) phy_reg_read(0, phy_addr, phy_reg);
	reg_val |= (tmp_val << 16);
#endif
	return reg_val;
}

void
athrs17_reg_write(struct mii_bus *bus, unsigned int reg_addr, unsigned int reg_val)
{
    uint32_t reg_word_addr;
    uint32_t phy_addr;
    uint16_t phy_val;
    uint8_t phy_reg;
#if 1	// Original
    /* change reg_addr to 16-bit word address, 32-bit aligned */
    reg_word_addr = (reg_addr & 0xfffffffc) >> 1;

    /* configure register high address */
    phy_addr = 0x18;
    phy_reg = 0x0;
    phy_val = (uint16_t) ((reg_word_addr >> 8) & 0x1ff);  /* bit16-8 of reg address */
    phy_reg_write(bus, phy_addr, phy_reg, phy_val);

    /* For S17 registers such as ARL and VLAN, since they include BUSY bit */
    /* in higher address, we should write the lower 16-bit register then the */
    /* higher one */

    /* write register in lower address */
    phy_addr = 0x10 | ((reg_word_addr >> 5) & 0x7); /* bit7-5 of reg address */
    phy_reg = (uint8_t) (reg_word_addr & 0x1f);   /* bit4-0 of reg address */
    phy_val = (uint16_t) (reg_val & 0xffff);
    phy_reg_write(bus, phy_addr, phy_reg, phy_val);

    /* write register in higher address */
    reg_word_addr++;
    phy_addr = 0x10 | ((reg_word_addr >> 5) & 0x7); /* bit7-5 of reg address */
    phy_reg = (uint8_t) (reg_word_addr & 0x1f);   /* bit4-0 of reg address */
    phy_val = (uint16_t) ((reg_val >> 16) & 0xffff);
    phy_reg_write(bus, phy_addr, phy_reg, phy_val);

#else	// From DSPG;  This does not work!
	/* change reg_addr to 16-bit word address, 32-bit aligned */
	reg_word_addr = (reg_addr & 0xfffffffc) >> 1;
	
	/* configure register high address */
	phy_addr = 0x18;
	phy_reg = 0x0;
	phy_val = (uint16_t) ((reg_word_addr >> 8) & 0x1ff);  /* bit16-8 of reg address */
	phy_reg_write(0, phy_addr, phy_reg, phy_val);
	
	/* For some registers such as ARL and VLAN, since they include BUSY bit */
	/* in lower address, we should write the higher 16-bit register then the */
	/* lower one */
	
	/* write register in higher address */
	reg_word_addr++;
	phy_addr = 0x10 | ((reg_word_addr >> 5) & 0x7); /* bit7-5 of reg address */
	phy_reg = (uint8_t) (reg_word_addr & 0x1f);   /* bit4-0 of reg address */
	phy_val = (uint16_t) ((reg_val >> 16) & 0xffff);
	phy_reg_write(0, phy_addr, phy_reg, phy_val);
	
	/* write register in lower address */
	reg_word_addr--;
	phy_addr = 0x10 | ((reg_word_addr >> 5) & 0x7); /* bit7-5 of reg address */
	phy_reg = (uint8_t) (reg_word_addr & 0x1f);   /* bit4-0 of reg address */
	phy_val = (uint16_t) (reg_val & 0xffff);
	phy_reg_write(0, phy_addr, phy_reg, phy_val);
#endif
}


int qca8334_mdio_read(struct mii_bus *bus, int phyaddr, int reg) 
{
    int i, value = 0; 

	if ((phyaddr != 1) && (phyaddr != 2))
		return 0xFFFF;

    for (i = 1; i <= 2; i++){
        value |= gmac_mdio_read(bus, i, reg);
    }    
    return value;
}


int qca8334_mdio_write(struct mii_bus *bus, int phyaddr, int reg, int value)
{
    int i, ret = 0; 

	if ((phyaddr != 1) && (phyaddr != 2))
		return 0xFFFF;

    for (i = 1; i <= 2; i++){
        ret |= gmac_mdio_write(bus, i, reg, value);
    }
	return ret;
}

