/*
 * Copyright (c) 2013 Grandstream Networks, Inc.
 * By Fanan Chen <fnchen@grandstream.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option)any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/slab.h>
#include <sound/soc.h>

struct rda_priv {
	struct snd_soc_codec codec;
};

static int rda_hw_params(struct snd_pcm_substream *substream,
			   struct snd_pcm_hw_params *params,
			   struct snd_soc_dai *dai)
{
	return 0;
}

static int rda_mute(struct snd_soc_dai *dai, int mute)
{
	return 0;
}

static int rda_set_dai_sysclk(struct snd_soc_dai *codec_dai,
				int clk_id, unsigned int freq, int dir)
{
	return 0;
}

static int rda_set_dai_fmt(struct snd_soc_dai *codec_dai,
			     unsigned int fmt)
{
	return 0;
}

static struct snd_soc_dai_ops rda_dai_ops = {
	.hw_params		= rda_hw_params,
	.digital_mute	= rda_mute,
	.set_sysclk		= rda_set_dai_sysclk,
	.set_fmt		= rda_set_dai_fmt,
};

static struct snd_soc_dai_driver rda5876_dai = {
	.name		= "rda5876-hifi",
	.playback	= {
		.stream_name	= "Playback",
		.channels_min	= 1,
		.channels_max	= 2,
		.rates			= SNDRV_PCM_RATE_8000,
		.formats		= SNDRV_PCM_FMTBIT_S16_LE,
	},
	.capture	= {
		.stream_name	= "Capture",
		.channels_min	= 1,
		.channels_max	= 2,
		.rates			= SNDRV_PCM_RATE_8000,
		.formats		= SNDRV_PCM_FMTBIT_S16_LE,
	},
	.ops		= &rda_dai_ops,
};

static int rda_set_bias_level(struct snd_soc_codec *codec,
		enum snd_soc_bias_level level)
{
	codec->dapm.bias_level = level;

	return 0;
}

static int rda_probe(struct snd_soc_codec *codec)
{
	return 0;
}

static int rda_remove(struct snd_soc_codec *codec)
{
	return 0;
}

static int rda_suspend(struct snd_soc_codec *codec)
{
	rda_set_bias_level(codec, SND_SOC_BIAS_OFF);

	return 0;
}

static int rda_resume(struct snd_soc_codec *codec)
{
	rda_set_bias_level(codec, SND_SOC_BIAS_STANDBY);

	return 0;
}

static struct snd_soc_codec_driver soc_codec_dev_rda5876 = {
	.set_bias_level	= rda_set_bias_level,
	.probe			= rda_probe,
	.remove			= rda_remove,
	.suspend		= rda_suspend,
	.resume			= rda_resume,

	.reg_cache_size	= 0,
	.reg_word_size	= 0,
	.reg_cache_default = NULL,
};

static int __devinit rda_dev_probe(struct platform_device *pdev)
{
	struct rda_priv *rda5876 = NULL;
	struct snd_soc_codec *codec = NULL;
	int ret;

	rda5876 = kzalloc(sizeof(struct rda_priv), GFP_KERNEL);
	if (rda5876 == NULL)
		return -ENOMEM;

	pdev->dev.platform_data = rda5876;
	codec = &rda5876->codec;
	codec->dev = &pdev->dev;

	ret = snd_soc_register_codec(codec->dev,
			&soc_codec_dev_rda5876, &rda5876_dai, 1);
	if (ret != 0) {
		dev_err(codec->dev, "Register rda5876 failed\n");
		return ret;
	}
	snd_soc_codec_set_drvdata(codec, rda5876);

	return 0;
}

static int __devexit rda_dev_remove(struct platform_device *pdev)
{
	struct rda_priv *rda5876 = pdev->dev.platform_data;
	struct snd_soc_codec *codec = &rda5876->codec;

	snd_soc_unregister_codec(codec->dev);
	kfree(rda5876);

	return 0;
}

static struct platform_driver rda_dev_driver = {
	.driver	= {
		.name	= "rda5876",
		.owner	= THIS_MODULE,
	},
	.probe	= rda_dev_probe,
	.remove	= __devexit_p(rda_dev_remove),
};

static int __init rda5876_init(void)
{
	return platform_driver_register(&rda_dev_driver);
}

static void __exit rda5876_exit(void)
{
	platform_driver_unregister(&rda_dev_driver);
}

module_init(rda5876_init);
module_exit(rda5876_exit);

MODULE_AUTHOR("Fanan Chen <fnchen@grandstream.com>");
MODULE_DESCRIPTION("ASoC rda5867 stub driver");
MODULE_LICENSE("GPL");
