/*
 * sound\soc\sun4i\i2s\sun4i_sndi2s.c
 * (C) Copyright 2007-2011
 * Reuuimlla Technology Co., Ltd. <www.reuuimllatech.com>
 * chenpailin <chenpailin@Reuuimllatech.com>
 *
 * some simple description for this code
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 */

#include <linux/module.h>
#include <linux/clk.h>
#include <linux/mutex.h>
#include <linux/proc_fs.h>
#include <linux/gpio.h>

#include <sound/pcm.h>
#include <sound/soc.h>
#include <sound/pcm_params.h>
#include <sound/soc-dapm.h>
#include <sound/tlv320aic3x.h>
#include <mach/gs_plat.h>
#include <linux/io.h>
#include <linux/delay.h>

#include "sun7i-i2s.h"
#include "sun7i-i2sdma.h"

#include "sndi2s.h"

#define SUN5I_AUDIO_CODEC_BASE	(0x01c22c00)
#define SUN5I_AUDIO_CODEC_ADC (0x28)

#define CODEC_CLOCK	12000000

#ifdef CONFIG_SND_SOC_CODEC_MASTER
#define AUDIO_FORMAT (SND_SOC_DAIFMT_I2S | \
		SND_SOC_DAIFMT_CBM_CFM |\
		SND_SOC_DAIFMT_NB_NF)
#else
#define AUDIO_FORMAT (SND_SOC_DAIFMT_I2S |\
		SND_SOC_DAIFMT_CBS_CFS |\
		SND_SOC_DAIFMT_NB_NF)
#endif

void __iomem *codec_base;
static struct clk *xtal, *codec_apbclk, *codec_pll2clk, *codec_moduleclk;
static int clk_users;
static DEFINE_MUTEX(clk_lock);

#ifdef ENFORCE_RATES
static struct snd_pcm_hw_constraint_list hw_constraints_rates = {
	.count	= ARRAY_SIZE(rates),
	.list	= rates,
	.mask	= 0,
};
#endif

static struct gs_board_snd_pdata g_brd_snd_gpios;
struct snd_soc_codec *g_codec;

static int sun4i_sndi2s_startup(struct snd_pcm_substream *substream)
{
	int ret = 0;
	#ifdef ENFORCE_RATES
		struct snd_pcm_runtime *runtime = substream->runtime;
	#endif

	if (!ret) {
	#ifdef ENFORCE_RATES
		ret = snd_pcm_hw_constraint_list(runtime, 0,
						 SNDRV_PCM_HW_PARAM_RATE,
						 &hw_constraints_rates);
		if (ret < 0)
			return ret;
	#endif
	}
	return ret;
}

static void sun4i_sndi2s_shutdown(struct snd_pcm_substream *substream)
{
	mutex_lock(&clk_lock);
	clk_users -= 1;
	if (clk_users == 0) {
		clk_put(xtal);
		xtal = NULL;

	}
	mutex_unlock(&clk_lock);
}

typedef struct __MCLK_SET_INF {
	__u32 samp_rate;      /* sample rate */
	__u16 mult_fs;        /* multiply of smaple rate */
	__u8 clk_div;        /* mpll division */
	__u8 mpll;           /* select mpll, 0 - 24.576 Mhz, 1 - 22.5792 Mhz */

} __mclk_set_inf;


typedef struct __BCLK_SET_INF {
	__u8 bitpersamp;     /* bits per sample */
	__u8 clk_div;        /* clock divisiona */
	__u16 mult_fs;        /* multiplay of sample rate */

} __bclk_set_inf;


static __bclk_set_inf BCLK_INF[] = {
    /* 16bits per sample */
	{16,  4, 128}, {16,  6, 192}, {16,  8, 256},
	{16, 12, 384}, {16, 16, 512},

    /* 24 bits per sample */
	{24,  4, 192}, {24,  8, 384}, {24, 16, 768},

    /* 32 bits per sample */
	{32,  2, 128}, {32,  4, 256}, {32,  6, 384},
	{32,  8, 512}, {32, 12, 768},

    /* end flag */
	{0xff, 0, 0},
};

static __mclk_set_inf  MCLK_INF[] = {
    /* 8k bitrate */
	{  8000, 128, 24, 0}, {  8000, 192, 16, 0}, {  8000, 256, 12, 0},
	{  8000, 384,  8, 0}, {  8000, 512,  6, 0}, {  8000, 768,  4, 0},

    /* 16k bitrate */
	{ 16000, 128, 12, 0}, { 16000, 192,  8, 0}, { 16000, 256,  6, 0},
	{ 16000, 384,  4, 0}, { 16000, 768,  2, 0},

    /* 32k bitrate */
	{ 32000, 128,  6, 0}, { 32000, 192,  4, 0}, { 32000, 384,  2, 0},
	{ 32000, 768,  1, 0},

    /* 64k bitrate */
	{ 64000, 192,  2, 0}, { 64000, 384,  1, 0},

    /* 128k bitrate */
	{128000, 192,  1, 0},

    /* 12k bitrate */
	{ 12000, 128, 16, 0}, { 12000, 256, 8, 0}, { 12000, 512, 4, 0},

    /* 24k bitrate */
	{ 24000, 128,  8, 0}, { 24000, 256, 4, 0}, { 24000, 512, 2, 0},

    /* 48K bitrate */
	{ 48000, 128,  4, 0}, { 48000, 256,  2, 0}, { 48000, 512, 1, 0},

    /* 96k bitrate */
	{ 96000, 128 , 2, 0}, { 96000, 256,  1, 0},

    /* 192k bitrate */
	{192000, 128,  1, 0},

    /* 11.025k bitrate */
	{ 11025, 128, 16, 1}, { 11205, 256,  8, 1}, { 11205, 512,  4, 1},

    /* 22.05k bitrate */
	{ 22050, 128,  8, 1}, { 22050, 256,  4, 1},
	{ 22050, 512,  2, 1},

    /* 44.1k bitrate */
	{ 44100, 128,  4, 1}, { 44100, 256,  2, 1}, { 44100, 512,  1, 1},

    /* 88.2k bitrate */
	{ 88200, 128,  2, 1}, { 88200, 256,  1, 1},

    /* 176.4k bitrate */
	{176400, 128, 1, 1},

    /* end flag 0xffffffff */
	{0xffffffff, 0, 0, 0},
};

static s32 get_clock_divder(u32 sample_rate, u32 sample_width,
		u32 *mclk_div, u32 *mpll, u32 *bclk_div, u32 *mult_fs)
{
	u32 i, j, ret = -EINVAL;

	for (i = 0; i < 100; i++) {
		if ((MCLK_INF[i].samp_rate == sample_rate) &&
				((MCLK_INF[i].clk_div == 2)
				 || (MCLK_INF[i].clk_div == 24))) {
			for (j = 0; j < ARRAY_SIZE(BCLK_INF); j++) {
				if ((BCLK_INF[j].bitpersamp == sample_width) &&
					(BCLK_INF[j].mult_fs == MCLK_INF[i].mult_fs)) {
					/* set mclk and bclk division */
					*mclk_div = MCLK_INF[i].clk_div;
					*mpll = MCLK_INF[i].mpll;
					*bclk_div = BCLK_INF[j].clk_div;
					*mult_fs = MCLK_INF[i].mult_fs;
					ret = 0;
					break;
				}
			}
		} else if (MCLK_INF[i].samp_rate == 0xffffffff)
			break;
	}

	return ret;
}

static int sun4i_sndi2s_hw_params(struct snd_pcm_substream *substream,
					struct snd_pcm_hw_params *params)
{
	struct snd_soc_pcm_runtime *rtd = substream->private_data;
	struct snd_soc_dai *codec_dai = rtd->codec_dai;
	struct snd_soc_dai *cpu_dai = rtd->cpu_dai;
	int ret = 0;
	unsigned long rate = params_rate(params);
	u32 mclk_div = 0, mpll = 0, bclk_div = 0, mult_fs = 0;

	sun7i_iis.ws_size = 32;
	get_clock_divder(rate, 32, &mclk_div, &mpll, &bclk_div, &mult_fs);
	if (rate == 8000) {
		sun7i_iis.pcm_txtype = 0;
		sun7i_iis.pcm_rxtype = 0;
		sun7i_iis.pcm_sync_type = 0; /* short sync */
		sun7i_iis.pcm_lsb_first = 0; /* MSB first */
		sun7i_iis.pcm_sync_period = 256;
		sun7i_iis.pcm_start_slot = 1;
		sun7i_iis.pcm_sw = 16;
		sun7i_iis.tdm_mode = 1;
		mclk_div = 2;
		bclk_div = 6;
		mult_fs = 1536;

		ret = snd_soc_dai_set_fmt(codec_dai, SND_SOC_DAIFMT_DSP_A
				| SND_SOC_DAIFMT_IB_NF | SND_SOC_DAIFMT_CBS_CFS);
		if (ret < 0)
			return ret;

		ret = snd_soc_dai_set_fmt(cpu_dai, SND_SOC_DAIFMT_DSP_A
				| SND_SOC_DAIFMT_IB_NF | SND_SOC_DAIFMT_CBS_CFS);
		if (ret < 0)
			return ret;

		/* set aic3104 tristate */
		ret = snd_soc_dai_set_tristate(codec_dai, 1);
		if (ret < 0)
			return ret;

		ret = snd_soc_dai_set_tdm_slot(codec_dai, 0, 0, 1, 16);
		if (ret < 0)
			return ret;
	} else {
		/* Disable TDM mode */
		sun7i_iis.tdm_mode = 0;

		ret = snd_soc_dai_set_fmt(codec_dai, AUDIO_FORMAT);
		if (ret < 0)
			return ret;

		ret = snd_soc_dai_set_fmt(cpu_dai, AUDIO_FORMAT);
		if (ret < 0)
			return ret;
	}
	ret = snd_soc_dai_set_sysclk(cpu_dai, 0,
			mpll ? 22579200 : 24576000, 0);
	if (ret < 0)
		return ret;

	ret = snd_soc_dai_set_sysclk(codec_dai, 0, rate * mult_fs,
			SND_SOC_CLOCK_OUT);
	if (ret < 0)
		return ret;

	ret = snd_soc_dai_set_clkdiv(cpu_dai, SUN7I_DIV_MCLK, mclk_div);
	if (ret < 0)
		return ret;

	ret = snd_soc_dai_set_clkdiv(cpu_dai, SUN7I_DIV_BCLK, bclk_div);
	if (ret < 0)
		return ret;

	return 0;
}

static int gxv32xx_brd_init_sound_gpios(struct gs_board_snd_pdata *pgpio)
{
	int ret;
	if (pgpio == NULL)
		return -ENXIO;

	/*default Handfree speak on*/
	ret = gpio_request(pgpio->hf_spk_sw, "gs-audio");
	if (ret != 0) {
		pr_err("Failed to request gpio\n");
		return  -EBUSY;
	}

	gpio_direction_output(pgpio->hf_spk_sw, 1);
	g_brd_snd_gpios.hf_spk_sw = pgpio->hf_spk_sw;

#ifdef CONFIG_GS_GXV3275
	/*handset SPK PA control*/
	ret = gpio_request(pgpio->hs_spk_sw, "gs-audio");
	if (ret != 0) {
		pr_err("Failed to request gpio\n");
		return -EBUSY;
	}
	gpio_direction_output(pgpio->hs_spk_sw, 0);
	g_brd_snd_gpios.hs_spk_sw = pgpio->hs_spk_sw;
	ret = gpio_request(pgpio->ehs_amp_sw, "gs-audio");
	if (ret != 0) {
		pr_err("Failed to request gpio\n");
		return -EBUSY;
	}
	gpio_direction_output(pgpio->ehs_amp_sw, 0);
	g_brd_snd_gpios.ehs_amp_sw = pgpio->ehs_amp_sw;
#else
	g_brd_snd_gpios.hs_spk_sw = -1;
#endif

	ret = gpio_request(pgpio->hs_hf_sw, "gs-audio");
	if (ret != 0) {
		pr_err("Failed to request gpio\n");
		return -EBUSY;
	}
	gpio_direction_output(pgpio->hs_hf_sw, 0);
	g_brd_snd_gpios.hs_hf_sw = pgpio->hs_hf_sw;

	return 0;
}

static const char *switch_function[] = {"Off", "On"};
static const struct soc_enum mic_enum =
	SOC_ENUM_SINGLE_EXT(2, switch_function);
static const char *hd_hs_switch[] = {"Handfree", "Handset"};
static const struct soc_enum hd_hs_enum =
	SOC_ENUM_SINGLE_EXT(2, hd_hs_switch);

#ifdef CONFIG_GS_GXV3275
static int gxp_get_handset_speak(struct snd_kcontrol *kcontrol,
		struct snd_ctl_elem_value *ucontrol)
{
	int hds_sel_st;
	const uint p_gpio = g_brd_snd_gpios.hs_spk_sw;

	if (p_gpio == -1) {
		pr_warn("Handset SPK selection is not supported\n");
		return 0;
	}
	hds_sel_st = !gpio_get_value(p_gpio);
	pr_info("%s %s\n", "Handset SPK selection is:",
			switch_function[hds_sel_st]);
	ucontrol->value.integer.value[0] = hds_sel_st;

	return 0;
}

static int gxp_set_handset_speak(struct snd_kcontrol *kcontrol,
		struct snd_ctl_elem_value *ucontrol)
{
	int hds_sel_st;
	const uint p_gpio = g_brd_snd_gpios.hs_spk_sw;

	if (p_gpio == -1) {
		pr_err("Handset SPK selection is not supported\n");
		return 0;
	}

	hds_sel_st = !gpio_get_value(p_gpio);
	if (hds_sel_st == ucontrol->value.integer.value[0]) {
		pr_warn("%s %s\n", "Handset SPK selection Unchanged:",
				switch_function[hds_sel_st]);
		return 0;
	}

	hds_sel_st = ucontrol->value.integer.value[0];
	pr_info("%s %s\n", "Setting Handset SPK selection to:",
			switch_function[hds_sel_st]);

	gpio_set_value(p_gpio, !hds_sel_st);

	return 1;
}

static int gxp_ehs_amp_event(struct snd_soc_dapm_widget *w,
		struct snd_kcontrol *k, int event)
{
	const uint p_gpio = g_brd_snd_gpios.ehs_amp_sw;

	if (p_gpio == -1) {
		pr_warn("EHS AMP is not supported\n");
		return 0;
	}

	if (SND_SOC_DAPM_EVENT_ON(event))
		gpio_set_value(p_gpio, 1);
	else
		gpio_set_value(p_gpio, 0);

	return 0;
}
#endif

static int gxp_get_hd_hf_switch(struct snd_kcontrol *kcontrol,
		struct snd_ctl_elem_value *ucontrol)
{
	int hd_hs_sel_st;
	const uint p_gpio = g_brd_snd_gpios.hs_hf_sw;

	if (p_gpio == -1) {
		pr_err("Handset Handfree selection is not supported\n");
		return 0;
	}
	hd_hs_sel_st = gpio_get_value(p_gpio);
	pr_info("%s %s\n", "Handset Handfree selection is:",
			hd_hs_switch[hd_hs_sel_st]);
	ucontrol->value.integer.value[0] = hd_hs_sel_st;

	return 0;
}

static int gxp_set_hd_hf_switch(struct snd_kcontrol *kcontrol,
		struct snd_ctl_elem_value *ucontrol)
{
	int hd_hs_sel_st;
	const uint p_gpio = g_brd_snd_gpios.hs_hf_sw;

	if (p_gpio == -1) {
		pr_err("Handset Handfree selection is not supported\n");
		return 0;
	}

	hd_hs_sel_st = gpio_get_value(p_gpio);
	if (hd_hs_sel_st == ucontrol->value.integer.value[0]) {
		pr_warn("%s %s\n", "Handset Handfree selection Unchanged:",
				hd_hs_switch[hd_hs_sel_st]);
		return 0;
	}

	hd_hs_sel_st = ucontrol->value.integer.value[0];
	pr_info("%s %s\n", "Setting Handset Handfree selection to:",
			hd_hs_switch[hd_hs_sel_st]);

	gpio_set_value(p_gpio, hd_hs_sel_st);
#ifdef CONFIG_GS_GXV3275
	if (hd_hs_sel_st) {
		gpio_set_value(g_brd_snd_gpios.hf_spk_sw, 0);
		gpio_set_value(g_brd_snd_gpios.hs_spk_sw, 1);
	} else {
		gpio_set_value(g_brd_snd_gpios.hf_spk_sw, 1);
		gpio_set_value(g_brd_snd_gpios.hs_spk_sw, 0);
	}
#else
	if (hd_hs_sel_st)
		gpio_set_value(g_brd_snd_gpios.hf_spk_sw, 0);
	else
		gpio_set_value(g_brd_snd_gpios.hf_spk_sw, 1);
#endif

	return 1;
}

static int gxp_get_mic1_pwr(struct snd_kcontrol *kcontrol,
		struct snd_ctl_elem_value *ucontrol)
{
	int value = 0;

	if (g_codec) {
		value = snd_soc_read(g_codec, 19);
		ucontrol->value.integer.value[0] = (value & 0x78) ? 1 : 0;
	}

	return 0;
}

static int gxp_set_mic1_pwr(struct snd_kcontrol *kcontrol,
		struct snd_ctl_elem_value *ucontrol)
{
	int value = 0;

	if (g_codec) {
		value = snd_soc_read(g_codec, 19);
		mdelay(1);
		if (ucontrol->value.integer.value[0])
			snd_soc_write(g_codec, 19, (value&(~0x78))|0x78);
		else
			snd_soc_write(g_codec, 19, value&(~0x78));
	}

	return 0;

}

static int gxp_get_mic2_pwr(struct snd_kcontrol *kcontrol,
		struct snd_ctl_elem_value *ucontrol)
{
	int value = 0;

	if (g_codec) {
		value = snd_soc_read(g_codec, 21);
		ucontrol->value.integer.value[0] = (value & 0x78) ? 1 : 0;
	}

	return 0;
}

static int gxp_set_mic2_pwr(struct snd_kcontrol *kcontrol,
		struct snd_ctl_elem_value *ucontrol)
{
	int value = 0;

	if (g_codec) {
		value = snd_soc_read(g_codec, 21);

		if (ucontrol->value.integer.value[0])
			snd_soc_write(g_codec, 21, (value&(~0x78))|0x78);
		else
			snd_soc_write(g_codec, 21, value&(~0x78));

	}

	return 0;
}


static int gxp_get_handset_pwr(struct snd_kcontrol *kcontrol,
		struct snd_ctl_elem_value *ucontrol)
{
	int value = 0;
	if (g_codec) {
		value = snd_soc_read(g_codec, 44);
		ucontrol->value.integer.value[0] = (value & 0x80) ? 1 : 0;
	}

	return 0;
}

static int gxp_set_handset_pwr(struct snd_kcontrol *kcontrol,
		struct snd_ctl_elem_value *ucontrol)
{
	int value = 0;

	if (g_codec) {
		value = snd_soc_read(g_codec, 44);

		if (ucontrol->value.integer.value[0])
			snd_soc_write(g_codec, 44, (value&(~0x80))|0x80);
		else
			snd_soc_write(g_codec, 44, value&(~0x80));
	}
	return 0;
}

static int gxp_get_headset_pwr(struct snd_kcontrol *kcontrol,
		struct snd_ctl_elem_value *ucontrol)
{
	int value = 0;

	if (g_codec) {
		value = snd_soc_read(g_codec, 43);
		ucontrol->value.integer.value[0] = (value & 0x80) ? 1 : 0;
	}
	return 0;
}

static int gxp_set_headset_pwr(struct snd_kcontrol *kcontrol,
		struct snd_ctl_elem_value *ucontrol)
{
	int value = 0;

	if (g_codec) {
		value = snd_soc_read(g_codec, 43);

		if (ucontrol->value.integer.value[0])
			snd_soc_write(g_codec, 43, (value&(~0x80))|0x80);
		else
			snd_soc_write(g_codec, 43, value&(~0x80));
	}

	return 0;

}

static int gxp_set_handset_sidetone_pwr(struct snd_kcontrol *kcontrol,
		struct snd_ctl_elem_value *ucontrol)
{
	int value1 = 0;
	int value2 = 0;

	if (g_codec) {
		value1 = snd_soc_read(g_codec, 60);
		value2 = snd_soc_read(g_codec, 67);
		if (ucontrol->value.integer.value[0]) {
			snd_soc_write(g_codec, 60, (value1&(~0x80))|0x80);
			snd_soc_write(g_codec, 67, (value2&(~0x80))|0x80);
		} else {
			snd_soc_write(g_codec, 60, value1&(~0x80));
			snd_soc_write(g_codec, 67, value2&(~0x80));
		}
	}

	return 0;
}

static int gxp_get_handset_sidetone_pwr(struct snd_kcontrol *kcontrol,
		struct snd_ctl_elem_value *ucontrol)
{
	int value = 0;
	if (g_codec) {
		value = snd_soc_read(g_codec, 60) | snd_soc_read(g_codec, 67);
		ucontrol->value.integer.value[0] = (value & 0x80) ? 1 : 0;
	}
	return 0;
}

static int gxp_set_headset_sidetone_pwr(struct snd_kcontrol *kcontrol,
		struct snd_ctl_elem_value *ucontrol)
{
	int value1 = 0;
	int value2 = 0;

	if (g_codec) {
		value1 = snd_soc_read(g_codec, 49);
		value2 = snd_soc_read(g_codec, 56);
		if (ucontrol->value.integer.value[0]) {
			snd_soc_write(g_codec, 49, (value1&(~0x80))|0x80);
			snd_soc_write(g_codec, 56, (value2&(~0x80))|0x80);
		} else {
			snd_soc_write(g_codec, 49, value1&(~0x80));
			snd_soc_write(g_codec, 56, value2&(~0x80));
		}
	}

	return 0;
}

static int gxp_get_headset_sidetone_pwr(struct snd_kcontrol *kcontrol,
		struct snd_ctl_elem_value *ucontrol)
{
	int value = 0;
	if (g_codec) {
		value = snd_soc_read(g_codec, 49) | snd_soc_read(g_codec, 56);
		ucontrol->value.integer.value[0] = (value & 0x80) ? 1 : 0;
	}

	return 0;
}

/* handfree sidetone for test only */
static int gxp_set_handfree_sidetone_pwr(struct snd_kcontrol *kcontrol,
		struct snd_ctl_elem_value *ucontrol)
{
	int value1 = 0;
	int value2 = 0;

	if (g_codec) {
		value1 = snd_soc_read(g_codec, 88);
		value2 = snd_soc_read(g_codec, 89);
		if (ucontrol->value.integer.value[0]) {
			snd_soc_write(g_codec, 88, (value1&(~0x80))|0x80);
			snd_soc_write(g_codec, 89, (value2&(~0x80))|0x80);
		} else {
			snd_soc_write(g_codec, 88, value1&(~0x80));
			snd_soc_write(g_codec, 89, value2&(~0x80));
		}
	}

	return 0;
}

static int gxp_get_handfree_sidetone_pwr(struct snd_kcontrol *kcontrol,
		struct snd_ctl_elem_value *ucontrol)
{
	int value = 0;
	if (g_codec) {
		value = snd_soc_read(g_codec, 88) | snd_soc_read(g_codec, 89);
		ucontrol->value.integer.value[0] = (value & 0x80) ? 1 : 0;
	}

	return 0;
}

static const struct snd_kcontrol_new gxp_controls[] = {
#ifdef CONFIG_GS_GXV3275
	SOC_ENUM_EXT("handset-spk-switch", mic_enum,
			gxp_get_handset_speak, gxp_set_handset_speak),
#endif
	SOC_ENUM_EXT("handset-handfree-switch", hd_hs_enum,
			gxp_get_hd_hf_switch, gxp_set_hd_hf_switch),
	SOC_ENUM_EXT("Mic1-mute", mic_enum, gxp_get_mic1_pwr,
			gxp_set_mic1_pwr),
	SOC_ENUM_EXT("Mic2-mute", mic_enum, gxp_get_mic2_pwr,
			gxp_set_mic2_pwr),

	SOC_ENUM_EXT("handset-spk-mute", mic_enum, gxp_get_handset_pwr,
			gxp_set_handset_pwr),
	SOC_ENUM_EXT("headset-spk-mute", mic_enum, gxp_get_headset_pwr,
			gxp_set_headset_pwr),

	SOC_ENUM_EXT("handset-sidetone-route-switch", mic_enum,
			gxp_get_handset_sidetone_pwr,
			gxp_set_handset_sidetone_pwr),
	SOC_ENUM_EXT("headset-sidetone-route-switch", mic_enum,
			gxp_get_headset_sidetone_pwr,
			gxp_set_headset_sidetone_pwr),
	SOC_ENUM_EXT("handfree-sidetone-route-switch", mic_enum,
			gxp_get_handfree_sidetone_pwr,
			gxp_set_handfree_sidetone_pwr),
};

static const struct snd_soc_dapm_widget tlv320aic3x_dapm_widgets[] = {
	SND_SOC_DAPM_REG(snd_soc_dapm_micbias, "HPRCOM EXTERNAL FEEDBACK",
			 38, 3, 7, 4, 0),
#ifdef CONFIG_GS_GXV3275
	SND_SOC_DAPM_HP("EHS", gxp_ehs_amp_event),
#endif
};

#ifdef CONFIG_GS_GXV3275
static const struct snd_soc_dapm_route audio_map[] = {
	{"EHS", NULL, "LLOUT"},
};
#endif

static int brd_gs_dai_link_init(struct snd_soc_pcm_runtime *rtd)
{
	int err;
	struct snd_soc_codec *codec = rtd->codec;
	struct snd_soc_dapm_context *dapm = &codec->dapm;

	err = snd_soc_add_codec_controls(codec, gxp_controls,
				ARRAY_SIZE(gxp_controls));
	if (err < 0)
		return err;

	err = snd_soc_dapm_new_controls(dapm, tlv320aic3x_dapm_widgets,
					ARRAY_SIZE(tlv320aic3x_dapm_widgets));
	if (err < 0)
		return err;

#ifdef CONFIG_GS_GXV3275
	err = snd_soc_dapm_add_routes(dapm, audio_map,
			ARRAY_SIZE(audio_map));
	if (err < 0)
		return err;
#endif

	g_codec = codec;

	snd_soc_dapm_force_enable_pin(dapm, "Mic Bias 2.5V");
	snd_soc_dapm_force_enable_pin(dapm, "Left HP Out");
#ifdef CONFIG_GS_GXV3275
	snd_soc_dapm_force_enable_pin(dapm, "HPRCOM EXTERNAL FEEDBACK");
	/* enable headset detection */
	snd_soc_write(codec, 13, 0x80);
#endif

	return 0;
}

static struct snd_soc_ops sun4i_sndi2s_ops = {
	.startup		= sun4i_sndi2s_startup,
	.shutdown		= sun4i_sndi2s_shutdown,
	.hw_params		= sun4i_sndi2s_hw_params,
};

#ifdef CONFIG_SND_SOC_BT_CODEC
static int rda_dai_link_init(struct snd_soc_pcm_runtime *rtd)
{
	int ret = 0;
	struct snd_soc_codec *codec = rtd->codec;
	struct snd_soc_dapm_context *dapm = &codec->dapm;

	ret = snd_soc_dapm_sync(dapm);
	if (ret) {
		pr_err("BT dapm sync failed\n");
		return ret;
	}

	return 0;
}

static int rda_hw_params(struct snd_pcm_substream *substream,
		struct snd_pcm_hw_params *params)
{
	struct snd_soc_pcm_runtime *rtd = substream->private_data;
	struct snd_soc_dai *cpu_dai = rtd->cpu_dai;
	int ret = 0;
	u32 mclk_div = 0, bclk_div = 0;

	/* PCM mode Linear 16bits */
	/* use 2.048MHz BCLK */
	/* 8KHZ sample rate */
	sun7i_iis.ws_size = 16;
	sun7i_iis.pcm_txtype = 0;
	sun7i_iis.pcm_rxtype = 0;
	sun7i_iis.pcm_sync_type = 0; /* short sync */
	sun7i_iis.pcm_lsb_first = 0; /* MSB first */
	sun7i_iis.pcm_sync_period = 256;
	sun7i_iis.pcm_start_slot = 1;
	sun7i_iis.pcm_sw = 16;
	mclk_div = 6;
	bclk_div = 2;

	ret = snd_soc_dai_set_fmt(cpu_dai, SND_SOC_DAIFMT_DSP_A |
			SND_SOC_DAIFMT_IB_NF | SND_SOC_DAIFMT_CBS_CFS);
	if (ret < 0) {
		pr_err("rda set cpu_dai format failed\n");
		return ret;
	}

	ret = snd_soc_dai_set_sysclk(cpu_dai, 0, 24576000, 0);
	if (ret < 0) {
		pr_err("rda set cpu_dai sysclk failed\n");
		return ret;
	}

	ret = snd_soc_dai_set_clkdiv(cpu_dai, SUN7I_DIV_MCLK, mclk_div);
	if (ret < 0) {
		pr_err("rda set cpu_dai mclkdiv failed\n");
		return ret;
	}

	ret = snd_soc_dai_set_clkdiv(cpu_dai, SUN7I_DIV_BCLK, bclk_div);
	if (ret < 0) {
		pr_err("rda set cpu_dai bclkdiv failed\n");
		return ret;
	}

	return 0;
}

static int rda_startup(struct snd_pcm_substream *substream)
{
	pr_info("BT-startup\n");

	/* set aic3104 tristate */

	snd_soc_update_bits(g_codec, 8, 0x20, 0x20);
	snd_soc_update_bits(g_codec, 19, 0x4, 0x0);
	snd_soc_update_bits(g_codec, 22, 0x4, 0x0);

	return 0;
}

static void rda_shutdown(struct snd_pcm_substream *substream)
{
	u32 value;

	pr_info("BT-shutdown\n");

	/* set aic3104 tristate */
	value = snd_soc_read(g_codec, 8);
	value &= ~(1 << 5);
	snd_soc_write(g_codec, 8, value);

	return;
}

static int rda_prepare(struct snd_pcm_substream *substream)
{
	pr_info("BT-prepare\n");
	return 0;
}

static struct snd_soc_ops rda_sndi2s_ops = {
	.hw_params	= rda_hw_params,
	.startup	= rda_startup,
	.shutdown	= rda_shutdown,
	.prepare	= rda_prepare,
};
#endif

static struct snd_soc_dai_link sun7i_sndi2s_dai_link[] = {
	{
		.name			= "TLV320AIC3X",
		.stream_name	= "AIC3X",
		.cpu_dai_name	= "sun7i-i2s.0",
		.codec_dai_name = "tlv320aic3x-hifi",
		.platform_name	= "sun7i-i2s-pcm-audio.0",
		.codec_name	= "tlv320aic3x-codec.2-0018",
		.init			= brd_gs_dai_link_init,
		.ops			= &sun4i_sndi2s_ops,
	},
#ifdef CONFIG_SND_SOC_BT_CODEC
	{
		.name			= "BT-CODEC",
		.stream_name	= "BT-RDA5876",
		.cpu_dai_name	= "sun7i-i2s.0",
		.codec_dai_name	= "rda5876-hifi",
		.platform_name	= "sun7i-i2s-pcm-audio.0",
		.codec_name		= "rda5876",
		.init			= rda_dai_link_init,
		.ops			= &rda_sndi2s_ops,
	},
#endif
};

static struct snd_soc_card brd_gxv32xx_card = {
	.name = "sndi2s",
	.dai_link = sun7i_sndi2s_dai_link,
	.num_links = ARRAY_SIZE(sun7i_sndi2s_dai_link),
};

static int gxv32xx_snd_probe(struct platform_device *pdev)
{
	int ret = 0;
	int reg_val = 0;
	struct resource *mres = NULL;
	struct snd_soc_card *card = &brd_gxv32xx_card;


	struct gs_board_snd_pdata *pdata;

	pdata = (struct gs_board_snd_pdata *)pdev->dev.platform_data;

	gxv32xx_brd_init_sound_gpios(pdata);

	/*add MCLK TBD*/

	card->dev = &pdev->dev;
	platform_set_drvdata(pdev, card);
	ret = snd_soc_register_card(card);
	if (ret) {
		pr_err("%s %i\n", "snd_soc_register_card Failed", ret);
		goto ERR_CLEAR;
	}

    /* enable VMIC */
	codec_apbclk = clk_get(NULL, "apb_audio_codec");
	if (-1 == clk_enable(codec_apbclk))
		pr_err("codec_apbclk failed.\n");

	/* codec_pll2clk */
	codec_pll2clk = clk_get(NULL, "audio_pll");

	/* codec_moduleclk */
	codec_moduleclk = clk_get(NULL, "audio_codec");

	if (clk_set_parent(codec_moduleclk, codec_pll2clk))
		pr_err("try to set parent of codec_moduleclk to codec_pll2clk failed!\n");

	if (clk_set_rate(codec_moduleclk, 24576000))
		pr_err("set codec_moduleclk clock freq 24576000 failed!\n");

	if (-1 == clk_enable(codec_moduleclk))
		pr_err("open codec_moduleclk failed.\n");

	mres = request_mem_region(SUN5I_AUDIO_CODEC_BASE, 0x40, "sun4i-codec");
	if (mres == NULL) {
		pr_err("request_mem_region failed\n");
		ret = -EIO;
		goto ERR_CLEAR;
	}
	codec_base = ioremap_nocache(SUN5I_AUDIO_CODEC_BASE, 0x40);
	if (codec_base == NULL) {
		pr_err("ioremap audio codec failed\n");
		ret = -ENOMEM;
		goto ERR_CLEAR;
	}
	reg_val = readl(codec_base + SUN5I_AUDIO_CODEC_ADC);
	reg_val |= 0x8000000;
	writel(reg_val, (codec_base + SUN5I_AUDIO_CODEC_ADC));

	return 0;
ERR_CLEAR:

	snd_soc_card_set_drvdata(card, NULL);
	platform_set_drvdata(pdev, NULL);
	card->dev = NULL;
	return ret;
}

static int gxv32xx_snd_remove(struct platform_device *pdev)
{
	struct snd_soc_card *card = platform_get_drvdata(pdev);

	snd_soc_unregister_card(card);

	/* Cleanup: */
	snd_soc_card_set_drvdata(card, NULL);
	platform_set_drvdata(pdev, NULL);
	card->dev = NULL;
	return 0;
}

static struct platform_driver g_brd_gxv32xx_snd_drv = {
	.driver = {
		.name = "snd-gxv3275-mach-drv",
		.owner = THIS_MODULE,
	},
	.probe = gxv32xx_snd_probe,
	.remove = __devexit_p(gxv32xx_snd_remove),
};

module_platform_driver(g_brd_gxv32xx_snd_drv);

MODULE_AUTHOR("ALL WINNER");
MODULE_DESCRIPTION("SUN4I_sndi2s ALSA SoC audio driver");
MODULE_LICENSE("GPL");
