/*
 * sound\soc\sun7i\i2s\sun7i-i2s.c
 * (C) Copyright 2007-2011
 * Reuuimlla Technology Co., Ltd. <www.reuuimllatech.com>
 * chenpailin <chenpailin@Reuuimllatech.com>
 *
 * some simple description for this code
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/device.h>
#include <linux/delay.h>
#include <linux/clk.h>
#include <linux/jiffies.h>
#include <linux/io.h>
#include <sound/core.h>
#include <sound/pcm.h>
#include <sound/pcm_params.h>
#include <sound/initval.h>
#include <sound/soc.h>
#include <mach/clock.h>
#include <mach/sys_config.h>
#include <mach/hardware.h>
#include <asm/dma.h>
#include <mach/dma.h>
#include <mach/gpio.h>
#include <linux/gpio.h>
#include "sun7i-i2sdma.h"
#include "sun7i-i2s.h"

#define SUN7I_PCM_TX_CHANNEL_MAX 8
#define SUN7I_PCM_RX_CHANNEL_MAX 4

struct sun7i_i2s_info sun7i_iis;

struct sun7i_i2s {
	struct platform_device *pdev;
	void __iomem *addr;
	u32 base;
	struct clk *i2s_apbclk;
	struct clk *i2s_pll2clk;
	struct clk *i2s_pllx8;
	struct clk *i2s_moduleclk;
	u32 regsave[8];
};

static struct sun7i_dma_params sun7i_i2s_pcm_stereo_out = {
	.name        = "I2S PCM Stereo out",
	.dma_addr    =	SUN7I_IISBASE + SUN7I_IISTXFIFO,
};

static struct sun7i_dma_params sun7i_i2s_pcm_stereo_in = {
	.name       = "I2S PCM Stereo in",
	.dma_addr	=	SUN7I_IISBASE + SUN7I_IISRXFIFO,
};


void sun7i_snd_txctrl_i2s(struct snd_pcm_substream *substream, int on)
{
	u32 reg_val;
	struct snd_soc_pcm_runtime *rtd = substream->private_data;
	struct sun7i_i2s *i2s = snd_soc_dai_get_drvdata(rtd->cpu_dai);
	void __iomem *regs = i2s->addr;

	reg_val = readl(regs + SUN7I_TXCHSEL);
	reg_val &= ~0x7;
	if (sun7i_iis.tdm_mode)
		reg_val |= 0x3;
	else
		reg_val |= SUN7I_TXCHSEL_CHNUM(substream->runtime->channels);
	writel(reg_val, regs + SUN7I_TXCHSEL);

	reg_val = 0;
	if (substream->runtime->channels == 1)
		reg_val = 0x76543200;
	else if (sun7i_iis.tdm_mode)
		reg_val = 0x76543110;
	else
		reg_val = 0x76543210;
	writel(reg_val, regs + SUN7I_TXCHMAP);

	reg_val = readl(regs + SUN7I_IISCTL);
	reg_val &= ~SUN7I_IISCTL_SDO3EN;
	reg_val &= ~SUN7I_IISCTL_SDO2EN;
	reg_val &= ~SUN7I_IISCTL_SDO1EN;
	reg_val &= ~SUN7I_IISCTL_SDO0EN;
	switch (substream->runtime->channels) {
	case 1:
	case 2:
		reg_val |= SUN7I_IISCTL_SDO0EN;
		break;
	case 3:
	case 4:
		reg_val |= SUN7I_IISCTL_SDO0EN | SUN7I_IISCTL_SDO1EN;
		break;
	case 5:
	case 6:
		reg_val |= SUN7I_IISCTL_SDO0EN | SUN7I_IISCTL_SDO1EN | SUN7I_IISCTL_SDO2EN;
		break;
	case 7:
	case 8:
		reg_val |= SUN7I_IISCTL_SDO0EN | SUN7I_IISCTL_SDO1EN | SUN7I_IISCTL_SDO2EN | SUN7I_IISCTL_SDO3EN;
		break;
	default:
		reg_val |= SUN7I_IISCTL_SDO0EN;
	}
	writel(reg_val, regs + SUN7I_IISCTL);

	/* flush TX FIFO */
	reg_val = readl(regs + SUN7I_IISFCTL);
	reg_val |= SUN7I_IISFCTL_FTX;
	writel(reg_val, regs + SUN7I_IISFCTL);

	/* clear TX counter */
	writel(0, regs + SUN7I_IISTXCNT);

	if (on) {
		/* IIS TX ENABLE */
		reg_val = readl(regs + SUN7I_IISCTL);
		reg_val |= SUN7I_IISCTL_TXEN;
		writel(reg_val, regs + SUN7I_IISCTL);

		/* enable DMA DRQ mode for play */
		reg_val = readl(regs + SUN7I_IISINT);
		reg_val |= SUN7I_IISINT_TXDRQEN;
		writel(reg_val, regs + SUN7I_IISINT);
	} else {
		/* IIS TX DISABLE */
		reg_val = readl(regs + SUN7I_IISCTL);
		reg_val &= ~SUN7I_IISCTL_TXEN;
		writel(reg_val, regs + SUN7I_IISCTL);

		/* DISBALE dma DRQ mode */
		reg_val = readl(regs + SUN7I_IISINT);
		reg_val &= ~SUN7I_IISINT_TXDRQEN;
		writel(reg_val, regs + SUN7I_IISINT);
	}
}

void sun7i_snd_rxctrl_i2s(struct snd_pcm_substream *substream, int on)
{
	u32 reg_val;
	struct snd_soc_pcm_runtime *rtd = substream->private_data;
	struct sun7i_i2s *i2s = snd_soc_dai_get_drvdata(rtd->cpu_dai);
	void __iomem *regs = i2s->addr;

	reg_val = readl(regs + SUN7I_RXCHSEL);
	reg_val &= ~0x3;
	if (sun7i_iis.tdm_mode)
		reg_val |= 0x3;
	else
		reg_val |= SUN7I_RXCHSEL_CHNUM(substream->runtime->channels);
	writel(reg_val, regs + SUN7I_RXCHSEL);

	reg_val = 0;
	if (substream->runtime->channels == 1)
		reg_val = 0x00000000;
	else if (sun7i_iis.tdm_mode)
		reg_val = 0x00003210;
	else
		reg_val = 0x00003210;
	writel(reg_val, regs + SUN7I_RXCHMAP);

	/* flush RX FIFO */
	reg_val = readl(regs + SUN7I_IISFCTL);
	reg_val |= SUN7I_IISFCTL_FRX;
	writel(reg_val, regs + SUN7I_IISFCTL);

	/* clear RX counter */
	writel(0, regs + SUN7I_IISRXCNT);

	if (on) {
		/* IIS RX ENABLE */
		reg_val = readl(regs + SUN7I_IISCTL);
		reg_val |= SUN7I_IISCTL_RXEN;
		writel(reg_val, regs + SUN7I_IISCTL);

		/* enable DMA DRQ mode for record */
		reg_val = readl(regs + SUN7I_IISINT);
		reg_val |= SUN7I_IISINT_RXDRQEN;
		writel(reg_val, regs + SUN7I_IISINT);
	} else {
		/* IIS RX DISABLE */
		reg_val = readl(regs + SUN7I_IISCTL);
		reg_val &= ~SUN7I_IISCTL_RXEN;
		writel(reg_val, regs + SUN7I_IISCTL);

		/* DISBALE dma DRQ mode */
		reg_val = readl(regs + SUN7I_IISINT);
		reg_val &= ~SUN7I_IISINT_RXDRQEN;
		writel(reg_val, regs + SUN7I_IISINT);
	}
}

static int sun7i_i2s_set_fmt(struct snd_soc_dai *cpu_dai, unsigned int fmt)
{
	u32 reg_val;
	u32 reg_val1;
	struct sun7i_i2s *i2s = snd_soc_dai_get_drvdata(cpu_dai);
	void __iomem *regs = i2s->addr;

	reg_val = readl(regs + SUN7I_IISCTL);
	reg_val |= (SUN7I_IISCTL_SDO0EN);
	writel(reg_val, regs + SUN7I_IISCTL);

	/* master or slave selection */
	reg_val = readl(regs + SUN7I_IISCTL);
	switch (fmt & SND_SOC_DAIFMT_MASTER_MASK) {
	case SND_SOC_DAIFMT_CBM_CFM:   /* codec clk & frm master */
		reg_val |= SUN7I_IISCTL_MS;
		break;
	case SND_SOC_DAIFMT_CBS_CFS:   /* codec clk & frm slave */
		reg_val &= ~SUN7I_IISCTL_MS;
		break;
	default:
		return -EINVAL;
	}
	writel(reg_val, regs + SUN7I_IISCTL);

	/* pcm or i2s mode selection */
	reg_val = readl(regs + SUN7I_IISCTL);
	reg_val1 = readl(regs + SUN7I_IISFAT0);
	reg_val1 &= ~SUN7I_IISFAT0_FMT_RVD;
	switch (fmt & SND_SOC_DAIFMT_FORMAT_MASK) {
	case SND_SOC_DAIFMT_I2S:        /* I2S mode */
		reg_val &= ~SUN7I_IISCTL_PCM;
		reg_val1 |= SUN7I_IISFAT0_FMT_I2S;
		break;
	case SND_SOC_DAIFMT_RIGHT_J:    /* Right Justified mode */
		reg_val &= ~SUN7I_IISCTL_PCM;
		reg_val1 |= SUN7I_IISFAT0_FMT_RGT;
		break;
	case SND_SOC_DAIFMT_LEFT_J:     /* Left Justified mode */
		reg_val &= ~SUN7I_IISCTL_PCM;
		reg_val1 |= SUN7I_IISFAT0_FMT_LFT;
		break;
	case SND_SOC_DAIFMT_DSP_A:      /* L data msb after FRM LRC */
		reg_val |= SUN7I_IISCTL_PCM;
		reg_val1 &= ~SUN7I_IISFAT0_LRCP;
		break;
	case SND_SOC_DAIFMT_DSP_B:      /* L data msb during FRM LRC */
		reg_val |= SUN7I_IISCTL_PCM;
		reg_val1 |= SUN7I_IISFAT0_LRCP;
		break;
	default:
		return -EINVAL;
	}
	writel(reg_val, regs + SUN7I_IISCTL);
	writel(reg_val1, regs + SUN7I_IISFAT0);

	/* DAI signal inversions */
	reg_val1 = readl(regs + SUN7I_IISFAT0);
	switch (fmt & SND_SOC_DAIFMT_INV_MASK) {
	case SND_SOC_DAIFMT_NB_NF:     /* normal bit clock + frame */
		reg_val1 &= ~SUN7I_IISFAT0_LRCP;
		reg_val1 &= ~SUN7I_IISFAT0_BCP;
		break;
	case SND_SOC_DAIFMT_NB_IF:     /* normal bclk + inv frm */
		reg_val1 |= SUN7I_IISFAT0_LRCP;
		reg_val1 &= ~SUN7I_IISFAT0_BCP;
		break;
	case SND_SOC_DAIFMT_IB_NF:     /* invert bclk + nor frm */
		reg_val1 &= ~SUN7I_IISFAT0_LRCP;
		reg_val1 |= SUN7I_IISFAT0_BCP;
		break;
	case SND_SOC_DAIFMT_IB_IF:     /* invert bclk + frm */
		reg_val1 |= SUN7I_IISFAT0_LRCP;
		reg_val1 |= SUN7I_IISFAT0_BCP;
		break;
	}
	writel(reg_val1, regs + SUN7I_IISFAT0);
	/* set FIFO control register */
	reg_val = readl(regs + SUN7I_IISFAT0);
	reg_val &= ~SUN7I_IISFAT0_WSS_32BCLK;
	if (sun7i_iis.ws_size == 16)
		reg_val |= SUN7I_IISFAT0_WSS_16BCLK;
	else if (sun7i_iis.ws_size == 20)
		reg_val |= SUN7I_IISFAT0_WSS_20BCLK;
	else if (sun7i_iis.ws_size == 24)
		reg_val |= SUN7I_IISFAT0_WSS_24BCLK;
	else
		reg_val |= SUN7I_IISFAT0_WSS_32BCLK;
	writel(reg_val, regs + SUN7I_IISFAT0);

	/* PCM REGISTER setup */
	reg_val = sun7i_iis.pcm_txtype & 0x3;
	reg_val |= sun7i_iis.pcm_rxtype << 2;

	if (!sun7i_iis.pcm_sync_type)
		reg_val |= SUN7I_IISFAT1_SSYNC;	/* short sync */
	if (sun7i_iis.pcm_sw == 16)
		reg_val |= SUN7I_IISFAT1_SW;

	reg_val |= ((sun7i_iis.pcm_start_slot - 1) & 0x3) << 6; /* start slot index */
	reg_val |= sun7i_iis.pcm_lsb_first << 9;	/* MSB or LSB first */

	if (sun7i_iis.pcm_sync_period == 256)
		reg_val |= 0x4 << 12;
	else if (sun7i_iis.pcm_sync_period == 128)
		reg_val |= 0x3 << 12;
	else if (sun7i_iis.pcm_sync_period == 64)
		reg_val |= 0x2 << 12;
	else if (sun7i_iis.pcm_sync_period == 32)
		reg_val |= 0x1 << 12;
	writel(reg_val, regs + SUN7I_IISFAT1);
	reg_val = 1 & 0x3;
	reg_val |= (1 & 0x1) << 2;
	reg_val |= SUN7I_IISFCTL_RXTL(0xf);	/* RX FIFO trigger level */
	reg_val |= SUN7I_IISFCTL_TXTL(0x40); /* TX FIFO empty trigger level */
	writel(reg_val, regs + SUN7I_IISFCTL);
	return 0;
}

static int sun7i_i2s_hw_params(struct snd_pcm_substream *substream,
		struct snd_pcm_hw_params *params, struct snd_soc_dai *dai)
{
	u32 reg_val;
	int sample_size;
	struct sun7i_i2s *i2s = snd_soc_dai_get_drvdata(dai);
	void __iomem *regs = i2s->addr;

	reg_val = readl(regs + SUN7I_IISFAT0);
	reg_val &= ~SUN7I_IISFAT0_SR_RVD;

	switch (params_format(params)) {
	case SNDRV_PCM_FORMAT_S16_LE:
		reg_val |= SUN7I_IISFAT0_SR_16BIT;
		sample_size = 16;
		break;
	case SNDRV_PCM_FORMAT_S24_LE:
		reg_val |= SUN7I_IISFAT0_SR_24BIT;
		sample_size = 24;
		break;
	default:
		return -EINVAL;
	}

	writel(reg_val, regs + SUN7I_IISFAT0);

	return 0;
}

static int sun7i_i2s_trigger(struct snd_pcm_substream *substream,
		int cmd, struct snd_soc_dai *dai)
{
	int ret = 0;

	switch (cmd) {
	case SNDRV_PCM_TRIGGER_START:
	case SNDRV_PCM_TRIGGER_RESUME:
	case SNDRV_PCM_TRIGGER_PAUSE_RELEASE:
		if (substream->stream == SNDRV_PCM_STREAM_CAPTURE)
			sun7i_snd_rxctrl_i2s(substream, 1);
		else
			sun7i_snd_txctrl_i2s(substream, 1);

		break;
	case SNDRV_PCM_TRIGGER_STOP:
	case SNDRV_PCM_TRIGGER_SUSPEND:
	case SNDRV_PCM_TRIGGER_PAUSE_PUSH:
		if (substream->stream == SNDRV_PCM_STREAM_CAPTURE)
			sun7i_snd_rxctrl_i2s(substream, 0);
		else
			sun7i_snd_txctrl_i2s(substream, 0);

		break;
	default:
		ret = -EINVAL;
		break;
	}

	return ret;
}

/* freq: 1: 22.5792MHz   0: 24.576MHz */
static int sun7i_i2s_set_sysclk(struct snd_soc_dai *cpu_dai, int clk_id,
		unsigned int freq, int i2s_pcm_select)
{
	struct sun7i_i2s *i2s = snd_soc_dai_get_drvdata(cpu_dai);

	if (clk_set_rate(i2s->i2s_pll2clk, freq))
		pr_err("try to set the i2s_pll2clk failed!\n");

	return 0;
}

static int sun7i_i2s_set_clkdiv(struct snd_soc_dai *cpu_dai,
		int div_id, int div)
{
	u32 reg;
	struct sun7i_i2s *i2s = snd_soc_dai_get_drvdata(cpu_dai);
	void __iomem *regs = i2s->addr;

	switch (div_id) {
	case SUN7I_DIV_MCLK:
		if (div <= 8)
			div  = (div >> 1);
		else if (div == 12)
			div = 0x5;
		else if (div == 16)
			div = 0x6;
		else if (div == 24)
			div = 0x7;
		else if (div == 32)
			div = 0x8;
		else if (div == 48)
			div = 0x9;
		else if (div == 64)
			div = 0xa;
		reg = (readl(regs + SUN7I_IISCLKD) & ~SUN7I_IISCLKD_MCLK_MASK);
		reg |= (div << SUN7I_IISCLKD_MCLK_OFFS);
		writel(reg, regs + SUN7I_IISCLKD);
		break;
	case SUN7I_DIV_BCLK:
		if (div <= 8)
			div = (div >> 1) - 1;
		else if (div == 12)
			div = 0x4;
		else if (div == 16)
			div = 0x5;
		else if (div == 32)
			div = 0x6;
		else if (div == 64)
			div = 0x7;
		reg = (readl(regs + SUN7I_IISCLKD) & ~SUN7I_IISCLKD_BCLK_MASK);
		reg |= (div << SUN7I_IISCLKD_BCLK_OFFS);
		writel(reg, regs + SUN7I_IISCLKD);
		break;
	default:
		return -EINVAL;
	}

	reg = readl(regs + SUN7I_IISCLKD);
	if (!(reg & 0xF)) {
		reg &= ~SUN7I_IISCLKD_MCLKOEN;
		writel(reg, regs + SUN7I_IISCLKD);
	} else {
		reg |= SUN7I_IISCLKD_MCLKOEN;
		writel(reg, regs + SUN7I_IISCLKD);
	}

	return 0;
}

static int sun7i_i2s_set_channel_map(struct snd_soc_dai *dai,
		unsigned int tx_num, unsigned int *tx_slot,
		unsigned int rx_num, unsigned int *rx_slot)
{
	u32 tx_map = 0, rx_map = 0;
	int i;
	struct sun7i_i2s *i2s = snd_soc_dai_get_drvdata(dai);
	void __iomem *regs = i2s->addr;

	if ((tx_num > SUN7I_PCM_TX_CHANNEL_MAX) ||
			(rx_num > SUN7I_PCM_RX_CHANNEL_MAX))
		return -EINVAL;

	for (i = 0; i < tx_num; i++)
		tx_map |= ((tx_slot[i] & 0x7) << (i * 4));

	for (i = 0; i < rx_num; i++)
		rx_map |= ((rx_slot[i] & 0x3) << (i * 4));

	writel(tx_map, regs + SUN7I_TXCHMAP);
	writel(rx_map, regs + SUN7I_RXCHMAP);

	return 0;
}

static int sun7i_i2s_dai_probe(struct snd_soc_dai *dai)
{
	dai->capture_dma_data = &sun7i_i2s_pcm_stereo_in;
	dai->playback_dma_data = &sun7i_i2s_pcm_stereo_out;

	return 0;
}
static int sun7i_i2s_dai_remove(struct snd_soc_dai *dai)
{
	return 0;
}

static void iisregsave(struct sun7i_i2s *i2s)
{
	void __iomem *regs = i2s->addr;
	i2s->regsave[0] = readl(regs + SUN7I_IISCTL);
	i2s->regsave[1] = readl(regs + SUN7I_IISFAT0);
	i2s->regsave[2] = readl(regs + SUN7I_IISFAT1);
	i2s->regsave[3] = readl(regs + SUN7I_IISFCTL) | (0x3<<24);
	i2s->regsave[4] = readl(regs + SUN7I_IISINT);
	i2s->regsave[5] = readl(regs + SUN7I_IISCLKD);
	i2s->regsave[6] = readl(regs + SUN7I_TXCHSEL);
	i2s->regsave[7] = readl(regs + SUN7I_TXCHMAP);
}

static void iisregrestore(struct sun7i_i2s *i2s)
{
	void __iomem *regs = i2s->addr;
	writel(i2s->regsave[0], regs + SUN7I_IISCTL);
	writel(i2s->regsave[1], regs + SUN7I_IISFAT0);
	writel(i2s->regsave[2], regs + SUN7I_IISFAT1);
	writel(i2s->regsave[3], regs + SUN7I_IISFCTL);
	writel(i2s->regsave[4], regs + SUN7I_IISINT);
	writel(i2s->regsave[5], regs + SUN7I_IISCLKD);
	writel(i2s->regsave[6], regs + SUN7I_TXCHSEL);
	writel(i2s->regsave[7], regs + SUN7I_TXCHMAP);
}

static int sun7i_i2s_suspend(struct snd_soc_dai *cpu_dai)
{
	u32 reg_val;
	struct sun7i_i2s *i2s = snd_soc_dai_get_drvdata(cpu_dai);
	void __iomem *regs = i2s->addr;

	/* Global Enable Digital Audio Interface */
	reg_val = readl(regs + SUN7I_IISCTL);
	reg_val &= ~SUN7I_IISCTL_GEN;
	writel(reg_val, regs + SUN7I_IISCTL);

	iisregsave(i2s);
	clk_disable(i2s->i2s_moduleclk);
	clk_disable(i2s->i2s_apbclk);

	return 0;
}
static int sun7i_i2s_resume(struct snd_soc_dai *cpu_dai)
{
	u32 reg_val;
	struct sun7i_i2s *i2s = snd_soc_dai_get_drvdata(cpu_dai);
	void __iomem *regs = i2s->addr;

	if (clk_enable(i2s->i2s_apbclk))
		pr_err("try to enable i2s_apbclk output failed!\n");

	if (clk_enable(i2s->i2s_moduleclk))
		pr_err("try to enable i2s_moduleclk output failed!\n");

	iisregrestore(i2s);

	reg_val = readl(regs + SUN7I_IISCTL);
	reg_val |= SUN7I_IISCTL_GEN;
	writel(reg_val, regs + SUN7I_IISCTL);

	return 0;
}

#define SUN7I_I2S_RATES (SNDRV_PCM_RATE_8000_192000 | SNDRV_PCM_RATE_KNOT)
static struct snd_soc_dai_ops sun7i_iis_dai_ops = {
	.trigger	= sun7i_i2s_trigger,
	.hw_params	= sun7i_i2s_hw_params,
	.set_fmt	= sun7i_i2s_set_fmt,
	.set_clkdiv = sun7i_i2s_set_clkdiv,
	.set_sysclk = sun7i_i2s_set_sysclk,
	.set_channel_map = sun7i_i2s_set_channel_map,
};

static struct snd_soc_dai_driver sun7i_iis_dai = {
	.probe = sun7i_i2s_dai_probe,
	.suspend = sun7i_i2s_suspend,
	.resume = sun7i_i2s_resume,
	.remove = sun7i_i2s_dai_remove,
	.playback = {
		.channels_min = 1,
		.channels_max = 2,
		.rates = SUN7I_I2S_RATES,
		.formats = SNDRV_PCM_FMTBIT_S16_LE | SNDRV_PCM_FMTBIT_S20_3LE
			| SNDRV_PCM_FMTBIT_S24_LE,
	},
	.capture = {
		.channels_min = 1,
		.channels_max = 2,
		.rates = SUN7I_I2S_RATES,
		.formats = SNDRV_PCM_FMTBIT_S16_LE | SNDRV_PCM_FMTBIT_S20_3LE
			| SNDRV_PCM_FMTBIT_S24_LE,
	},
	.ops = &sun7i_iis_dai_ops,
};

static int __devinit sun7i_i2s_dev_probe(struct platform_device *pdev)
{
	struct sun7i_i2s *i2s;
	int ret = 0;
	int reg_val = 0;

	i2s = devm_kzalloc(&pdev->dev, sizeof(struct sun7i_i2s), GFP_KERNEL);
	if (!i2s)
		return -ENOMEM;

	i2s->pdev = pdev;
	i2s->addr = ioremap(SUN7I_IISBASE, 0x100);
	if (i2s->addr == NULL)
		return -ENXIO;

	i2s->i2s_apbclk = clk_get(NULL, CLK_APB_I2S0);
	if (!i2s->i2s_apbclk)
		pr_err("try to get i2s_apbclk failed\n");

	if (clk_enable(i2s->i2s_apbclk))
		pr_err("i2s_apbclk failed!\n");

	i2s->i2s_pllx8 = clk_get(NULL, CLK_SYS_PLL2X8);
	if (!i2s->i2s_pllx8)
		pr_err("try to get i2s_pllx8 failed\n");

	if (clk_enable(i2s->i2s_pllx8)) {
		pr_err("enable i2s_pll2clk failed\n");
		goto out0;
	}

	i2s->i2s_pll2clk = clk_get(NULL, CLK_SYS_PLL2);
	if (!i2s->i2s_pll2clk)
		pr_err("try to get i2s_pll2clk failed\n");

	if (clk_enable(i2s->i2s_pll2clk)) {
		pr_err("enable i2s_pll2clk failed\n");
		goto out1;
	}

	i2s->i2s_moduleclk = clk_get(NULL, CLK_MOD_I2S0);
	if (!i2s->i2s_moduleclk)
		pr_err("try to get i2s_moduleclk failed\n");

	if (clk_set_parent(i2s->i2s_moduleclk, i2s->i2s_pll2clk)) {
		pr_err("try to set parent of i2s_moduleclk to i2s_pll2ck failed!\n");
		goto out2;
	}

	if (clk_set_rate(i2s->i2s_moduleclk, 24576000/8)) {
		pr_err("set i2s_moduleclk clock freq to 24576000 failed!\n");
		goto out2;
	}

	if (-1 == clk_enable(i2s->i2s_moduleclk)) {
		pr_err("open i2s_moduleclk failed!\n");
		goto out2;
	}

	reg_val = readl(i2s->addr + SUN7I_IISCTL);
	reg_val |= SUN7I_IISCTL_GEN;
	writel(reg_val, i2s->addr + SUN7I_IISCTL);

	ret = snd_soc_register_dai(&pdev->dev, &sun7i_iis_dai);
	if (ret) {
		dev_err(&pdev->dev, "Failed to register DAI\n");
		goto out3;
	}

	dev_set_drvdata(&i2s->pdev->dev, i2s);

	goto out;

out3:
	clk_disable(i2s->i2s_moduleclk);
out2:
	clk_disable(i2s->i2s_pll2clk);
out1:
	clk_disable(i2s->i2s_pllx8);
out0:
	clk_disable(i2s->i2s_apbclk);
out:
	return 0;
}

static int __devexit sun7i_i2s_dev_remove(struct platform_device *pdev)
{
	struct sun7i_i2s *i2s = dev_get_drvdata(&pdev->dev);

	if (NULL == i2s->i2s_moduleclk) {
		pr_err("i2s_moduleclk handle is invalid\n");
		return -EFAULT;
	} else {
		clk_disable(i2s->i2s_moduleclk);
	}

	if (NULL == i2s->i2s_pllx8) {
		pr_err("i2s_pllx8 handle is invalid\n");
		return -EFAULT;
	} else {
		clk_put(i2s->i2s_pllx8);
	}

	if (NULL == i2s->i2s_pll2clk) {
		pr_err("i2s_pll2clk handle is invalid\n");
		return -EFAULT;
	} else {
		clk_put(i2s->i2s_pll2clk);
	}

	if (NULL == i2s->i2s_apbclk) {
		pr_err("i2s_apbclk handle is invalid\n");
		return -EFAULT;
	} else {
		clk_put(i2s->i2s_apbclk);

		snd_soc_unregister_dai(&pdev->dev);
		platform_set_drvdata(pdev, NULL);
	}

	return 0;
}

static struct platform_driver sun7i_i2s_driver = {
	.probe = sun7i_i2s_dev_probe,
	.remove = __devexit_p(sun7i_i2s_dev_remove),
	.driver = {
		.name = "sun7i-i2s",
		.owner = THIS_MODULE,
	},
};

module_platform_driver(sun7i_i2s_driver);

MODULE_AUTHOR("REUUIMLLA");
MODULE_DESCRIPTION("sun7i I2S SoC Interface");
MODULE_LICENSE("GPL");
MODULE_ALIAS("platform:sun7i-i2s");
