/*
 * (C) Copyright 2007-2011
 * Allwinner Technology Co., Ltd. <www.allwinnertech.com>
 * Tom Cubie <tangliang@allwinnertech.com>
 *
 * Some board init for the Allwinner A10-evb board.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/io.h>
#include <fastboot.h>
#include <exports.h>
#include <fat.h>

#include <asm/arch/nand_bsp.h>
#include <mmc.h>
#include <asm/arch/android_misc.h>
#include <asm/arch/gpio.h>
#include <asm/arch/partition.h>
#include <asm/arch/boot_type.h>
#include <asm/arch/sys_config.h>
#include <i2c.h>
#include <asm/arch/mmc.h>


DECLARE_GLOBAL_DATA_PTR;

/* added by grandstream.lfzhu  begin*/
#ifdef CONFIG_GS_RECOVERY
struct seg_header_t
{
	uint32_t size_used;
	uint32_t size;
	uint16_t checksum;
	uint16_t seg_mark;
	uint32_t idt_len;
	uint32_t signature;
	uint32_t version;
	uint32_t date;
	uint16_t id;
	uint16_t hw_id;
};

typedef struct
{
	const  char *name;
	unsigned long start;
} prov_t;

static prov_t provs[] =
{
	{"boot0",},
	{"boot1",},
	{"u-boot",},
	{"kernel",},
	{"system",},
	{"recovery",},
};
#define aligned_sector(x)  (x?(((x)-1)>>9) +1:0)
#define swap(a, b) \
	do { typeof(a) __tmp = (a); (a) = (b); (b) = __tmp; } while (0)

#define PWM_CTL_REG		0x200
#define TOUCH_KEY_NUMBER  5
#define TOUCH_KEY_RECOVERY1   1
#define TOUCH_KEY_RECOVERY2   3
#define TOUCH_KEY_FACTORYRESET1  2
#define TOUCH_KEY_FACTORYRESET2   4
#define TOUCH_KEY_BACK                 5

struct ts_event {
	u16	x1;
	u16	y1;
	u16	x2;
	u16	y2;
	u16	x3;
	u16	y3;
	u16	x4;
	u16	y4;
	u16	x5;
	u16	y5;
	u16	pressure;
	s16     touch_ID1;
	s16     touch_ID2;
	s16     touch_ID3;
	s16     touch_ID4;
	s16     touch_ID5;
       u8      touch_point;
};

struct touchkey_data {
	u16 key;
	u16 x_min;
	u16 x_max;
	u16 y_min;
	u16 y_max;
};

struct touchkey_data touch_key_data[TOUCH_KEY_NUMBER] = {
	{TOUCH_KEY_RECOVERY1, 80, 190, 600, 670},
	{TOUCH_KEY_FACTORYRESET1, 270, 380, 600, 670},
	{TOUCH_KEY_RECOVERY2, 460, 570, 600, 670},
	{TOUCH_KEY_FACTORYRESET2, 650, 760, 600, 670},
	{TOUCH_KEY_BACK, 840, 950, 600, 670},
};

static u8  revert_x_flag  = 0;
static u8  revert_y_flag  = 0;
static u8  exchange_x_y_flag = 0;
static u16 screen_max_x   = 1024;
static u16 screen_max_y   = 600;

/**********************Matrix keyboard ************************************/
#define MATRIX_MAX_ROWS 5
#define MATRIX_MAX_COLS 6

unsigned int gpio_rows[] = {
	SUNXI_GPH(8),
	SUNXI_GPH(9),
	SUNXI_GPH(10),
	SUNXI_GPH(11),
	SUNXI_GPH(14),
};

unsigned int gpio_cols[] = {
	SUNXI_GPH(18),
	SUNXI_GPH(19),
	SUNXI_GPH(22),
	SUNXI_GPH(23),
	SUNXI_GPH(24),
	SUNXI_GPH(25),
};

static void matrix_gpio_init(void);
static void gs_misc_init(void);

#endif
/* added by grandstream.lfzhu  end*/

static struct bootloader_message misc_message;

void fastboot_flash_partition_init(void)
{
	fastboot_ptentry fb_part;
	int index, part_total;
	char partition_sets[1024];
	char part_name[32];
	char *pa_index;
	int  part_name_count;

	printf("--------fastboot partitions--------\n");
	part_total = sunxi_partition_get_total_num();
	if((part_total <= 0) || (part_total > MBR_MAX_PART_COUNT))
	{
		printf("mbr not exist\n");

		return ;
	}
	printf("-total partitions:%d-\n", part_total);
	printf("%-12s  %-12s  %-12s\n", "-name-", "-start-", "-size-");
	memset(partition_sets, ' ', sizeof(partition_sets));

	memset(part_name, 0, sizeof(part_name));
	if(!storage_type)
	{
		memcpy(part_name, "nanda", 5);
	}
	else
	{
		memcpy(part_name, "mmcblk0p2", 9);
		part_name_count = 2;
	}
	pa_index = partition_sets;
	for(index = 0; index < part_total && index < MBR_MAX_PART_COUNT; index++) {
		sunxi_partition_get_name(index, &fb_part.name[0]);
		fb_part.start = sunxi_partition_get_offset(index) * 512;
		fb_part.length = sunxi_partition_get_size(index) * 512;
		fb_part.flags = 0;
		printf("%-12s: %-12x  %-12x\n", fb_part.name, fb_part.start>>9, fb_part.length>>9);
		fastboot_flash_add_ptn(&fb_part);

		strcpy(pa_index, fb_part.name);
		pa_index += strlen(fb_part.name);
		*pa_index = '@' ;
		pa_index ++;

		strcpy(pa_index, part_name);
		pa_index += strlen(part_name);
		*pa_index = ':';
		pa_index ++;

		if(!storage_type)
		{
			part_name[4] ++;
		}
		else
		{
			part_name_count ++;
			if(part_name_count < 5)
			{
				part_name_count = 5;
				part_name[8] = '5';
			}
			else if(part_name_count < 10)
			{
				part_name[8] ++;
			}
			else if(part_name_count == 10)
			{
				part_name[8] = '1';
				part_name[9] = '0';
			}
			else
			{
				part_name[9] ++;
			}
		}
	}
	pa_index--;
	if(storage_type)
	{
		if(part_name_count > 10)
		{
			pa_index--;
		}
		else
		{
			*(pa_index - 1) = '1';
		}
	}
	*pa_index = '\0';
	printf("-----------------------------------\n");

	setenv("partitions", partition_sets);

}

void fastboot_partition_init(void) {
#ifdef DEBUG
	printf("fastboot_partition_init storage type = %d\n", storage_type);
#endif
	fastboot_flash_partition_init();
}

int android_misc_flash_check(void) {

	u32   misc_offset = 0;
	char  buffer[2048];

	misc_offset = (u32)sunxi_partition_get_offset_byname("misc");
	if(misc_offset == (u32)(-1))
	{
		puts("no misc partition is found\n");
		return 0;
	}
	memset(buffer, 0, 2048);
#ifdef DEBUG
	printf("misc_offset  : %x\n", (int )misc_offset);
#endif
	sunxi_flash_read(misc_offset, 2048/512, buffer);
	memcpy(&misc_message, buffer, sizeof(misc_message));
#ifdef DEBUG
//	printf("misc.command  : %s\n", misc_message.command);
//	printf("misc.status   : %s\n", misc_message.status);
//	printf("misc.recovery : %s\n", misc_message.recovery);
#endif
	if(storage_type)
	{
		if(!strcmp(misc_message.command, "boot-recovery")) {
			/* there is a recovery command */
			puts("find boot recovery\n");
			setenv("bootcmd", "run setargs_mmc boot_recovery");
			puts("Recovery detected, will boot recovery\n");
			/* android recovery will clean the misc */
		}
		else{
			printf("bootcmd set setargs_mmc\n");
			setenv("bootcmd", "run setargs_mmc boot_normal");
		}
		if(!strcmp(misc_message.command, "boot-fastboot")) {
			/* there is a fastboot command */
			setenv("bootcmd", "run setargs_mmc boot_fastboot");
			puts("Fastboot detected, will enter fastboot\n");
			/* clean the misc partition ourself */
			memset(buffer, 0, 2048);
			sunxi_flash_write(misc_offset, 2048/512, buffer);
		}
	}
	else
	{
		if(!strcmp(misc_message.command, "boot-recovery")) {
			/* there is a recovery command */
			setenv("bootcmd", "run setargs_nand boot_recovery");
			puts("find boot recovery, but will not enter boot recovery\n");
			/* android recovery will clean the misc */
			//added by zlf for not enter recovery
			memset(buffer, 0, 2048);
			sunxi_flash_write(misc_offset, 2048/512, buffer);
		}
        else
        if (!strcmp(misc_message.command, "usb-recovery")) {
		/* there is a recovery command */
		puts("find usb recovery\n");
		setenv("bootcmd", "run setargs_nand boot_recovery");
		puts("Recovery detected, will usb recovery\n");
		/* android recovery will clean the misc */
		}
		else{
			printf("bootcmd set setargs_nand\n");
			setenv("bootcmd", "run setargs_nand boot_normal");
		}

		if(!strcmp(misc_message.command, "boot-fastboot")) {
			/* there is a fastboot command */
			setenv("bootcmd", "run setargs_nand boot_fastboot");
			puts("Fastboot detected, will enter fastboot\n");
			/* clean the misc partition ourself */
			memset(buffer, 0, 2048);
			sunxi_flash_write(misc_offset, 2048/512, buffer);
		}
	}

	return 0;
}

int check_android_misc(void){
#ifdef DEBUG
	printf("check_android_misc storage type = %d\n", storage_type);
#endif
	return android_misc_flash_check();
}

void set_boot_type_arg(void){
	char buf[32];
	int i = getenv_f("target", buf, sizeof(buf));
	if (i > 0) {
		setenv("target", buf);
	}

	if(storage_type){
		setenv("bootcmd", "run setargs_mmc boot_normal");
	}
	else{
		setenv("bootcmd", "run setargs_nand boot_normal");
	}
}
/* add board specific code here */
int board_init(void) {

	gd->bd->bi_arch_number = 3893;
	gd->bd->bi_boot_params = (PHYS_SDRAM_1 + 0x100);
	serial_init();
#ifdef DEBUG
	printf("board_init storage_type = %d\n",storage_type);
#endif

	gs_misc_init();
	return 0;
}

/* Partition init must be after NAND init, so we put the fastboot
 * partition init here in the board late init.
 */
int board_late_init(void)
{
	fastboot_partition_init();
	set_boot_type_arg();
	//check_android_misc();
	return 0;
}
void dram_init_banksize(void)
{
    /* We should init the Dram options, and kernel get it by tag. */
    int dram_size;
    int ret;

    ret = script_parser_fetch("dram_para", "dram_size", &dram_size, 1);
    if (!ret) {
        if (dram_size) {
            gd->bd->bi_dram[0].size = dram_size * 1024 * 1024;
        } else {
            gd->bd->bi_dram[0].size = get_ram_size((long *)PHYS_SDRAM_1, PHYS_SDRAM_1_SIZE);
        }
    } else {
        gd->bd->bi_dram[0].size = get_ram_size((long *)PHYS_SDRAM_1, PHYS_SDRAM_1_SIZE);
    }

	gd->bd->bi_dram[0].start = PHYS_SDRAM_1;
}

int dram_init(void)
{
	gd->ram_size = get_ram_size((long *)PHYS_SDRAM_1, PHYS_SDRAM_1_SIZE);
	return 0;
}

#ifdef CONFIG_GENERIC_MMC
int board_mmc_init(bd_t *bis)
{
	sunxi_mmc_init(CONFIG_MMC_SUNXI_SLOT);

	return 0;
}

int mmc_get_env_addr(struct mmc *mmc, u32 *env_addr) {

	*env_addr = sunxi_partition_get_offset_byname(CONFIG_SUNXI_ENV_PARTITION);
	return 0;
}
#endif

#ifdef CONFIG_DISPLAY_BOARDINFO
int checkboard(void)
{
	char buf[32];
	int i = getenv_f("target", buf, sizeof(buf));
	if (i > 0) {
		puts("Board: ");
		puts(buf);
		putc('\n');
	} else {
		printf("Get Board Info Failed\n");
	}

	return 0;
}
#endif

#ifdef CONFIG_WEMAC
extern int board_eth_init(bd_t *bd);
int board_eth_init(bd_t *bd)
{
//	puts("Wemac init\n");
	wemac_eth_init(0);
	return 0;
}
#endif


int board_eth_init(bd_t *bis)
{
#ifdef CONFIG_SUN7I_GMAC
	return gmac_eth_initialize(0, 0x01c50000, 0);
#else
	return 0;
#endif
}


/* added by grandstream.lfzhu  begin*/
#ifdef CONFIG_GS_RECOVERY

static int read_bcd(struct bootloader_message *bcd)
{
	int ret = -1;
	loff_t bcd_offset = 0, bcd_size = 0;

	size_t len = sizeof(struct bootloader_message);
	size_t sec_num = aligned_sector(len);

	memset((void*)bcd, 0, len);

#ifdef CONFIG_GS_STORAGE_EMMC
	struct mmc *mmc = find_mmc_device(CONFIG_MMC_MDISK_DEV);

	ret = mmc->block_dev.block_read(CONFIG_MMC_MDISK_DEV, CONFIG_MMC_BCD_OFFSET,
						sec_num, (uchar *)bcd);
#else

	sunxi_partition_get_info_byname("nvram", &bcd_offset, &bcd_size);
	if(!bcd_offset || !bcd_size) {
		sunxi_partition_get_info_byname("NVRAM", &bcd_offset, &bcd_size);
		if(!bcd_offset || !bcd_size) {
			puts("no nvram partition is found\n");
			return -1;
		}
	}

	bcd_offset += (BCD_SECTOR_OFFSET>>9);

	char* tmpbuf = (char*)malloc(sec_num<<9);
	memset(tmpbuf, 0, sec_num<<9);
	ret = sunxi_flash_read(bcd_offset, sec_num, (void *)tmpbuf);
	memcpy((void*)bcd, tmpbuf, len);
	free(tmpbuf);

#endif
	//printf("read_bcd ret is %d\n", ret);

	return ret;
}

static int write_bcd(struct bootloader_message *bcd)
{
	int ret = -1;
	int len = sizeof(struct bootloader_message);
	int sec_num = aligned_sector(len);

#ifdef CONFIG_GS_STORAGE_EMMC
	struct mmc *mmc = find_mmc_device(CONFIG_MMC_MDISK_DEV);
	ret = mmc->block_dev.block_write(CONFIG_MMC_MDISK_DEV, CONFIG_MMC_BCD_OFFSET,
						sec_num, (uchar *)bcd);

#else
	loff_t bcd_offset = 0, bcd_size = 0;

	sunxi_partition_get_info_byname("nvram", &bcd_offset, &bcd_size);
	if(!bcd_offset || !bcd_size) {
		sunxi_partition_get_info_byname("NVRAM", &bcd_offset, &bcd_size);
		if(!bcd_offset || !bcd_size) {
			puts("no nvram partition is found\n");
			return -1;
		}
	}

	bcd_offset += (BCD_SECTOR_OFFSET>>9);

	char* tmpbuf = (char*)malloc(sec_num<<9);
	memset((void*)tmpbuf, 0, sec_num<<9);
	memcpy((void*)tmpbuf, bcd, len);
	ret = sunxi_flash_write(bcd_offset, sec_num, (void *)tmpbuf);
	free(tmpbuf);

	#endif
	//printf("write_bcd ret is %d\n", ret);
	return ret;

}

static int read_provision(struct seg_header_t *seg,unsigned long sec_offset)
{
	int ret = -1;
       int len = sizeof(struct seg_header_t);
	int sec_num = aligned_sector(len);
	loff_t prov_offset = 0, prov_size = 0;

#ifdef CONFIG_GS_STORAGE_EMMC
	char* tmpbuf = malloc(sec_num<<9);
	memset(tmpbuf, 0, sec_num<<9);
	struct mmc *mmc = find_mmc_device(CONFIG_MMC_MDISK_DEV);

	ret = mmc->block_dev.block_read(CONFIG_MMC_MDISK_DEV, sec_offset ,sec_num, (char *)tmpbuf);
	memcpy(seg, tmpbuf, sizeof(struct seg_header_t));
	free(tmpbuf);
#else

	sunxi_partition_get_info_byname("nvram", &prov_offset, &prov_size);
	if(!prov_offset || !prov_size) {
		sunxi_partition_get_info_byname("NVRAM", &prov_offset, &prov_size);
		if(!prov_offset || !prov_size) {
			puts("no nvram partition is found\n");
			return -1;
		}
	}

	prov_offset += (sec_offset>>9);

	char* tmpbuf = (char*)malloc(sec_num<<9);
	memset(tmpbuf, 0, sec_num<<9);
	ret = sunxi_flash_read(prov_offset, sec_num, (void *)tmpbuf);
	memcpy((void*)seg, tmpbuf, len);
	free(tmpbuf);
#endif
	printf("read_provision sec_offset :%lx, ret is %d\n", sec_offset, ret);
	return ret;
}

int read_provision_version(int i, char *version)
{
	unsigned int offset;
	struct seg_header_t seg;

	offset = provs[i].start = (CONFIG_VERSIONHEADER_OFFSET+i*SZ_128K);
	if (read_provision(&seg, offset) == 0) {
		if (seg.signature == GS_PROD_SIGNATURE) {
			sprintf(version, "%u.%u.%u.%u", seg.version>>24,
					(seg.version>>16)&0xff,
					(seg.version>>8)&0xff,
					seg.version&0xff);
			printf("%s ", provs[i].name);
            printf("version:%s\n", version);
		} else {
			printf("signature error!\n");
		}
	} else {
		printf("verson head read error\n");
	}

	return 0;
}

static void erase_nvram(void)
{
	int i,ret = -1;
	int time = (MTD_SIZE_NVRAM-1)/SZ_1M + 1;
	// TODO: for lazy, so nvram size must SZ_1M multiple
	int sec_num = aligned_sector(SZ_1M);
	loff_t bcd_offset = 0, bcd_size = 0;
	char* tmpbuf;

	sunxi_partition_get_info_byname("nvram", &bcd_offset, &bcd_size);
	if(!bcd_offset || !bcd_size) {
		sunxi_partition_get_info_byname("NVRAM", &bcd_offset, &bcd_size);
		if(!bcd_offset || !bcd_size) {
			puts("no nvram partition is found\n");
			return;
		}
	}

	tmpbuf = (char*)malloc(sec_num<<9);
	memset((void*)tmpbuf, 0, sec_num<<9);
	bcd_offset += (NAND_OFFSET_NVRAM>>9);
	for(i=0;i<time;i++)
	{
		ret = sunxi_flash_write(bcd_offset, sec_num, (void *)tmpbuf);
		bcd_offset += (SZ_1M>>9);
	}
	free(tmpbuf);
	printf("erase_nvram ret is %d\n", ret);
}

static int reduce_command(char *str, int len)
{
	int ret=0, times=0;
	char *endl;
	char *c = str;
	while(1)
	{
		int value=0;

		if(*c == '\n' || *c == '\0' || c-str>len)
			break;
		if(*c == ' ')
		{
			c++;
			continue;
		}
		value = simple_strtoul(c, &endl, 10);
		if(c == endl)
			break;
		else
			c = endl;
		if(value > 0 && value < 5)
		{
			if(times++ >= 4 )
				break;
			switch(value)
			{
				case 1:
					//erase userdata
					ret |= 0x01;
					break;
				case 2:
					ret |= 0x02;
			              //erase cache
					break;
				case 3:
			             //erase nvram
					erase_nvram();
					ret |= 0x04;
					break;
				case 4:
					//erase sdcard
					ret |= 0x08;
					break;
				default:
					break;
			}

		}
	}
	return ret;
}

static int handle_abnormal(void)
{
	int i;
	unsigned long offset;
	struct seg_header_t seg;

	printf("enter handle_abnormal\n");

	int part_num = sizeof(provs)/sizeof(provs[0]);

	for (i = 0; i < part_num; i++)
	{
		/*skip u-boot recovery*/
		if(!strncmp(provs[i].name, "kernel", strlen("kernel"))
			|| !strncmp(provs[i].name, "system", strlen("system")))
		{
#ifdef CONFIG_GS_STORAGE_EMMC
			offset = provs[i].start = (CONFIG_MMC_VERSIONHEADER_OFFSET+i*(CONFIG_MMC_VERSIONHEADER_SECNUM / part_num));
			if (read_provision(&seg, offset) == 0 )
				return 1;
#else
			printf("begin to read provision\n");
			offset = provs[i].start = (CONFIG_VERSIONHEADER_OFFSET+i*SZ_128K);
			if (read_provision(&seg, offset) != 0 )
				return 1;
#endif
			if (seg.signature != GS_PROD_SIGNATURE)
				return 1;
		}
	}
	return 0;
}

int gs_check_bcd(void)
{
	struct bootloader_message bcd;
	unsigned int *psize;
	int ret;

	/* factory reset */
	#ifdef CONFIG_GS_STORAGE_EMMC
	if(!check_mmc_partition_abnormal(0,0))
	{
		printf(" run factory reset\n");
		return -1;
	}

	if(strncmp(bcd.command,"factory_reset",strlen("factory_reset")) == 0)
	{
		reduce_command(bcd.command+strlen("factory_reset"), sizeof(bcd.command)/sizeof(bcd.command[0]) - strlen("factory_reset"));
		clean_one_sec(CONFIG_MMC_BCD_OFFSET);
		return 1;
	}

	ret += !strcmp("recovery", (char *)(DMW_INTRAM_BASE+0x10000 - DMW_RECOVERY_INFOSIZE));
       #endif

	if (handle_abnormal())
	{
		printf("Enter rescue mode!!!\n");
		memset(&bcd, 0, sizeof(struct bootloader_message));
		strcpy(bcd.command, "update_rescue");
		strcpy(bcd.recovery, "recovery\n--wipe_data=/data/dalvik-cache\n--update_package=all:rescue");
		write_bcd(&bcd);
		setenv("bootcmd", "run setargs_nand boot_recovery");
		return 1;
	}

	if(read_bcd(&bcd)<0)
	{
		return 0;
	}

	if(bcd.command[0]!=0 && !strncmp(bcd.command, "boot_recovery", strlen("boot_recovery")))
	{
		/* there is a recovery command */
		char* runcmd[128] = {0};
		unsigned char boot0_update = 0, boot1_update = 0;

		char *arg = strtok(bcd.command, "\n");
		if (arg != NULL && !strcmp(arg, "boot_recovery"))
		{
			while(arg = strtok(NULL, "\n"))
			{
				if(!strcmp(arg, "boot0"))
					boot0_update = 1;

				if(!strcmp(arg, "boot1"))
					boot1_update = 1;
			}
		}
		psize = bcd.status;
		if(*psize && boot0_update)
		{
			printf("find boot0 update, psize is 0x%x\n", *psize);
			sprintf(runcmd, "gs_nvram read tmp 0x40007800 0x0 0x%x", *psize);
			run_command(runcmd, 0);

			memset(runcmd, 0, sizeof(runcmd));
			sprintf(runcmd, "gs_boot write boot0 0x40007800 0x%x", *psize);
			run_command(runcmd, 0);
		}

		psize = bcd.status+4;
		if(*psize && boot1_update)
		{
			printf("find boot1 update, psize is 0x%x\n", *psize);
			sprintf(runcmd, "gs_nvram read tmp 0x40007800 0x100000 0x%x", *psize);
			run_command(runcmd, 0);

			memset(runcmd, 0, sizeof(runcmd));
			sprintf(runcmd, "gs_boot write boot1 0x40007800 0x%x", *psize);
			run_command(runcmd, 0);
		}

		if(strstr(bcd.status+8, "empty"))
		{
			printf("find boot empty\n");
			memset(&bcd, 0, sizeof(struct bootloader_message));
			write_bcd(&bcd);
			return 0;
		}
		printf("find boot recovery\n");
		//puts("Recovery detected, will boot recovery\n");
		setenv("bootcmd", "run setargs_nand boot_recovery");
		return 1;
	}

	if(bcd.command[0]!=0 && !strncmp(bcd.command, "boot_fastboot", strlen("boot_fastboot")))
	{
		/* there is a fastboot command */
		printf("find boot fastboot\n");
		//setenv("bootcmd", "run setargs_nand boot_fastboot");
		//puts("Fastboot detected, will enter fastboot\n");
		/* clean the misc partition ourself */
		//memset(&bcd, 0, sizeof(bcd));
		//sunxi_flash_write(bcd_offset, sec_num, (void *)&bcd);
		return 0;
	}

	if(bcd.command[0]!=0 && !strncmp(bcd.command, "update_sdcard", strlen("update_sdcard")))
	{
		printf("find update_sdcard\n");
		setenv("bootcmd", "run setargs_nand boot_recovery");
		return 1;
	}

	if(bcd.command[0]!=0 && !strncmp(bcd.command,"factory_reset",strlen("factory_reset")))
	{
		ret = reduce_command(bcd.command+strlen("factory_reset"), sizeof(bcd.command)/sizeof(bcd.command[0]) - strlen("factory_reset"));
		memset(&bcd, 0, sizeof(struct bootloader_message));

		if((ret & 0x01) || (ret & 0x02) || (ret & 0x08))
		{
			strcpy(bcd.command, "boot_recovery");
			strcpy(bcd.recovery, "recovery");
			if(ret & 0x01)
				strcat(bcd.recovery, "\n--wipe_data");

			if(ret & 0x02)
				strcat(bcd.recovery, "\n--wipe_cache");

			if(ret & 0x08)
				strcat(bcd.recovery, "\n--wipe_sdcard");
			write_bcd(&bcd);
			setenv("bootcmd", "run setargs_nand boot_recovery");
			printf("find factory_reset\n");
			return 1;
		}
		printf("leave factory_reset\n");
		write_bcd(&bcd);
	}

       return 0;
}

#ifdef CONFIG_GS_STORAGE_EMMC
int get_mmc_partition_secnum(int ext_part_sector, int relative,int sec_offset)
{
	unsigned char buffer[DEFAULT_SECTOR_SIZE];
	dos_partition_t *pt;
	block_dev_desc_t *dev_desc;
	int i;
	int lba_start;
	static int lba_size=0;

	dev_desc = mmc_get_dev(0);
	dev_desc->block_read(dev_desc->dev, ext_part_sector, 1, (ulong *) buffer);

	pt = (dos_partition_t *) (buffer + DOS_PART_TBL_OFFSET);
	for (i = 0; i < 4; i++, pt++)
	{
		if ((pt->sys_ind != 0) && (ext_part_sector == 0 || !is_extended (pt->sys_ind)) )
		{
			lba_start = ext_part_sector + le32_to_int (pt->start4);
			if(lba_start == sec_offset)
			{
				lba_size  = le32_to_int (pt->size4);
			}
		}
	}
	pt = (dos_partition_t *) (buffer + DOS_PART_TBL_OFFSET);
	for (i = 0; i < 4; i++, pt++)
	{
		if (is_extended (pt->sys_ind))
		{
			int lba_start = le32_to_int (pt->start4) + relative;
			get_mmc_partition_secnum (lba_start, ext_part_sector == 0 ? lba_start:relative,sec_offset);
		}
	}
	return lba_size;
}

int get_mmc_secnum(int secnum)
{
	return get_mmc_partition_secnum(0,0,secnum);
}

int clean_one_sec(long offset)
{

	struct mmc *mmc = find_mmc_device(CONFIG_MMC_MDISK_DEV);
	mmc->block_dev.block_write(CONFIG_MMC_MDISK_DEV, offset, 1, 0);
	return 0;
}

int erase_mmc(unsigned long offset , unsigned long sec_num,int whicharea)
{
	char *areaname;
	struct mmc *mmc = find_mmc_device(CONFIG_MMC_MDISK_DEV);

	switch(whicharea)
	{
		case 1: areaname = "userdata";break;
		case 2: areaname = "cache";break;
		case 3: areaname = "nvram";break;
	}

	printf("Factory reset:  clean...%s\n", areaname);
	mmc->erase(CONFIG_MMC_MDISK_DEV,(offset-1),(offset+sec_num-2));
	return 1;
}

int check_mmc_partition_abnormal(int ext_part_sector, int relative)
{
       static int partition_bad=1;

	unsigned char buffer[DEFAULT_SECTOR_SIZE];
	dos_partition_t *pt;

	block_dev_desc_t *dev_desc;
	int i;

	dev_desc = mmc_get_dev(0);

	dev_desc->block_read(dev_desc->dev, ext_part_sector, 1, (ulong *) buffer);

	if((buffer[DOS_PART_MAGIC_OFFSET + 0] != 0x55)||(buffer[DOS_PART_MAGIC_OFFSET + 1] != 0xaa))
	{
		partition_bad=0;
	}

	pt = (dos_partition_t *) (buffer + DOS_PART_TBL_OFFSET);
	for (i = 0; i < 4; i++, pt++)
	{
		if (is_extended (pt->sys_ind))
		{
			int lba_start = le32_to_int (pt->start4) + relative;
			check_mmc_partition_abnormal (lba_start, ext_part_sector == 0  ? lba_start:relative);
		}
	}


	return partition_bad;

}
#endif

static int matrix_keypad_scan(int check_row, int check_col)
{
	int row_state;
	int row, col;

	/* read checked column and read checked row status out */
	if(check_row >= ARRAY_SIZE(gpio_rows) || check_row < 0)
		return  0;

	if(check_col >= ARRAY_SIZE(gpio_cols) || check_col < 0)
		return  0;

	for(col=0; col<ARRAY_SIZE(gpio_cols); col++)
	{
		sunxi_gpio_output(gpio_cols[col], 0);
	}

	udelay(1000);
	sunxi_gpio_output(gpio_cols[check_col], 1);
	row_state = sunxi_gpio_input(gpio_rows[check_row]);
	//printf("row:%d,%d\n", check_row, row_state);
	sunxi_gpio_output(gpio_cols[check_col], 0);

	return row_state;

	//for test key
       while (1)
       {
		for(col=0; col<ARRAY_SIZE(gpio_cols); col++)
		{
			sunxi_gpio_output(gpio_cols[col], 0);
		}
	       udelay(1000);
		for(col=0; col<ARRAY_SIZE(gpio_cols); col++)
		{
			sunxi_gpio_output(gpio_cols[col], 1);
			printf("col:%d\n", col);
			for(row=0; row<ARRAY_SIZE(gpio_rows); row++)
			{
				row_state = sunxi_gpio_input(gpio_rows[row]);
				printf("row:%d,%d\n", row, row_state);
			}
			sunxi_gpio_output(gpio_cols[col], 0);
		}
		udelay(1000*1000);
       }

	return 0;
}

static void matrix_gpio_init()
{
	int i;
	/* initialized strobe lines as outputs, activated */
	for (i = 0; i< ARRAY_SIZE(gpio_cols); i++)
	{
		gpio_direction_output(gpio_cols[i], 1);
	}

	for (i = 0; i< ARRAY_SIZE(gpio_rows); i++)
	{
		gpio_direction_output(gpio_rows[i], 0);
		udelay(1000);
		gpio_direction_input(gpio_rows[i]);
	}
}

static void gs_misc_init(void)
{
	unsigned int reg_val;
	gpio_direction_output(SUNXI_GPG(5),0);//green led on
	gpio_direction_output(SUNXI_GPG(4),0);//red led on

	#if defined(CONFIG_GXV3240)
	matrix_gpio_init();
	#elif defined(CONFIG_GXV3275)
	{
		user_gpio_set_t twi0_ctrl[] =
		{
			{
				.gpio_name = "twi0 scl",
			       .port = 2,
			       .port_num = 0,
			       .mul_sel = 2,
			       .pull = 1,
			       .drv_level = 1,
			       .data = 0,
			},
			{
				.gpio_name = "twi0 sda",
			       .port = 2,
			       .port_num = 1,
			       .mul_sel = 2,
			       .pull = 1,
			       .drv_level = 1,
			       .data = 0,
			},
		};

		user_gpio_set_t twi1_ctrl[] =
		{
			{
				.gpio_name = "twi1 scl",
			       .port = 2,
			       .port_num = 18,
			       .mul_sel = 2,
			       .pull = 1,
			       .drv_level = 1,
			       .data = 0,
			},
			{
				.gpio_name = "twi1 sda",
			       .port = 2,
			       .port_num = 19,
			       .mul_sel = 2,
			       .pull = 1,
			       .drv_level = 1,
			       .data = 0,
			},
		};

		user_gpio_set_t twi2_ctrl[] =
		{
			{
				.gpio_name = "twi2 scl",
			       .port = 2,
			       .port_num = 20,
			       .mul_sel = 2,
			       .pull = 1,
			       .drv_level = 1,
			       .data = 0,
			},
			{
				.gpio_name = "twi2 sda",
			       .port = 2,
			       .port_num = 21,
			       .mul_sel = 2,
			       .pull = 1,
			       .drv_level = 1,
			       .data = 0,
			},
		};

		user_gpio_set_t ft5x_int =
		{
			.gpio_name = "ft5x int",
		       .port = 8,
		       .port_num = 3,
		       .mul_sel = 6,
		       .pull = 1,
		       .drv_level = 1,
		       .data = 0,
		};

		user_gpio_set_t  gpio_pwm1 =
		{
			.gpio_name = "pwm1",
		       .port = 9,
		       .port_num = 3,
		       .mul_sel = 2,
		       .pull = 1,
		       .drv_level = 1,
		       .data = 0,
		};

		reg_val = gpio_request(twi0_ctrl, 2);
		reg_val = gpio_request(twi1_ctrl, 2);
		reg_val = gpio_request(twi2_ctrl, 2);

		//reset ft5x
		//gpio_direction_output(SUNXI_GPG(1),0);
		//gpio_direction_output(SUNXI_GPG(1),1);
		gpio_request(&ft5x_int, 1);

	       //disable pwm1
	       gpio_request(&gpio_pwm1, 1);
		reg_val = readl(SUNXI_TIMER_BASE + PWM_CTL_REG);
		reg_val &= ~0x80000;
		writel(reg_val, SUNXI_TIMER_BASE + PWM_CTL_REG);
	}
	#endif

}

static int check_touch_key_mode(unsigned char* buf, struct ts_event* event)
{
	int i;
	u16 key_val[2] = {0};

	memset(event, 0, sizeof(struct ts_event));
	event->touch_point = buf[2] & 0x07;// 000 0111
	//printf("touch point=%d, ", event->touch_point);

	if (event->touch_point < 2) {
		//release or touch_point less than 2
		return 0;
	}

	for(i=0;i<32;i+=4)
	{
		//printf("touch [%d]  point=%d, ", i, event->touch_point);
		//printf("%d,%d,%d,%d\n", buf[i], buf[i+1], buf[i+2], buf[i+3]);
	}

	switch (event->touch_point) {
	case 5:
		event->x5 = (s16)(buf[0x1b] & 0x0F)<<8 | (s16)buf[0x1c];
		event->y5 = (s16)(buf[0x1d] & 0x0F)<<8 | (s16)buf[0x1e];
		//printf("source data:event->x5 = %d, event->y5 = %d. \n", event->x5, event->y5);

		if(1 == revert_x_flag){
			event->x5 = screen_max_x - event->x5;
		}
		if(1 == revert_y_flag){
			event->y5 = screen_max_y - event->y5;
		}
		if(1 == exchange_x_y_flag){
			swap(event->x5, event->y5);
		}
		event->touch_ID5=(s16)(buf[0x1d] & 0xF0)>>4;

		//printf("touch id : %d. \n",event->touch_ID5);
	case 4:
		event->x4 = (s16)(buf[0x15] & 0x0F)<<8 | (s16)buf[0x16];
		event->y4 = (s16)(buf[0x17] & 0x0F)<<8 | (s16)buf[0x18];
		//printf("source data:event->x4 = %d, event->y4 = %d. \n", event->x4, event->y4);

		if(1 == revert_x_flag){
			event->x4 = screen_max_x - event->x4;
		}
		if(1 == revert_y_flag){
			event->y4 = screen_max_y - event->y4;
		}
		if(1 == exchange_x_y_flag){
			swap(event->x4, event->y4);
		}
		event->touch_ID4=(s16)(buf[0x17] & 0xF0)>>4;

		//printf("touch id : %d. \n",event->touch_ID4);
	case 3:
		event->x3 = (s16)(buf[0x0f] & 0x0F)<<8 | (s16)buf[0x10];
		event->y3 = (s16)(buf[0x11] & 0x0F)<<8 | (s16)buf[0x12];
		//printf("source data:event->x3 = %d, event->y3 = %d. \n", event->x3, event->y3);

		if(1 == revert_x_flag){
			event->x3 = screen_max_x - event->x3;
		}
		if(1 == revert_y_flag){
			event->y3 = screen_max_y - event->y3;
		}
		if(1 == exchange_x_y_flag){
			swap(event->x3, event->y3);
		}
		event->touch_ID3=(s16)(buf[0x11] & 0xF0)>>4;
		//printf("touch id : %d. \n",event->touch_ID3);
	case 2:
		event->x2 = (s16)(buf[9] & 0x0F)<<8 | (s16)buf[10];
		event->y2 = (s16)(buf[11] & 0x0F)<<8 | (s16)buf[12];
		//printf("source data:event->x2 = %d, event->y2 = %d. \n", event->x2, event->y2);

		if(1 == revert_x_flag){
			event->x2 = screen_max_x - event->x2;
		}
		if(1 == revert_y_flag){
			event->y2 = screen_max_y - event->y2;
		}
		if(1 == exchange_x_y_flag){
			swap(event->x2, event->y2);
		}
		event->touch_ID2=(s16)(buf[0x0b] & 0xF0)>>4;

		//printf("touch id : %d. \n",event->touch_ID2);
	case 1:
		event->x1 = (s16)(buf[3] & 0x0F)<<8 | (s16)buf[4];
		event->y1 = (s16)(buf[5] & 0x0F)<<8 | (s16)buf[6];
		//printf("source data:event->x1 = %d, event->y1 = %d. \n", event->x1, event->y1);

		if(1 == revert_x_flag){
			event->x1 = screen_max_x - event->x1;
		}
		if(1 == revert_y_flag){
			event->y1 = screen_max_y - event->y1;
		}
		if(1 == exchange_x_y_flag){
			swap(event->x1, event->y1);
		}
		event->touch_ID1=(s16)(buf[0x05] & 0xF0)>>4;
		//printf("touch id : %d. \n",event->touch_ID1);
		break;
	default:
		return 0;
	}
	event->pressure = 200;

	if(2!=event->touch_point)
		return 0;

	if((event->y1 < touch_key_data[0].y_min) || (event->y1 > touch_key_data[0].y_max))
		return 0;

	for(i=0;i<TOUCH_KEY_NUMBER;i++)
	{
		if(event->x1 < touch_key_data[i].x_max && event->x1 > touch_key_data[i].x_min)
		{
			key_val[0] = touch_key_data[i].key;
			break;
		}
	}

	for(i=0;i<TOUCH_KEY_NUMBER;i++)
	{
		if(event->x2 < touch_key_data[i].x_max && event->x2 > touch_key_data[i].x_min)
		{
			key_val[1] = touch_key_data[i].key;
			break;
		}
	}

	if((key_val[0]==TOUCH_KEY_RECOVERY1 && key_val[1]==TOUCH_KEY_RECOVERY2) ||
		(key_val[0]==TOUCH_KEY_RECOVERY2 && key_val[1]==TOUCH_KEY_RECOVERY1))
    {
        printk("key_val[0] = %d key_val[1] = %d, run recovery\n", key_val[0], key_val[1]);
		return 1;
    }

	if((key_val[0]==TOUCH_KEY_FACTORYRESET1 && key_val[1]==TOUCH_KEY_FACTORYRESET2) ||
		(key_val[0]==TOUCH_KEY_FACTORYRESET2 && key_val[1]==TOUCH_KEY_FACTORYRESET1))
    {
        printk("key_val[0] = %d key_val[1] = %d, run factoryreset\n", key_val[0], key_val[1]);
		return 2;
    }

	return 0;
}

int check_enter_update_event(void)
{
	int ret0, ret1;
	struct bootloader_message msg;

	#if defined(CONFIG_GXV3240)
	{
		int menu_asterisk_keypad_row = 0;
		int menu_asterisk_keypad_col = 5;
		int menu_pound_keypad_row = 2;
		int menu_pound_keypad_col = 5;

		int menu_one_keypad_row = 0;
		int menu_one_keypad_col = 2;
		int menu_nine_keypad_row = 2;
		int menu_nine_keypad_col = 4;
		printf("CONFIG_GXV3240 is defined\n");

		ret0 = matrix_keypad_scan(menu_asterisk_keypad_row, menu_asterisk_keypad_col);
	       ret1 = matrix_keypad_scan(menu_pound_keypad_row, menu_pound_keypad_col);
	       if(!ret0 || !ret1)
		goto factoryreset;

	       udelay(20*1000);
		ret0 = matrix_keypad_scan(menu_asterisk_keypad_row, menu_asterisk_keypad_col);
	       ret1 = matrix_keypad_scan(menu_pound_keypad_row, menu_pound_keypad_col);
		if(!ret0 || !ret1)
		goto factoryreset;

		//start key and pound is both pressed
		memset(&msg, 0, sizeof(struct bootloader_message));
		strcpy(msg.command, "update_sdcard");
		strcpy(msg.recovery, "recovery\n--wipe_data=/data/dalvik-cache\n--update_package=all:/sdcard/gxv3240fw.bin:wrescue/sdcard/gxv3240_rescue.bin");
		write_bcd(&msg);

		/* there is a recovery command */
		puts("Recovery detected, will boot recovery\n");
		/* android recovery will clean the misc */
		setenv("bootcmd", "run setargs_nand boot_recovery");
		return 1;

factoryreset:
		ret0 = matrix_keypad_scan(menu_one_keypad_row, menu_one_keypad_col);
	       ret1 = matrix_keypad_scan(menu_nine_keypad_row, menu_nine_keypad_col);
	       if(!ret0 || !ret1)
		return 0;

	       udelay(20*1000);
		ret0 = matrix_keypad_scan(menu_one_keypad_row, menu_one_keypad_col);
	       ret1 = matrix_keypad_scan(menu_nine_keypad_row, menu_nine_keypad_col);
		if(!ret0 || !ret1)
		return 0;

		erase_nvram();

		memset(&msg, 0, sizeof(struct bootloader_message));
		strcpy(msg.command, "boot_recovery");
		strcpy(msg.recovery, "recovery\n--wipe_data\n--wipe_cache");
		write_bcd(&msg);

		/* there is a factory reset command */
		puts("factoryreset detected, will boot factoryreset\n");
		setenv("bootcmd", "run setargs_nand boot_recovery");

	}
	#elif defined(CONFIG_GXV3275)
	{
		struct ts_event event;
		unsigned short reg_addr;
		unsigned char data_len;
		unsigned char read_data[32]={0};
		unsigned int reg_val, gpio_buf;
		unsigned int cycle = 0, cycle1=0, cycle2=0;
		unsigned char touches[2] = {0};

#if 0
	       user_gpio_set_t twi0_ctrl[] =
		{
			{
				.gpio_name = "twi0 scl",
			       .port = 2,
			       .port_num = 0,
			       .mul_sel = 2,
			       .pull = 1,
			       .drv_level = 1,
			       .data = 0,
			},
			{
				.gpio_name = "twi0 sda",
			       .port = 2,
			       .port_num = 1,
			       .mul_sel = 2,
			       .pull = 1,
			       .drv_level = 1,
			       .data = 0,
			},
		};
#endif
		user_gpio_set_t twi1_ctrl[] =
		{
			{
				.gpio_name = "twi1 scl",
			       .port = 2,
			       .port_num = 18,
			       .mul_sel = 2,
			       .pull = 1,
			       .drv_level = 1,
			       .data = 0,
			},
			{
				.gpio_name = "twi1 sda",
			       .port = 2,
			       .port_num = 19,
			       .mul_sel = 2,
			       .pull = 1,
			       .drv_level = 1,
			       .data = 0,
			},
		};
#if 0
		user_gpio_set_t twi2_ctrl[] =
		{
			{
				.gpio_name = "twi2 scl",
			       .port = 2,
			       .port_num = 20,
			       .mul_sel = 2,
			       .pull = 1,
			       .drv_level = 1,
			       .data = 0,
			},
			{
				.gpio_name = "twi2 sda",
			       .port = 2,
			       .port_num = 21,
			       .mul_sel = 2,
			       .pull = 1,
			       .drv_level = 1,
			       .data = 0,
			},
		};
#endif
		//reset nt99141
		//gpio_direction_output(SUNXI_GPH(23),0);
		//udelay(1000);
		//gpio_direction_output(SUNXI_GPH(23),1);
		printf("CONFIG_GXV3275 is defined\n");

	      #if TWI_TEST
		reg_addr = 0x03;
		i2c0_init(200000, 0xff);
		ret = i2c0_read(0x30, reg_addr, 1, read_data, 1);
	       printf("ret0 is %x, buf is %x\n", ret, read_data[0]);

		reg_addr = 0x3000;
		i2c2_init(200000, 0xff);
		ret = i2c2_read(0x54, reg_addr, 2, read_data, 1);
	       printf("ret2 is %x, buf is %x\n", ret, read_data[0]);
		#endif

		//reg_addr = 0x00;
		//data_len = 31;
		//reg_val = gpio_read_one_pin_value(gpio_buf, "ft5x int");
		//printf("idle gpio_read_one_pin_value gpio_buf:%d, value: %d\n", gpio_buf, reg_val);

		//reg_val = sunxi_gpio_input(SUNXI_GPH(3));
		eGon2_twi_init(1, twi1_ctrl, 24000000, 200000);
		while(cycle++<5000)
	      {
		       reg_addr = 0x00;
			data_len = 31;
			memset(read_data, 0, sizeof(read_data));
			reg_val = sunxi_gpio_input(SUNXI_GPH(3));
			if(reg_val)
			{
				//cycle1++;
				continue;
			}
			else
			{
				//printf("idle gpio_get_value is %d, cycle:%d, cycle1:%d, cycle2:%d\n", reg_val, cycle, cycle1, cycle2);
			}

			//ret0 = i2c_read(CONFIG_SYS_I2C_FT5X_ADDR, reg_addr, 1, read_data, data_len);
			if(BOOT_TWI_Read(CONFIG_SYS_I2C_FT5X_ADDR, &reg_addr, read_data, data_len))
			{
				//cycle2++;
				continue;
			}

			ret0 = check_touch_key_mode(read_data, &event);
			if(ret0>0)
			{
				touches[ret0-1]++;
				if(touches[ret0-1]>=2)
				{
					break;
				}
			}

	       }

		if(touches[0]<2 && touches[1]<2)
			return 0;

		if(touches[0]>=2)
		{
			//start key and pound is both pressed
			memset(&msg, 0, sizeof(struct bootloader_message));
			strcpy(msg.command, "update_sdcard");
			strcpy(msg.recovery, "recovery\n--wipe_data=/data/dalvik-cache\n--update_package=all:/sdcard/gxv3275fw.bin:wrescue/sdcard/gxv3275_rescue.bin");
			write_bcd(&msg);

			/* there is a recovery command */
			puts("Recovery detected, will boot recovery\n");
			/* android recovery will clean the misc */
			setenv("bootcmd", "run setargs_nand boot_recovery");
		}
		else if(touches[1]>=2)
		{
			//one key and nine is both pressed
			erase_nvram();

			memset(&msg, 0, sizeof(struct bootloader_message));
			strcpy(msg.command, "boot_recovery");
			strcpy(msg.recovery, "recovery\n--wipe_data\n--wipe_cache");
			write_bcd(&msg);

			/* there is a factory reset command */
			puts("factoryreset detected, will boot factoryreset\n");
			setenv("bootcmd", "run setargs_nand boot_recovery");
		}
	}
	#endif

	return 1;
}

#endif

#if defined(CONFIG_GXV3275) || defined(CONFIG_GXV3240)
extern int do_gs_nvram(cmd_tbl_t*, int, int, char *const[]);
#ifndef CONFIG_MMC_SDCARD_DEV
#define CONFIG_MMC_SDCARD_DEV	0
#endif
int gs_check_cfg(void)
{
	cmd_tbl_t *bcmd;
	char *const cmd_args[] = {"gs_nvram","write", "fact", "0x40008000", "0x0", "0x100000", NULL};
	char *target;
	char cfg_name[16];
	int i, ret = 0;
	struct mmc *mmc = find_mmc_device(CONFIG_MMC_SDCARD_DEV);

	if (mmc == NULL)
	{
		ret = -1;
		goto exit;
	}

	mmc_init(mmc);

      if (!check_fact())
      {
		ret = 1;
		goto exit;
	}

	if (IS_SD(mmc))
	{
		for (i = 0; i < 4; i++)
		{
			if (fat_register_device(&mmc->block_dev, i) == 0)
			{
				target = getenv("target");
				sprintf(cfg_name, "%s.cfg", target);
				if (file_fat_read(cfg_name, 0x40008000, 0) != -1)
				{
					bcmd = find_cmd("gs_nvram");
					do_gs_nvram(bcmd, 0, 6, cmd_args);
					ret = 0;
					goto exit;
				}
			}
		}
		printf("** Can't find cfg file in sdcard **\n");
             ret = -2;
		goto exit;
	}
	printf("** No sdcard in slot **\n");
	ret = -3;
exit:
	/*if(!ret)
	{
		gpio_direction_output(SUNXI_GPG(5),0);//green led on
		gpio_direction_output(SUNXI_GPG(4),1);//red led off
		while(1)
			udelay(1000*1000);
	}*/

	gpio_direction_output(SUNXI_GPG(5),1);//green led off
	gpio_direction_output(SUNXI_GPG(4),1);//red led off
	return ret;
}
#endif
/* added by grandstream.lfzhu  end*/
