/*
 * (C) Copyright 2000
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.,
 */

#include <common.h>
#include <asm/arch/android_misc.h>
#include <command.h>
#include <asm/arch/partition.h>
#include <config.h>
#include <common.h>
#include <nand.h>
#include <bmp_layout.h>
#include <lcd.h>
#include <mmc.h>

#include <net.h>
#include "../drivers/net/gsgmac/sun7i_gmac_base.h"

#ifdef CONFIG_LZMA
#include <lzma/LzmaTools.h>
#endif
 
#ifdef CONFIG_SUNXI
#include <asm/arch/boot_type.h>
#endif

#include <configs/gs_logo.h>

#ifdef CONFIG_GVR3550
#define  DEFAULT_GS_MAC_ADDRESS {0x00, 0x0b, 0x82, 0xff, 0xff, 0xff}
#else
#define  DEFAULT_GS_MAC_ADDRESS {0x00, 0x0b, 0x82, 0x00, 0x00, 0x00}
#endif

static char *logo_data = NULL;
static unsigned short logo_def = 0;
static unsigned int logo_data_len = 0;
static unsigned long logo_len = 0;
#define logo_decompressed 0x58100000
extern int lcd_display_bitmap(ulong bmp_image, int x, int y);

enum gs_nand_part
{
	PART_NVRAM   =  0,
       PART_SEGHDR,
	PART_FACTORY_CFG,
	PART_RESERVED,
	PART_BCD,
};

#define aligned_sector(x)  (x?(((x)-1)>>9) +1:0)
unsigned char factParaBuf[GS_FACTORY_CFG_SIZE];
typedef struct
{
    unsigned  short int length[2];       //the length of factorypara.confg file,Length[1]<<16| Length[0]
    unsigned  short int checksum;        //This filler ensures that all fields add up to 0x0000
    unsigned  short int mac[3];          //Ethernet MAC address
    unsigned  short int hw_model;
    unsigned  short int RevID[4][2];     //RevID[0][0]main PCB, major component change,(V2.0 = 0x0200),(V2.1 = 0x0201).
                                                                                //RevID[0][1] main PCB, minor component change(RevB = 0x0001),(RevA = 0x0000)
    unsigned  short int version;
    unsigned  short int PartNumber[6];
    unsigned  short int reserved[21];
    unsigned  short int hw_feature_table_word[];
} GS_FACTORYPARA;
typedef enum
{
    HW_FEATURE_LOWEST_FIRMWARE_VERSION = 2,   
    HW_FEATURE_VENDOR = 3,                    

    HW_FEATURE_HEADSET_TYPE = 600,
    HW_FEATURE_ETH_DEV = 601,
    HW_FEATURE_FXO_CS = 602,
    HW_FEATURE_FXS_CS = 603,
    HW_FEATURE_FLASH_TYPE = 604,

    HW_FEATURE_MODELSTR = 605,                 
    HW_FEATURE_MODELALIAS = 606,               
    HW_FEATURE_MODELBASE  = 650,

    HW_FEATURE_FWKEY = 607,                    
    HW_FEATURE_LOGODEF = 608,                  
    HW_FEATURE_LOGODATA = 609,                 
    HW_FEATURE_OEM_ID = 610,                   
    HW_FEATURE_VENDOR_FULLNAME = 611,          

    HW_FEATURE_CAMERA_SENSOR_TYPE = 612,
    HW_FEATURE_CDC_DEV = 613,
    HW_FEATURE_PA_SHUTDOWN_TYPE = 614,
    HW_FEATURE_CAMERA_STANDBY_CTRL = 615,
    HW_FEATURE_CPU_TYPE = 616,

    HW_FEATURE_HF_PA_GAIN = 617,                  
    HW_FEATURE_HF_SPEAKER_TYPE = 618,             
    HW_FEATURE_HF_MIC_TYPE = 619,                 
    HW_FEATURE_HANDSET_TYPE = 620,            
    HW_FEATURE_AUDIO_CDC_MCLK_RATE = 621,

    HW_FEATURE_TOUCH_SCREEN_TYPE = 700,
    HW_FEATURE_USB_GPIO_SET = 701,
    HW_FEATURE_USB_HUB_CLK_SRC = 702,
    HW_FEATURE_CAM_BITS = 945,

    HW_FEATURE_NOT_RELATED = 0xFFFF,
	HW_FEATURE_START_SCREEN = 0
}hw_feature_type_t;
#ifdef CONFIG_GS_STORAGE_FLASH
static unsigned short fact_para_checksum(GS_FACTORYPARA *factPara)
{
    unsigned char  *p = (unsigned char *)factPara;
    unsigned short sum = 0;
    unsigned short checksum = 0;
    unsigned int length = 0;
    int i;

    // the length field is designed to be able expand the structure but since we need copy
    // the whole structure, it makes nothing except has chance to crash when length is invalid
    length = factPara->length[1]<<16|factPara->length[0]; // le
    
    if (length > GS_FACTORY_CFG_SIZE)
        length = GS_FACTORY_CFG_SIZE;

    for (i = 0; i < length; i+=2)   // first 4 byts are 
    {
        if ( i == offsetof(GS_FACTORYPARA,checksum))
        {
            continue;   // skip checksum itself
        }
        sum += ((p[i] << 8) | p[i+1]);    // fact config use be for checksum calc
    }
    checksum = 0xffff - sum + 0x1;

    return ((checksum >> 8) | (checksum << 8)); // be
}


static int read_nandflash(char* part, char* buf, unsigned long offset, unsigned long len)
{
	int ret = -1;
	loff_t nand_offset = 0, nand_size = 0;	
	size_t sec_num = aligned_sector(len);
	char* tmpbuf = NULL;
		
	sunxi_partition_get_info_byname(part, &nand_offset, &nand_size);
	if(!nand_offset || !nand_size)
	{
		puts("no part partition is found\n");
		return -1;
	}
	   
	nand_offset += aligned_sector(offset);
	tmpbuf = (char*)malloc(sec_num<<9);
	if(tmpbuf==NULL)
	{
		puts("malloc failed\n");
		return -1;
	}
	
	memset(tmpbuf, 0, sec_num<<9);
	ret = sunxi_flash_read(nand_offset, sec_num, (void *)tmpbuf);
	memcpy(buf, tmpbuf, len);
	free(tmpbuf);
	
	return ret;
}


static int write_nandflash(char* part, char* buf, unsigned long offset, unsigned long len)
{
	int ret = -1;
	loff_t nand_offset = 0, nand_size = 0;
	char* tmpbuf = NULL;
	unsigned long  buf_offset=0, last_len=0;
	unsigned long write_count = 0;
		
	sunxi_partition_get_info_byname(part, &nand_offset, &nand_size);
	if(!nand_offset || !nand_size)
	{
		puts("no partition is found\n");
		return -1;
	}

	nand_offset += aligned_sector(offset);	
	write_count = len>>9;
	last_len = len%512;

	if(write_count)
	{
		ret = sunxi_flash_write(nand_offset, write_count, (void *)buf);
	}
	buf_offset = write_count<<9;		
	nand_offset += write_count;
	
	if(last_len)
	{
		tmpbuf = (char*)malloc(512);
		if(tmpbuf==NULL)
		{
			puts("malloc failed\n");
			return -1;
		}
		memset((void*)tmpbuf, 0, 512);
		ret = sunxi_flash_read(nand_offset, 1, (void *)tmpbuf);
		memcpy(tmpbuf, buf+buf_offset, last_len);
		ret = sunxi_flash_write(nand_offset, 1, (void *)tmpbuf);
		free(tmpbuf);
	}	
	
	return ret;
}
#endif

int do_gs_nvram(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret = -1;
	unsigned long length;
	unsigned long ramaddr, nandoffset;
	char part[16]  = {0};
	char cmd[16];
	unsigned char mac[6];
	char* tmpbuf = NULL;
	GS_FACTORYPARA* factp = NULL;
	if (argc < 6)
		return cmd_usage(cmdtp);

	strcpy(part, argv[2]);
	ramaddr =  simple_strtoul(argv[3], NULL, 16);
	nandoffset =  simple_strtoul(argv[4], NULL, 16);
	length =  simple_strtoul(argv[5], NULL, 16);
	tmpbuf = (char*)ramaddr;
	
	switch(part[0])
	{
		case 'n':
			if((length+nandoffset)>MTD_SIZE_NVRAM)
				return -1;
			nandoffset += NAND_OFFSET_NVRAM;
			break;
		case 's':
			if((length+nandoffset)>MTD_SIZE_SEGHDR)
				return -1;
			nandoffset += NAND_OFFSET_SEGHDR0;
			break;
		case 'f':
			{
#if 0
			factp = (GS_FACTORYPARA*)tmpbuf;
			eth_parse_enetaddr(getenv("ethaddr"), mac);
			factp->mac[0] = (0xff&mac[0]) | (0xff00&(mac[1] << 8));
			factp->mac[1] = (0xff&mac[2]) | (0xff00&(mac[3] << 8));
			factp->mac[2] = (0xff&mac[4]) | (0xff00&(mac[5] << 8));
			factp->checksum = fact_para_checksum(factp);
#endif
			if((length+nandoffset)>GS_FACTORY_CFG_SIZE)
				return -1;
			nandoffset += GS_FACTORY_CFG_OFFSET;
			break;
			}
		case 'r':
			if((length+nandoffset)>MTD_SIZE_RESERVED)
				return -1;
			nandoffset += GS_OFFSET_RESERVED;
			break;
		case 'b':
			if((length+nandoffset)>MTD_SIZE_BCD)
				return -1;
			nandoffset += BCD_SECTOR_OFFSET;
			break;
#ifndef CONFIG_GVR3550
		case 't':
			if((length+nandoffset)>MTD_SIZE_TEMP)
			{
				printf("oh, access tmp area data over 4M, 0x%x\n", (length+nandoffset));
				return -1;
			}
			nandoffset += GS_TEMP_SECTOR_OFFSET;
			break;
#endif
		default:
			printf("Do not support this command\n");
			return cmd_usage(cmdtp);
	}
	
       strcpy(cmd, argv[1]);
	if (strncmp(cmd, "read", 4) == 0)
	{
#ifdef CONFIG_GVR3550
		ret = read_nandflash("rawdata", tmpbuf, nandoffset, length);
#else
        ret = read_nandflash("nvram", tmpbuf, nandoffset, length);
#endif
	}
	else if (strncmp(cmd, "write", 5) == 0)
	{
#ifdef CONFIG_GVR3550
		ret = write_nandflash("rawdata", tmpbuf, nandoffset, length);
#else
        ret = write_nandflash("nvram", tmpbuf, nandoffset, length);
#endif
	}
	
	return ret;

}

static int do_gs_flash(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret = -1;
	unsigned long length;
	unsigned long ramaddr, nandoffset;
	char part[16] = {0};
	char cmd[16];
	char* tmpbuf = NULL;
	if (argc < 6)
		return cmd_usage(cmdtp);

	strcpy(part, argv[2]);
	ramaddr =  simple_strtoul(argv[3], NULL, 16);
	nandoffset =  simple_strtoul(argv[4], NULL, 16);
	length =  simple_strtoul(argv[5], NULL, 16);
	tmpbuf = (char*)ramaddr;
	
	switch(part[0])
	{
		case 'u':
			if((length+nandoffset)>SZ_16M)
				length = (SZ_16M-nandoffset);
			strcpy(part, "bootloader");
			break;
		case 'k':
			if((length+nandoffset)>SZ_32M)
				length = (SZ_32M-nandoffset);
			strcpy(part, "boot");
			break;
		case 's':
			if((length+nandoffset)>SZ_512M)
				length = (SZ_512M-nandoffset); 
			strcpy(part, "system");
			break;
		case 'r':
			if((length+nandoffset)>SZ_32M)
				length = (SZ_32M-nandoffset); 
			strcpy(part, "recovery");
			break;
		case 'n':			
			if((length+nandoffset)>SZ_16M)
				length = (SZ_16M-nandoffset); 
			strcpy(part, "nvram");
			break;
		default:
			printf("Do not support this command\n");
			return cmd_usage(cmdtp);
	}
	
       strcpy(cmd, argv[1]);
	if (strncmp(cmd, "read", 4) == 0)
	{
		ret = read_nandflash(part, tmpbuf, nandoffset, length);
	}
	else if (strncmp(cmd, "write", 5) == 0)
	{
		ret = write_nandflash(part, tmpbuf, nandoffset, length);
	}
	
	return ret;

}

U_BOOT_CMD(
	gs_flash, 6,	1,	do_gs_flash,
	"gs_flash nand flash operation",
	"param 0: cmd[read/write]\n"
	"param 1: part name[uboot/kernel/system/recovery/nvram]\n"
	"param 2: ram addr in hex\n"
	"param 3: nand flash offset from part in hex\n"
	"param 4: data length in hex"
);

U_BOOT_CMD(
	gs_nvram, 6,	1,	do_gs_nvram,
	"gs_nvram nvram operation",
	"param 0: cmd[read/write]\n"
	"param 1: nvram part name[nvram/seghdr/fact/reserve/bcd]\n"
	"param 2: ram addr in hex\n"
	"param 3: nand flash offset from part in hex\n"
	"param 4: data length in hex\n"
);

#ifdef CONFIG_GS_BOOT_TEST
static int do_gs_boot(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret = -1;
	unsigned long length;
	unsigned long ramaddr;
	char part[16] = {0};
	char cmd[16];
	char* tmpbuf = NULL;
	if (argc < 5)
		return cmd_usage(cmdtp);
	
	strcpy(part, argv[2]);
	ramaddr =  simple_strtoul(argv[3], NULL, 16);
	length =  simple_strtoul(argv[4], NULL, 16);
	tmpbuf = (char*)ramaddr;

	if (strncmp(part, "boot0", 5) == 0)
	{
	
	}
	else if (strncmp(part, "boot1", 5) == 0)
	{
	}
	else
	{
		printf("Do not support this command\n");
		return cmd_usage(cmdtp);
	}
	
       strcpy(cmd, argv[1]);
	if (strncmp(cmd, "read", 4) == 0)
	{
		ret = read_boot_from_nand(part, ramaddr, length);
	}
	else if (strncmp(cmd, "write", 5) == 0)
	{
		ret = write_boot_to_nand(part, ramaddr, length);
	}
	printf("do_gs_boot ret is %d\n", ret);
	
	return ret;
}

U_BOOT_CMD(
	gs_boot, 5,	0,	do_gs_boot,
	"boot0 and boot1 operation",
	"read - part[boot0/boot1] ramaddr size[hex]\n"
	"gs_boot write - part[boot0/boot1] ramaddr size[hex]\n"
);
#endif

int check_fact(void)
{
	GS_FACTORYPARA *factPara = NULL;
	memset((char*)0x42000000, 0, 0x100000);
	memset(factParaBuf, 0, GS_FACTORY_CFG_SIZE);
	
	run_command ("gs_nvram read fact 0x42000000 0x0 0x100000", 0);
	memcpy(factParaBuf, (char*)0x42000000, 0x100000);
	factPara=factParaBuf;
    if ((factPara->checksum == 0x0000) || (factPara->checksum == 0xffff) ||
			(fact_para_checksum(factParaBuf) != factPara->checksum))
		return 1;

	return 0;
}

int load_fact_para(void)
{
      GS_FACTORYPARA *factPara = NULL;
      unsigned int length = 0;
      int ii;
      int ret = 0;
	unsigned long ramaddr, nandoffset;
	char* tmpbuf = NULL;
    char mac_addr[32]="";
    u8 mac_addr0[6] = DEFAULT_GS_MAC_ADDRESS;

    memset((char*)0x42000000, 0, 0x100000);
    memset(factParaBuf, 0, GS_FACTORY_CFG_SIZE);//add by dean
    run_command ("gs_nvram read fact 0x42000000 0x0 0x100000", 0);
    memcpy(factParaBuf, (char*)0x42000000, 0x100000);
    factPara=factParaBuf;
	printf("factPara->checksum:    %d\n", factPara->checksum);
    if ((factPara->checksum != 0x0000) && (fact_para_checksum(factParaBuf) == factPara->checksum))
    {
        char *pn_ver = (char *)factPara->PartNumber;
        printf("PN:    %.11s\n", pn_ver);
        if(((factPara->mac[0] & 0xFF) == 0xFF) && (factPara->checksum == 0xffff)){
            printf("**factory data is empty or invalid!\n");
            return -1;
        }
		setenv("pn", pn_ver);
        sprintf(mac_addr, "%02X:%02X:%02X:%02X:%02X:%02X",
            factPara->mac[0] & 0xFF, (factPara->mac[0] >> 8) & 0xFF,
            factPara->mac[1] & 0xFF, (factPara->mac[1] >> 8) & 0xFF,
            factPara->mac[2] & 0xFF, (factPara->mac[2] >> 8) & 0xFF);
        setenv("ethaddr", mac_addr);
        printf("MAC:   %s\n", mac_addr);

        /* check feature table for logo data */
        length = (factPara->length[1] << 16) | factPara->length[0]; // le
        if (length > sizeof(GS_FACTORYPARA))
        {
            /* hw feature table is available */
            unsigned short *featurePtr;
            unsigned short hw_feature_table_len = factPara->hw_feature_table_word[0];
            
            if (hw_feature_table_len > 3)
            {
                ii = 0;
                while (ii < (hw_feature_table_len - 3))
                {
                    featurePtr = (unsigned short*)factPara->hw_feature_table_word + ii + 3;
                    if (*featurePtr != HW_FEATURE_LOGODEF && *featurePtr != HW_FEATURE_LOGODATA)
                    {
                        ii += *(featurePtr+1) + 2;
                        continue;
                    }
                    if (*featurePtr == HW_FEATURE_LOGODEF)
                    {
                        logo_def = *(featurePtr+2);
                    }
                    else
                    {
                        logo_data = (char *)(featurePtr+2);
                        logo_data_len = (*(featurePtr+1) << 1);
                    }
                    ii += *(featurePtr+1) + 2;
                    continue;
                }
            }
        }
    }
    else
    {
    	   printf("**factory data is empty or error!\n");
		   ret = -1;
    }
    return ret;
}

static int save_fact_para(char *buf_ptr)
{
    size_t total;
    int ret = 0;

#ifdef CONFIG_GVR3550
    ret = write_nandflash("rawdata", buf_ptr, GS_FACTORY_CFG_OFFSET, GS_FACTORY_CFG_SIZE);
#else
    ret = write_nandflash("nvram", buf_ptr, GS_FACTORY_CFG_OFFSET, GS_FACTORY_CFG_SIZE);
#endif

    puts ("done\n");

    return ret;
}

static void bmptolcd(void)
{
	//add by dean   
    bmp_image_t *bmp=(bmp_image_t *)logo_decompressed;
    int i,j;
    unsigned long width, height;
    unsigned colors, bmp_bpix;
    unsigned long compression;
    uchar *bmap;
    uchar *fb;
    uchar *pt_fb;
    uchar cmap[256][4];
	
	if (!((bmp->header.signature[0]=='B') &&
		(bmp->header.signature[1]=='M'))) {
		printf ("Error: no valid bmp image at %lx\n", bmp);
		return 1;
	}

	width = le32_to_cpu (bmp->header.width);
	//printf("dean:width %d\n",width);
	height = le32_to_cpu (bmp->header.height);
	//printf("dean:height %d\n",height);
	bmp_bpix = le16_to_cpu(bmp->header.bit_count);
	//printf("dean:bmp_bpix %d\n",bmp_bpix);
	colors = 1 << bmp_bpix;
	//printf("dean:colors %d\n",colors);
	compression = le32_to_cpu (bmp->header.compression);
	//printf("dean:compression %d\n",compression);


	bmap = (uchar *)bmp + le32_to_cpu (bmp->header.data_offset);
      fb   = (uchar *) (0x48000000)+(480*(272-1))*4;//bmp from lower to high
	pt_fb=fb;
	switch (bmp_bpix) 
	{
		case 4:
			//printf("dean:4444\n");
			/* Set color map */
			for (i=0; i<colors; ++i)
			{
				bmp_color_table_entry_t cte = bmp->color_table[i];
				cmap[i][0]=cte.blue;
				cmap[i][1]=cte.green;
				cmap[i][2]=cte.red;
				cmap[i][3]=cte.reserved;
			}
			for (i = 0; i < height; i++) 
			{
				for (j = 0; j < width/2; j++) 
				{
					unsigned char low=*bmap&0x0f;
					unsigned char high=(*bmap>>4)&0x0f;
					*(pt_fb++) = cmap[low][0];//R
					*(pt_fb++) = cmap[low][1];//G
					*(pt_fb++) = cmap[low][2];//B
					*(pt_fb++) = cmap[low][3];//empty

					*(pt_fb++) = cmap[high][0];//R
					*(pt_fb++) = cmap[high][1];//G
					*(pt_fb++) = cmap[high][2];//B
					*(pt_fb++) = cmap[high][3];//empty
					
					bmap++;
				}
				fb  -=  480*4;
				pt_fb=fb;
			}
			break;
		case 8:
			//printf("dean:8888\n");
			/* Set color map */
			for (i=0; i<colors; ++i)
			{
				bmp_color_table_entry_t cte = bmp->color_table[i];
				cmap[i][0]=cte.blue;
				cmap[i][1]=cte.green;
				cmap[i][2]=cte.red;
				cmap[i][3]=cte.reserved;
			}
			for (i = 0; i < height; i++) 
			{
				for (j = 0; j < width; j++) 
				{
					*(pt_fb++) = cmap[*bmap][0];//R
					*(pt_fb++) = cmap[*bmap][1];//G
					*(pt_fb++) = cmap[*bmap][2];//B
					*(pt_fb++) = cmap[*bmap][3];//empty
					bmap++;
				}
				fb  -=  480*4;
				pt_fb=fb;
			}
			break;
		case 24:
			//printf("dean:2424\n");
			for (i = 0; i < height; i++) 
			{
				for (j = 0; j < width; j++) 
				{
					*(pt_fb++) = bmap[0] ;//R
					*(pt_fb++) = bmap[1];//G
					*(pt_fb++) = bmap[2];//B
					*(pt_fb++) = 0x00;//empty
					bmap += 3;
				}
				fb  -=  480*4;
				pt_fb=fb;
			}
			break;
		default:
			//printf("bmp_bpix must be 8/16/32\n ");
			break;
	}
}

void gxv_display_logo(void)
{
    bmp_image_t *bmp;
    int i;
	load_fact_para();
    if (!logo_data)
    {
        /* use built-in logo */
        logo_data = gs_logo;
        logo_data_len = gs_logo_size;
        logo_def = 1;
    }
        
    bmp = (bmp_image_t *)logo_data;
      
    if (!((bmp->header.signature[0]=='B') &&
		  (bmp->header.signature[1]=='M'))) {
        i = lzmaBuffToBuffDecompress ((char*)logo_decompressed,
                             &logo_len, (char *)logo_data, logo_data_len);
        if (i != 0) {
            printf("decompress logo error!\n");
        }
    }
	bmptolcd();
}


U_BOOT_CMD(
	gxvlogo,   1,   0,     gxv_display_logo,
	"enable gxv logo",
	"\n"
	"    - enable gxv logo"
);

#ifdef CONFIG_GVR3550

typedef enum
{
    RAW_PART_NVRAM = 1,
    RAW_PART_USERCFG = 2
}rawdata_part_t;

struct ihdr_desc
{
	const char *name;
	unsigned int start;
	unsigned int size;
};

typedef struct _image_header {
    unsigned int Signature;
    unsigned int Version;
    unsigned int Size;
    unsigned int SizeUsed;

    unsigned short Id;
    unsigned short Checksum;
    unsigned short Year;
    unsigned short MonthDay;
    unsigned short HourMinute;
    unsigned short ModelKey;

    unsigned short FwVerMask;
    unsigned short reserved[1];
} ImageHeader, *pImageHeader;

static struct ihdr_desc ihdr_dtbl[]=
{
	{"bootr", NAND_OFFSET_IMGHDR0_BOOTR, MTD_SIZE_IMGHDR},
	{"boota", NAND_OFFSET_IMGHDR0_BOOTA, MTD_SIZE_IMGHDR},
	{"bootb", NAND_OFFSET_IMGHDR0_BOOTB, MTD_SIZE_IMGHDR},
	{"systema", NAND_OFFSET_IMGHDR0_SYSTEMA, MTD_SIZE_IMGHDR},
	{"systemb", NAND_OFFSET_IMGHDR0_SYSTEMB, MTD_SIZE_IMGHDR},
};

/*
* read lastupdate identify.
*/
static int read_lastupdate_id(ImageHeader *id)
{
	int rval;
	char buf[512] = "";

    rval = read_nandflash("rawdata", buf, NAND_OFFSET_LASTUPDID, SECTORSIZE);

	if(rval < 0) {
        printf("read lastupdate ID error!\n");
    } else {
        memcpy(id, buf, sizeof(ImageHeader));
    }

	return rval;
}

/*
* read current identity.
*/
static int read_current_id(ImageHeader *id)
{
	int rval;
	char buf[512] = "";

    rval = read_nandflash("rawdata", buf, NAND_OFFSET_CURRENTID, SECTORSIZE);

    if(rval < 0) {
        printf("read current ID error!\n");
    } else {
        memcpy(id, buf, sizeof(ImageHeader));
    }

	return rval;
}

/*
* write current identity.
*/
static int write_current_id(ImageHeader *id)
{
	int rval;
	char buf[512] = "";

	memcpy(buf, id, sizeof(ImageHeader));

    rval = write_nandflash("rawdata", buf, NAND_OFFSET_CURRENTID, SECTORSIZE);

    if(rval < 0) {
        printf("write current ID error!\n");
    }

	return rval;
}

/*
* is firmware identified by the id valid, 1 - valid, 0 - invalid.
*/
static int is_valid(unsigned int id)
{
	int rval;
	char bootbuf[512] = "";
	char sysbuf[512] = "";

	ImageHeader *boothdr = (ImageHeader *)bootbuf;
	ImageHeader *syshdr = (ImageHeader *)sysbuf;

	if(id)
	{
	    rval = read_nandflash("rawdata", bootbuf, NAND_OFFSET_IMGHDR0_BOOTB, SECTORSIZE);
        rval |= read_nandflash("rawdata", sysbuf, NAND_OFFSET_IMGHDR0_SYSTEMB, SECTORSIZE);
	}
	else
	{
	    rval = read_nandflash("rawdata", bootbuf, NAND_OFFSET_IMGHDR0_BOOTA, SECTORSIZE);
        rval |= read_nandflash("rawdata", sysbuf, NAND_OFFSET_IMGHDR0_SYSTEMA, SECTORSIZE);
	}

    if(rval!=0){
        printf("read_nand flash rawdata error!\n");
    }

	if(GS_PROD_SIGNATURE!=boothdr->Signature)
	{
		printf("invalid boot hdr 0x%x\n", id);
		return 0;
	}
	if(GS_PROD_SIGNATURE!=syshdr->Signature)
	{
		printf("invalid system hdr 0x%x\n", id);
		return 0;
	}

	return 1;
}

/*
* show version information, for test.
*/
static int gvr35xx_version(int argc, char * const argv[])
{
	int rval;
	char ibuf[512] = "";
	ImageHeader current;
	ImageHeader lastupdate;
	ImageHeader *ihdr = (ImageHeader *)ibuf;
	unsigned int i;

	if(argc>2 && argv[2])
	{
		if(!strcmp(argv[2], "clean"))
		{
			for(i=0; i<ARRAY_SIZE(ihdr_dtbl) ; i++)
			{
				ihdr->Signature = 0;

                rval = write_nandflash("rawdata", ibuf, ihdr_dtbl[i].start, SECTORSIZE);

				if(rval<0)
				{
					printf("clean %s image header err\n", ihdr_dtbl[i].name);
				}
				else
				{
					printf("%s image header cleaned\n", ihdr_dtbl[i].name);
				}
			}
			return 0;
		}
	}

	memset(&current, 0, sizeof(current));
	memset(&lastupdate, 0, sizeof(lastupdate));

	printf("signature <0x%08x>\n", GS_PROD_SIGNATURE);
	rval = read_lastupdate_id(&lastupdate);
	if(rval<0)
	{
		printf("read LUID fail\n");
	}
	else
	{
		if(GS_PROD_SIGNATURE==lastupdate.Signature)
		{
			printf("LUID: 0x%08x\n", lastupdate.Id);
		}
		else
		{
			printf("LUID: invalid signature <0x%08x>\n", lastupdate.Signature);
		}
	}

	rval = read_current_id(&current);
	if(rval<0)
	{
		printf("read CUID fail\n");
	}
	else
	{
		if(GS_PROD_SIGNATURE==current.Signature)
		{
			printf("CUID: 0x%08x\n", current.Id);
		}
		else
		{
			printf("CUID: invalid signature <0x%08x>\n", current.Signature);
		}
	}

	for(i=0; i<ARRAY_SIZE(ihdr_dtbl) ; i++)
	{
		ihdr->Signature = 0;

        rval = read_nandflash("rawdata", ibuf, ihdr_dtbl[i].start, SECTORSIZE);

		if(rval<0)
		{
			printf("read %s image header err\n", ihdr_dtbl[i].name);
			continue;
		}

		if(GS_PROD_SIGNATURE!=ihdr->Signature)
		{
			printf("%s: invalid signature <0x%08x>\n", ihdr_dtbl[i].name, ihdr->Signature);
		}
		else
		{
			printf("%s: %u.%u.%u.%u\n", ihdr_dtbl[i].name, ihdr->Version>>24,
				(ihdr->Version>>16)&0xff, (ihdr->Version>>8)&0xff, ihdr->Version&0xff);
		}
	}

	return rval;
}

static int gvr3550_erase_rawdata(rawdata_part_t part)
{
	int i,ret = -1;

	int sec_num = 0;

	loff_t rawdata_offset = 0, rawdata_size = 0;
	char* tmpbuf;

	sunxi_partition_get_info_byname("rawdata", &rawdata_offset, &rawdata_size);
	if(!rawdata_offset || !rawdata_size) {
		sunxi_partition_get_info_byname("rawdata", &rawdata_offset, &rawdata_size);
		if(!rawdata_offset || !rawdata_size) {
			puts("no rawdata partition is found\n");
			return -1;
		}
	}

    switch(part) {
        case RAW_PART_NVRAM:
            rawdata_offset += (NAND_OFFSET_NVRAM>>9);
            sec_num = aligned_sector(MTD_SIZE_NVRAM);
            break;
        case RAW_PART_USERCFG:
            rawdata_offset += (NAND_OFFSET_USERCFG>>9);
            sec_num = aligned_sector(MTD_SIZE_USERCFG);
            break;
        default:
            printf("invalid rawdata part\n");
            return -1;
    }
	tmpbuf = (char*)malloc(sec_num<<9);
	memset((void*)tmpbuf, 0, sec_num<<9);

	rawdata_offset += (NAND_OFFSET_NVRAM>>9);

    ret = sunxi_flash_write(rawdata_offset, sec_num, (void *)tmpbuf);

	free(tmpbuf);
	return ret;
}

/*
* reset factory setting.
*/
static int gvr3550_factreset(int argc, char * const argv[])
{
	int rval = 0;
	const char *which;
	int i;

	if(argc<3) return -1;

#ifdef CONFIG_LCD_UC1701
    kgui_string("Factory Reset", 0);
    kgui_string("Don't Power OFF!", 2);
    fb_sync_region(0, 0, 128, 32);
#endif

	for(i=2; i<argc; i++)
	{
		which = argv[i];
		if (strcmp("ucfg", which) == 0)
		{
			rval = gvr3550_erase_rawdata(RAW_PART_USERCFG);
		}
        else if(strcmp("nvram", which) == 0)
		{
			rval = gvr3550_erase_rawdata(RAW_PART_NVRAM);
		}
		else if(strcmp("all", which) == 0)
		{
			rval = gvr3550_erase_rawdata(RAW_PART_NVRAM);
			rval |= gvr3550_erase_rawdata(RAW_PART_USERCFG);
		}
		else
		{
			rval = -1;
		}
		printf("erase %s: %s\n", which, rval?"fail":"success");
	}

#ifdef CONFIG_LCD_UC1701
    kgui_string("Factory Reset", 0);
    kgui_string("Finished!", 2);
    fb_sync_region(0, 0, 132, 32);
#endif

	return rval;
}

int gvr35xx_set_mac_addr(void)
{
    struct eth_device *eth_cur;
    struct gsGMACDeviceStruct *gmacdev;
    u8 mac_addr0[6] = DEFAULT_GS_MAC_ADDRESS;

    eth_cur = eth_get_dev();
    if (!eth_cur){
        printf("gvr35xx_set_mac_addr error. no ethe active\n");
    }
    gmacdev = container_of(eth_cur, struct gsGMACDeviceStruct, netdev);
    eth_getenv_enetaddr("ethaddr", mac_addr0);
    #ifdef CONFIG_SUN7I_GMAC
    gmac_clk_ctl(1);
    gs_gmac_set_mac_addr(gmacdev, GmacAddr0High, GmacAddr0Low, mac_addr0);
    #endif	
}

/*
* boot the normal system by id.
*/
static int gvr35xx_boot_normal(unsigned int id)
{
	int rval;
	unsigned long addr;
	char *laddr;
	char boot_cmd[20] = "";

	printf("boot normal 0x%x\n", id);
	laddr = getenv("loadaddr");
	if(laddr)
	{
		addr = simple_strtoul(laddr, NULL, 16);
	}
	else
	{
		addr = CONFIG_SYS_LOAD_ADDR;
	}
	if(id) {
		sprintf(boot_cmd, "sunxi_flash read %lx bootb", addr);
	} else {
		sprintf(boot_cmd, "sunxi_flash read %lx boota", addr);
	}

    rval = run_command(boot_cmd, 0);

	if(rval != -1)
	{
		gvr35xx_set_mac_addr();
		sprintf(boot_cmd, "bootm %lx", addr);
		run_command(boot_cmd, 0);
	}
	else
	{
		printf("run_command:%s error!\n", boot_cmd);
	}

	return rval;
}

/*
* boot the normal system by id.
*/
static int gvr35xx_boot_rescue()
{
	int rval;
	unsigned long addr;
	char *laddr;
	char boot_cmd[256] = "";
	char *bootargs;

	printf(">>>>>>>>>>>>boot rescue \n");
	laddr = getenv("loadaddr");
	if(laddr)
	{
		addr = simple_strtoul(laddr, NULL, 16);
	}
	else
	{
		addr = CONFIG_SYS_LOAD_ADDR;
	}

	sprintf(boot_cmd, "sunxi_flash read %lx bootr", addr);

	rval = run_command(boot_cmd, 0);

	if(rval != -1)
	{
		bootargs = getenv("bootargs");
		if(!bootargs)
		{
			sprintf(boot_cmd, "run setargs_nand");
			if(run_command(boot_cmd, 0) < 0)
			{
				printf("run setargs_nand error!\n");
				return 0;
			}
		}
		if((strlen(bootargs) + strlen("setenv bootargs  bootr=1") + 1) > 256)
		{
			printf("bootargs strings too long\n");
			return 0;
		}
		sprintf(boot_cmd, "setenv bootargs %s bootr=1", bootargs);
		if(run_command(boot_cmd, 0) < 0)
		{
			printf("run_command: %s error\n", boot_cmd);
			return 0;
		}

		sprintf(boot_cmd, "bootm %lx", addr);
		run_command(boot_cmd, 0);
	}
	else
	{
		printf("run_command:%s error!\n", boot_cmd);
	}
	return rval;
}

/*
* is power on reset, 0 -no, others -yes.
*/
static int is_pwr_on_rst(void)
{
     return 0;
}

/*
* is reset button pressed, 0 -no, other -yes.
*/
static int is_rst_btn_down(void)
{
    return 0;
}

/*
* power on reset and reset button is pressed boot recovery system
* if there is valid normal system, boot it, or boot recovery system.
*/
static int gvr35xx_boot(int argc, char * const argv[])
{
	int rval = 0;
	ImageHeader current;
	ImageHeader lastupdate;

	//gxe51xx_ptck();
	//gxe51xx_BCD_proc();
	if(!is_pwr_on_rst() || !is_rst_btn_down())
	{
		memset(&current, 0, sizeof(current));
		memset(&lastupdate, 0, sizeof(lastupdate));
		rval = read_lastupdate_id(&lastupdate);
		if(rval<0) printf("read LUID fail\n");
		rval = read_current_id(&current);
		if(rval<0) printf("read CUID fail\n");

		if(is_valid(lastupdate.Id))
		{
			if(memcmp(&current, &lastupdate, sizeof(lastupdate)))
			{
				write_current_id(&lastupdate);
			}
			gvr35xx_boot_normal(lastupdate.Id);
		}
		else if(is_valid(!lastupdate.Id))
		{
			lastupdate.Id = !lastupdate.Id;
			if(memcmp(&current, &lastupdate, sizeof(lastupdate)))
			{
				write_current_id(&lastupdate);
			}
			gvr35xx_boot_normal(lastupdate.Id);
		}
	}
    gvr35xx_boot_rescue();
	return 0;
}

/*
* set identity, for test.
*/
static int gvr35xx_setid(int argc, char * const argv[])
{
	int rval;
	char buf[512] = "";
	ImageHeader *hdr = (ImageHeader *)buf;
	int which;

	if(argc<4) return -1;
	which = simple_strtoul(argv[2], NULL, 16);
	hdr->Id = simple_strtoul(argv[3], NULL, 16);
	hdr->Signature = GS_PROD_SIGNATURE;
	if(which)
	{
	    rval = write_nandflash("rawdata", buf, NAND_OFFSET_LASTUPDID, SECTORSIZE);
	}
	else
	{
	    rval = write_nandflash("rawdata", buf, NAND_OFFSET_CURRENTID, SECTORSIZE);
	}

	if(rval<0) printf("setid fail\n");
	return 0;
}

/*
* custom u-boot command gvr35xx.
*/
static int do_gvr35xx(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int res = 1;
	int brightness = 10;
	int gpio = 49;
	int delay = 5;

	if (argc < 2)
		goto usage;

    if (strcmp("factreset", argv[1]) == 0)
		res = gvr3550_factreset(argc, argv);
	else if (strcmp("boot", argv[1]) == 0)
		res = gvr35xx_boot(argc, argv);
	else if (strcmp("setid", argv[1]) == 0)
		res = gvr35xx_setid(argc, argv);
	else if (strcmp("version", argv[1]) == 0)
		res = gvr35xx_version(argc, argv);

	if (res >= 0)
		return res;

usage:
	cmd_usage(cmdtp);
	return 1;
}

U_BOOT_CMD(gvr35xx, 4, 0, do_gvr35xx,
	"GVR35xx utilities",
	"factreset <which> ...	- factory reset, which: all ucfg nvram\n"
	"gvr35xx boot			    - custom boot for double backup\n"
	"gvr35xx setid <id> <val>	- set identity, id: 0-current 1-lastupdate\n"
	"                                           val: 0-orignal parttion 1-backup parttion\n"
	"gvr35xx version		- show firmware version\n"
	"gvr35xx version clean	- clean firmware version\n"
);

#endif

/* Validate a hex character */
__inline__ static int
_is_hex (char c)
{
  return (((c >= '0') && (c <= '9')) ||
	  ((c >= 'A') && (c <= 'F')) || ((c >= 'a') && (c <= 'f')));
}

int do_setmac (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int rcode = 0;
    GS_FACTORYPARA *factPara = (GS_FACTORYPARA *)factParaBuf;
    unsigned long long newMacAddr = 0;
    int inputLen, i;
	char mac_addr[32]="";
	char res;

	if (argc > 1) {
		printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}
    /* reload factory data before set mac, as factory data may be just loaded in */
     load_fact_para();

    if (factPara->mac[0] == 0xffff && factPara->mac[1] == 0xffff)
    {
		printf ("No factory config data found in flash.\n");
        return 1;
    }

    if (fact_para_checksum(factPara) != factPara->checksum)
    {
		printf ("No valid factory config data found in flash.\n");
        return 1;
    }
	
	inputLen = readline_into_buffer("Input MAC addr:", mac_addr);

    for (i = 0; i < inputLen; i++) {
		if (_is_hex(mac_addr[i]))
			continue;
		memmove(&mac_addr[i], &mac_addr[i+1], inputLen -i - 1);
		inputLen--;
	}
	mac_addr[inputLen] = '\0';
    printf("\n");
    if (inputLen == 6)
    {
	    newMacAddr = simple_strtoull(mac_addr, NULL, 16);
    	factPara->mac[0] = 0x0b00;
    	factPara->mac[1] = 0x0082;
    	factPara->mac[1] = (factPara->mac[1] & 0x00FF) | ((newMacAddr >> 16) & 0xFF) << 8;
    	factPara->mac[2] = ((newMacAddr >> 8) & 0xFF) | ((newMacAddr) & 0xFF) << 8;
    }
    else if (inputLen == 12)
    {
	    newMacAddr = simple_strtoull(mac_addr, NULL, 16);
    	factPara->mac[0] = ((newMacAddr >> 40) & 0xFF) | ((newMacAddr >> 32) & 0xFF) << 8;
    	factPara->mac[1] = ((newMacAddr >> 24) & 0xFF) | ((newMacAddr >> 16) & 0xFF) << 8;
    	factPara->mac[2] = ((newMacAddr >> 8) & 0xFF) | ((newMacAddr) & 0xFF) << 8;
    }
    else
    {
		printf ("Invalid MAC address format.\n");
        return 1;
    }

    printf("MAC addr set to: %02X:%02X:%02X:%02X:%02X:%02X\n",
        factPara->mac[0] & 0xFF, (factPara->mac[0] >> 8) & 0xFF,
        factPara->mac[1] & 0xFF, (factPara->mac[1] >> 8) & 0xFF,
        factPara->mac[2] & 0xFF, (factPara->mac[2] >> 8) & 0xFF);

    factPara->checksum = fact_para_checksum(factPara);
    printf("Chksum is 0x%x\n", factPara->checksum);

    rcode = save_fact_para(factParaBuf);
    if (rcode)
    {
        printf("Write factory data failed\n");
    }

	sprintf(mac_addr, "%02X:%02X:%02X:%02X:%02X:%02X",
        factPara->mac[0] & 0xFF, (factPara->mac[0] >> 8) & 0xFF,
        factPara->mac[1] & 0xFF, (factPara->mac[1] >> 8) & 0xFF,
        factPara->mac[2] & 0xFF, (factPara->mac[2] >> 8) & 0xFF);
	setenv("ethaddr", mac_addr);

	return rcode;
}

void gxv32xx_set_mac(void)
{
	char mac_addr[32]="";
    struct eth_device *eth_cur;
    struct gsGMACDeviceStruct *gmacdev;
	u8 mac[6] = DEFAULT_GS_MAC_ADDRESS;
    GS_FACTORYPARA *factPara = (GS_FACTORYPARA *)factParaBuf;

	if (load_fact_para() >= 0)
		memcpy(mac, factPara->mac, sizeof(mac));
	
	sprintf(mac_addr, "%02X:%02X:%02X:%02X:%02X:%02X",
        mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
	setenv("ethaddr", mac_addr);

    eth_cur = eth_get_dev();
    if (eth_cur && eth_cur->state == ETH_STATE_ACTIVE){
        printf("gxv32xx_set_mac error. no ethe active\n");
    }
    gmacdev = container_of(eth_cur, struct gsGMACDeviceStruct, netdev);
    #ifdef CONFIG_SUN7I_GMAC
    gmac_clk_ctl(1);
    gs_gmac_set_mac_addr(gmacdev, GmacAddr0High, GmacAddr0Low, mac);
    #endif

	return;
}

U_BOOT_CMD(
    confmac,   1,  1,   do_setmac,
    "setmac - program MAC addr for GXV32xx/GVR35xx devices\n",
    "input xxxxxx or xxxxxxxxxxxx - last 6 digit or whole 12 digit of MAC, 00:0b:82:xx:xx:xx\n"
);


/*add by dean start*/
extern unsigned char  udpmac_buffer[6];
int setudpmac (void)
{
	int rcode = 0;
	char mac_addr[32]="";
    GS_FACTORYPARA *factPara = (GS_FACTORYPARA *)factParaBuf;

    /* reload factory data before set mac, as factory data may be just loaded in */
     load_fact_para();

    printf("factPara.mac[0] = %04x\n", factPara->mac[0]);
    printf("factPara.mac[1] = %04x\n", factPara->mac[1]);
    
    if (factPara->mac[0] == 0xffff && factPara->mac[1] == 0xffff)
    {
		printf ("No factory config data found in flash.\n");
        return 1;
    }

    if (fact_para_checksum(factPara) != factPara->checksum)
    {
		printf ("No valid factory config data found in flash.\n");
        	return 1;
    }
   
    factPara->mac[0] = (udpmac_buffer[1]<<8)|(udpmac_buffer[0]);
    factPara->mac[1] = (udpmac_buffer[3]<<8)|(udpmac_buffer[2]);
    factPara->mac[2] = (udpmac_buffer[5]<<8)|(udpmac_buffer[4]);
    
    printf("MAC addr set to: %02X:%02X:%02X:%02X:%02X:%02X\n",
        factPara->mac[0] & 0xFF, (factPara->mac[0] >> 8) & 0xFF,
        factPara->mac[1] & 0xFF, (factPara->mac[1] >> 8) & 0xFF,
        factPara->mac[2] & 0xFF, (factPara->mac[2] >> 8) & 0xFF);
        
    factPara->checksum = fact_para_checksum(factPara);
    printf("Chksum is 0x%x\n", factPara->checksum);

    rcode = save_fact_para(factParaBuf);
    if (rcode)
    {
        printf("Write factory data failed\n");
        return 1;
    }

	sprintf(mac_addr, "%02X:%02X:%02X:%02X:%02X:%02X",
        factPara->mac[0] & 0xFF, (factPara->mac[0] >> 8) & 0xFF,
        factPara->mac[1] & 0xFF, (factPara->mac[1] >> 8) & 0xFF,
        factPara->mac[2] & 0xFF, (factPara->mac[2] >> 8) & 0xFF);
	setenv("ethaddr", mac_addr);

	return rcode;
}
int getudpmac (unsigned char *mac)
{
	int rcode = 0;

    GS_FACTORYPARA *factPara = (GS_FACTORYPARA *)factParaBuf;

    /* reload factory data before set mac, as factory data may be just loaded in */
     load_fact_para();

    printf("factPara.mac[0] = %04x\n", factPara->mac[0]);
    printf("factPara.mac[1] = %04x\n", factPara->mac[1]);
    
    if (factPara->mac[0] == 0xffff && factPara->mac[1] == 0xffff)
    {
		printf ("No factory config data found in flash.\n");
        return 1;
    }

    if (fact_para_checksum(factPara) != factPara->checksum)
    {
		printf ("No valid factory config data found in flash.\n");
        	return 1;
    }

	mac[0]=factPara->mac[0] & 0xFF;
	mac[1]=(factPara->mac[0] >> 8) & 0xFF;
	mac[2]=factPara->mac[1] & 0xFF;
	mac[3]=(factPara->mac[1] >> 8) & 0xFF;
	mac[4]=factPara->mac[2] & 0xFF;
	mac[5]=(factPara->mac[2] >> 8) & 0xFF;

	 printf("MAC addr : %02X:%02X:%02X:%02X:%02X:%02X\n",
        mac[0], mac[1],
        mac[2], mac[3],
        mac[4], mac[5]);
	return 0;
}
int get_cfg_pn( unsigned char * pn )
{
     int rcode = 0;

    GS_FACTORYPARA *factPara = (GS_FACTORYPARA *)factParaBuf;

    /* reload factory data before set mac, as factory data may be just loaded in */
     load_fact_para();

    printf("factPara.mac[0] = %04x\n", factPara->mac[0]);
    printf("factPara.mac[1] = %04x\n", factPara->mac[1]);
    
    if (factPara->mac[0] == 0xffff && factPara->mac[1] == 0xffff)
    {
		printf ("No factory config data found in flash.\n");
        return 1;
    }

    if (fact_para_checksum(factPara) != factPara->checksum)
    {
		printf ("No valid factory config data found in flash.\n");
        	return 1;
    }

      pn[0]=factPara->PartNumber[0]&0xFF;
	pn[1]=(factPara->PartNumber[0] >> 8) & 0xFF;
	pn[2]=factPara->PartNumber[1] & 0xFF;
	pn[3]=(factPara->PartNumber[1] >> 8) & 0xFF;
	pn[4]=factPara->PartNumber[2] & 0xFF;
	pn[5]=(factPara->PartNumber[2] >> 8) & 0xFF;

	pn[6]=factPara->PartNumber[3]&0xFF;
	pn[7]=(factPara->PartNumber[3] >> 8) & 0xFF;
	pn[8]=factPara->PartNumber[4] & 0xFF;
	pn[9]=(factPara->PartNumber[4] >> 8) & 0xFF;
	pn[10]=factPara->PartNumber[5] & 0xFF;
	pn[11]=(factPara->PartNumber[5] >> 8) & 0xFF;

	printf ("PN:%s\n",pn);
    return 0;
}

/*add by dean end*/
#if 0
int do_setmac (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    GS_FACTORYPARA *factPara = (GS_FACTORYPARA *)factParaBuf;
	unsigned char mac[6];
	char mac_addr[32]="";
	int rcode;

	if (argc != 2) {
		printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}
    /* reload factory data before set mac, as factory data may be just loaded in */
    load_fact_para();

    if (fact_para_checksum(factPara) != factPara->checksum)
    {
		printf ("No valid factory config data found in flash.\n");
        return 1;
    }
	eth_parse_enetaddr(argv[1], mac);
	factPara->mac[0] = (0xff&mac[0]) | (0xff00&(mac[1] << 8));
	factPara->mac[1] = (0xff&mac[2]) | (0xff00&(mac[3] << 8));
	factPara->mac[2] = (0xff&mac[4]) | (0xff00&(mac[5] << 8));
    
	sprintf(mac_addr, "%02X:%02X:%02X:%02X:%02X:%02X",
        factPara->mac[0] & 0xFF, (factPara->mac[0] >> 8) & 0xFF,
        factPara->mac[1] & 0xFF, (factPara->mac[1] >> 8) & 0xFF,
        factPara->mac[2] & 0xFF, (factPara->mac[2] >> 8) & 0xFF);
    printf("Mac address set to:%s\n", mac_addr);
    factPara->checksum = fact_para_checksum(factPara);
    printf("Chksum is 0x%x\n", factPara->checksum);
    
	rcode = save_fact_para(factParaBuf);
    if (rcode)
    {
        printf("Write factory data failed\n");
    }
	setenv("ethaddr", mac_addr);

	return rcode;
}

U_BOOT_CMD_COMPLETE(
    confmac,   2,  1,   do_setmac,
    "setmac - set MAC addr for GXV3240/GXV3275 devices\n",
	"confmac macaddr\n"
	"	- macaddr format should be xx:xx:xx:xx:xx",
	var_complete
);
#endif
