/* ---------------------------------------------------------------------------
*
*    File Name:  cmd_udpmac.c
*        Model:  UdpMac Functional
*
*         Email:  dawu@grandstream.com
*    Company:  GrandStream Technology, Inc.
*
*  Description:  Burn MAC, Check Mac and Version via UDP protocol
*
*
*
* Rev  Author          Date       		Changes
* ---  --------------- ----------  -------------------------------
* 1.0      dean          7/3/2013  - 	First release of new model capable of 
*                                     		supporting GXP2140/GXP2160
*
* 1.1      dean          10/08/2013  - 	solve the problem: when running udpmac will 
*								cause the GXP2140/GXP2160 freeze on "UdpmacStart...."
*								"NetSendUDPPacket()" function first parameter must be set
*								to "NetServerEther", or will cause that issue.
*
*								Activate UdpMac method: change double keys :"3"+"7" 
*								to single key "5". It can be esay for worker.
*								
*
*
*
--------------------------------------------------------------------------- */

#include <common.h>
#include <asm/arch/android_misc.h>
#include <command.h>
#include <asm/arch/partition.h>
#include <config.h>
#include <common.h>
#include <nand.h>
#include <bmp_layout.h>
#include <lcd.h>
#include <mmc.h>

#include <common.h>
#include <asm/io.h>
#include <fastboot.h>
#include <exports.h>

#include <asm/arch/nand_bsp.h>
#include <mmc.h>
#include <asm/arch/android_misc.h>
#include <asm/arch/gpio.h>
#include <asm/arch/partition.h>
#include <asm/arch/boot_type.h>
#include <asm/arch/sys_config.h>
#include <i2c.h>
#include <asm/arch/mmc.h>


unsigned char  udpmac_buffer[6]={0x00,0x0B,0x82,0x00,0x00,0x00};

extern int setudpmac (void);
extern int getudpmac (unsigned char *mac);
extern int get_cfg_pn( unsigned char * pn );

/**********************************************************************/
IPaddr_t MacServerIP;
int MacServerPort=2002;
int MacTimeout=1;
int MacTimeoutMax=10;
static int udpmac_state=0;//connect 
static int mac_state=0;//state

unsigned int ascii2Hex(char c)
{
	unsigned int hex=0;
	if((c>='0')&&(c<='9'))
		hex = c - '0';
	if((c>='a')&&(c<='f'))
		hex = c - 'a' + 10;
	if((c>='A')&&(c<='F'))
		hex = c - 'A' + 10;

	return hex;
}
unsigned char Hex2ascii(int data)
{
	unsigned char chartmp='0';
	if(data >= 0 && data < 10)
		chartmp = data + '0';
	if(data >=10 && data < 16 )
		chartmp = (data-10) +'A';
	return chartmp;
}

static void ms_delay(int i)
{
	int j;
	int k=0;
	for(j=0;j<i;j++)
	{
		udelay(1000);
	}
}
static void UdpmacTimeout(void);
static void UdpmacSend_start(void)
{
	volatile uchar *pkt;
	volatile uchar *xp;
	int		len = 0;
      uchar cmd_start[6]={0xAA,0x55,0xFF,0x55,0xFF,0xF9};
      
	int ourport = 1024 + (get_timer(0) % 3072);
	pkt = NetTxPacket + NetEthHdrSize() + IP_HDR_SIZE;

			xp = pkt;
			strcpy((char *)pkt, cmd_start);
			pkt += 6 ;/*strlen("File has bad magic")*/
			len = pkt - xp;
			MacServerPort=2002;
	
	NetSendUDPPacket(NetServerEther, MacServerIP, MacServerPort,ourport, len);
}
static void UdpmacSend_ack(void)
{
	volatile uchar *pkt;
	volatile uchar *xp;
	int		len = 0;
      uchar cmd_ack[6]={0xAA,0x55,0xFF,0x55,0xFF,0xFB};
      
	int ourport = 1024 + (get_timer(0) % 3072);
	pkt = NetTxPacket + NetEthHdrSize() + IP_HDR_SIZE;

			xp = pkt;
			strcpy((char *)pkt, cmd_ack);
			pkt += 6 ;/*strlen("File has bad magic")*/
			len = pkt - xp;
			MacServerPort=2000;
	
	NetSendUDPPacket(NetServerEther, MacServerIP, MacServerPort,ourport, len);
}
static void UdpmacSend_ok(void)
			{
	volatile uchar *pkt;
	volatile uchar *xp;
	int		len = 0;
      uchar cmd_ok[6]={0xAA,0x55,0xFF,0x55,0xFF,0xFF};
      
	int ourport = 1024 + (get_timer(0) % 3072);
	pkt = NetTxPacket + NetEthHdrSize() + IP_HDR_SIZE;

	xp = pkt;
				strcpy((char *)pkt, cmd_ok);
	pkt += 6 ;/*strlen("File has bad magic")*/
	len = pkt - xp;
	MacServerPort=2000;
	
	NetSendUDPPacket(NetServerEther, MacServerIP, MacServerPort,ourport, len);
}
static void UdpmacSend_fail(void)
{
	volatile uchar *pkt;
	volatile uchar *xp;
	int		len = 0;
      uchar cmd_fail[6]={0xAA,0x55,0xFF,0x55,0xFF,0xFE};
      
	int ourport = 1024 + (get_timer(0) % 3072);
	pkt = NetTxPacket + NetEthHdrSize() + IP_HDR_SIZE;

	xp = pkt;
	strcpy((char *)pkt, cmd_fail);
			pkt += 6 ;/*strlen("File has bad magic")*/
			len = pkt - xp;
			MacServerPort=2000;
	
	NetSendUDPPacket(NetServerEther, MacServerIP, MacServerPort,ourport, len);
}
static void udpsend_ok_mac(void)
{
	int ret=0;
	volatile uchar *pkt;
	volatile uchar *xp;
	int		len = 0;
      uchar cmd_mac[12]={0xAA,0x55,0xFF,0x55,0xFF,0xFF,0xff,0xff,0xff,0xff,0xff,0xff};
	int ourport = 1024 + (get_timer(0) % 3072);
	
      
	getudpmac(&cmd_mac[6]);
	
	
	printf("udpsend_ok_mac\n");
	pkt = NetTxPacket + NetEthHdrSize() + IP_HDR_SIZE;
	xp = pkt;
	//strcpy((char *)pkt, cmd_mac); //have some error
	pkt[0]=cmd_mac[0];
	pkt[1]=cmd_mac[1];
	pkt[2]=cmd_mac[2];
	pkt[3]=cmd_mac[3];
	pkt[4]=cmd_mac[4];
	pkt[5]=cmd_mac[5];
	
	pkt[6]=Hex2ascii((cmd_mac[6]&0xf0)>>4);
	pkt[7]=Hex2ascii((cmd_mac[6]&0x0f));
	pkt[8]=Hex2ascii((cmd_mac[7]&0xf0)>>4);
	pkt[9]=Hex2ascii((cmd_mac[7]&0x0f));
	pkt[10]=Hex2ascii((cmd_mac[8]&0xf0)>>4);
	pkt[11]=Hex2ascii((cmd_mac[8]&0x0f));
	pkt[12]=Hex2ascii((cmd_mac[9]&0xf0)>>4);
	pkt[13]=Hex2ascii((cmd_mac[9]&0x0f));
	pkt[14]=Hex2ascii((cmd_mac[10]&0xf0)>>4);
	pkt[15]=Hex2ascii((cmd_mac[10]&0x0f));
	pkt[16]=Hex2ascii((cmd_mac[11]&0xf0)>>4);
	pkt[17]=Hex2ascii((cmd_mac[11]&0x0f));
	
	pkt += 18 ;/*strlen("File has bad magic")*/
	len = pkt - xp;
	MacServerPort=2000;
	
	NetSendUDPPacket(NetServerEther, MacServerIP, MacServerPort,ourport, len);
}

char pn_tmp[20];
char uboot_tmp[20];
char kernel_tmp[20];
char system_tmp[20];
char rcvr_tmp[20];

extern int read_provision_version(int ii, char * version_tmp);
static void udpsend_version_new(void)
{
	int i,count=0;
	volatile uchar *pkt;
	volatile uchar *xp;
	int		len = 0;
      uchar serverether[6];
      char version_tmp[20];
      
	int ourport = 1024 + (get_timer(0) % 3072);
	memset(serverether, 0, 6);
	
	pkt = NetTxPacket + NetEthHdrSize() + IP_HDR_SIZE;
      printf("udpsend_version_new\n");
	xp = pkt;

	pkt[0] = 0xAA; 
      pkt[1] = 0x55; 
      pkt[2] = 0xFF; 
      pkt[3] = 0x55; 
      pkt[4] = 0xFF; 
      pkt[5] = 0x42; 
	pkt += 6 ;/*strlen("File has bad magic")*/
	count=strlen(pn_tmp);
	//printf("1 udpsend_version:%d\n",count);
	if(count>0)
	{
		for(i=0;i<20;i++)
		{
			pkt[i]=pn_tmp[i];
		}
		pkt += 20;
	}
	else
	{
		pkt[0] = '0';
	      pkt[1] = '0'; 
	      pkt[2] = '0'; 
	      pkt[3] = '0'; 
	      pkt[4] = '0'; 
	      pkt[5] = '0'; 
	      pkt[6] = '0'; 
	      pkt[7] = '0'; 
	      pkt[8] = '0'; 
	      pkt[9] = '0'; 
	      pkt[10] = '0'; 
	      pkt[11] = '0'; 
	      for(i=12;i<20;i++)
		{
			pkt[i]=0;
		}
		pkt += 20 ;
	}
	
      pkt[0] = 0xAA; 
      pkt[1] = 0x55; 
      pkt[2] = 0xFF; 
      pkt[3] = 0x55; 
      pkt[4] = 0xFF; 
      pkt[5] = 0x44; 
	pkt += 6 ;/*strlen("File has bad magic")*/
	count=strlen(uboot_tmp);
	//printf("1 udpsend_version:%d\n",count);
	if(count>0)
	{
		for(i=0;i<20;i++)
		{
			pkt[i]=uboot_tmp[i];
		}
		pkt += 20;
	}
	else
	{
		pkt[0] = '0';
	      pkt[1] = '.'; 
	      pkt[2] = '0'; 
	      pkt[3] = '.'; 
	      pkt[4] = '0'; 
	      pkt[5] = '.'; 
	      pkt[6] = '0'; 
	      for(i=7;i<20;i++)
		{
			pkt[i]=0;
		}
		pkt += 20 ;
	}

	pkt[0] = 0xAA; 
      pkt[1] = 0x55; 
      pkt[2] = 0xFF; 
      pkt[3] = 0x55; 
      pkt[4] = 0xFF; 
      pkt[5] = 0x45; 
	pkt += 6 ;
	count=strlen(kernel_tmp);
	//printf("2 udpsend_version:%d\n",count);
	if(count>0)
	{
		for(i=0;i<20;i++)
		{
			pkt[i]=kernel_tmp[i];
		}
		pkt += 20;
	}
	else
	{
		pkt[0] = '0';
	      pkt[1] = '.'; 
	      pkt[2] = '0'; 
	      pkt[3] = '.'; 
	      pkt[4] = '0'; 
	      pkt[5] = '.'; 
	      pkt[6] = '0'; 
	      for(i=7;i<20;i++)
		{
			pkt[i]=0;
		}
		pkt += 20 ;
	}

	pkt[0] = 0xAA; 
      pkt[1] = 0x55; 
      pkt[2] = 0xFF; 
      pkt[3] = 0x55; 
      pkt[4] = 0xFF; 
      pkt[5] = 0x46; 
	pkt += 6 ;/*strlen("File has bad magic")*/
	count=strlen(system_tmp);
	//printf("4 udpsend_version:%d\n",count);
	if(count>0)
	{
		for(i=0;i<20;i++)
		{
			pkt[i]=system_tmp[i];
		}
		pkt += 20;
	}
	else
	{
		pkt[0] = '0';
	      pkt[1] = '.'; 
	      pkt[2] = '0'; 
	      pkt[3] = '.'; 
	      pkt[4] = '0'; 
	      pkt[5] = '.'; 
	      pkt[6] = '0'; 
	      for(i=7;i<20;i++)
		{
			pkt[i]=0;
		}
		pkt += 20 ;
	}

	pkt[0] = 0xAA; 
      pkt[1] = 0x55; 
      pkt[2] = 0xFF; 
      pkt[3] = 0x55; 
      pkt[4] = 0xFF; 
      pkt[5] = 0x4B; 
	pkt += 6 ;/*strlen("File has bad magic")*/
	count=strlen(rcvr_tmp);
	//printf("5 udpsend_version:%d\n",count);
	if(count>0)
	{
		for(i=0;i<20;i++)
		{
			pkt[i]=rcvr_tmp[i];
		}
		pkt += 20;
	}
	else
	{
		pkt[0] = '0';
	      pkt[1] = '.'; 
	      pkt[2] = '0'; 
	      pkt[3] = '.'; 
	      pkt[4] = '0'; 
	      pkt[5] = '.'; 
	      pkt[6] = '0'; 
	      for(i=7;i<20;i++)
		{
			pkt[i]=0;
		}
		pkt += 20 ;
	}
	
	len = pkt - xp;
	MacServerPort=2000;
	NetSendUDPPacket(NetServerEther, MacServerIP, MacServerPort,
			 ourport, len);
}

static void UdpmacHandler(uchar *pkt, unsigned dest, IPaddr_t sip, unsigned src,
	    unsigned len)
{
	uchar *pt=pkt;
	int i;
	int count=strlen(pkt);
	printf("\nrecevice: %d ==>> ",len);
	for(i=0;i<count;i++)
	{
		printf("0x%02X ",pkt[i]);
	}
	printf("\n");
	if((pt[0]==0xAA)&&(pt[1]==0x55)&&(pt[2]==0xFF)&&(pt[3]==0x55))
	{
		if((pt[4]==0xFF)&&(pt[5]==0xFA)) //start ack
		{
			udpmac_state=1;
			printf("start ack\n");
			ms_delay(1500);
			UdpmacSend_ack();
		}
		if((pt[4]==0xFF)&&(pt[5]==0xA9)) //mac write
		{
			printf("network mac : ");
			for(i=6;i<12;i++)
			{
				printf("%02X ",pkt[i]);
			}
			printf("\n");
			udpmac_buffer[0]=pkt[6];
			udpmac_buffer[1]=pkt[7];
			udpmac_buffer[2]=pkt[8];
			udpmac_buffer[3]=pkt[9];
			udpmac_buffer[4]=pkt[10];
			udpmac_buffer[5]=pkt[11];
			ms_delay(1500);
			if(setudpmac())
			{
				mac_state=1;
				UdpmacSend_fail();//mac fail
			}
			else
			{
				mac_state=0;
				UdpmacSend_ok();//mac ok
			}
		}
		if((pt[4]==0xFF)&&(pt[5]==0xFC)) //finish over
		{
			if(mac_state==1)
			{
				NetState = NETLOOP_FAIL;
			}
			else
			{
				NetState = NETLOOP_SUCCESS;
			}
		}
		if((pt[4]==0xFF)&&(pt[5]==0xB9)) //mac read
		{
			printf("send mac\n");
			ms_delay(1500);
			udpsend_ok_mac();
		}
		if((pt[4]==0xFF)&&(pt[5]==0x4F)) //version read
		{
			ms_delay(1500);
			printf("send version\n");
			memset(pn_tmp, 0, 20);
			memset(uboot_tmp, 0, 20);
			memset(kernel_tmp, 0, 20);
			memset(system_tmp, 0, 20);
			memset(rcvr_tmp, 0, 20);
			get_cfg_pn(pn_tmp);
			read_provision_version(2,uboot_tmp);
			read_provision_version(3,kernel_tmp);
			read_provision_version(4,system_tmp);
			read_provision_version(5,rcvr_tmp);
			udpsend_version_new();
		}
		if((pt[4]==0xFC)&&(pt[5]==0xFC)) //cfg write
		{
		}
	}
}


static void
UdpmacTimeout(void)
{
	if(++MacTimeout>MacTimeoutMax)
	{
		NetState = NETLOOP_FAIL;
	}
	if(udpmac_state==0)
	{
		printf("overtime connect again ......\n");
		NetSetTimeout(1000UL, UdpmacTimeout);
		UdpmacSend_start();
	}
	else
	{
		NetSetTimeout(100000UL, UdpmacTimeout);
	}
}


void
UdpmacStart(void)
{
	MacTimeout=1;
	udpmac_state=0;
	memset(NetServerEther, 0, 6);
	printf("UdpmacStart  ......\n");
	NetSetTimeout(1000UL, UdpmacTimeout);
	NetSetHandler(UdpmacHandler);
	UdpmacSend_start();
}


static int do_check_udpmac(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret0=1;
	unsigned int reg_val=1;
	char mac[6];
	
	memset(mac, 0, 6);
	gpio_direction_output(SUNXI_GPG(4),1);//red led off
	gpio_direction_output(SUNXI_GPG(5),1);//green led off
	//reg_val = gpio_direction_input(SUNXI_GPH(6));//cam_switch pin
	reg_val = gpio_direction_input(SUNXI_GPH(0));//hook_switch pin
	if(reg_val==0)
      {
      		//printf("Cam_Switch OFF\n");
      		printf("Hook_Switch OFF\n");
      		ret0=getudpmac (mac);
      		if(ret0==1)//no cfg in the flash
		{
			gpio_direction_output(SUNXI_GPG(4),0);//red led on
			setenv("serverip", "192.168.0.161");
			setenv("ipaddr", "192.168.0.160");
			setenv("ethaddr", "00:0B:82:00:00:00");

			run_command("run load_factcfg", 0);
			gpio_direction_output(SUNXI_GPG(4),1);//red led off
		}
		else if( (mac[3]==0x00 && mac[4]==0x00 && mac[5]==0x00) ||  (mac[3]==0xff && mac[4]==0xff && mac[5]==0xff))
		{
			run_command("udpmac", 0);
		}
      }
      else
      {
      		//printf("Cam_Switch ON\n");
      		//printf("Hook_Switch ON\n");
      }
	return 1;
}

U_BOOT_CMD(check_udpmac, 1, 0, do_check_udpmac,
	"input/set/clear/toggle gpio pins",
	"<input|set|clear|toggle> <pin>\n"
	"    - input/set/clear/toggle the specified pin");

	
