/*  sun7i_gmac.c: Version 1.0 2013.06.07

        GMAC Ethernet driver for u-boot.

        This program is free software; you can redistribute it and/or
        modify it under the terms of the GNU General Public License
        as published by the Free Software Foundation; either version 2
        of the License, or (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.


  (C)Copyright 2013 Grandstream. All Rights Reserved.
        xchen@grandstream.com

*/

#include <common.h>
#include <asm/io.h>
#include <miiphy.h>
#include "sun7i_gmac.h"
#include "sun7i_phy.h"
#include "malloc.h"
gs_gmac_device gmac_device;
gs_gmac_device* g_gsgmacdev = &gmac_device;

unsigned char *rx_buff = NULL;
unsigned char *rx_buff_org = NULL;
dma_desc *tx_desc_org = NULL;
dma_desc *rx_desc_org = NULL;

#define MALLOC_ALIGN(P) ((unsigned char *)( ( (unsigned long)P+0x1000 ) & 0xfffff000))

static s32 gs_gmac_setup_rx_desc_queue(gs_gmac_device *gmacdev ,u32 no_of_desc)
{
	s32 i;

	dma_desc *first_desc = NULL;
	gmacdev->RxDescCount = 0;

	gs_debug2("total size of memory required for Rx Descriptors in Ring Mode = 0x%08x\n",((sizeof(dma_desc) * no_of_desc)));
    first_desc = (dma_desc *)malloc (sizeof (dma_desc) * no_of_desc + 0x1000);
	if (first_desc == NULL){
		gs_debug0("Error in Tx Descriptors memory allocation\n");
		return -1;
	}

    rx_desc_org = first_desc;
    first_desc = (dma_desc *) MALLOC_ALIGN (first_desc);

	gmacdev->RxDescCount = no_of_desc;
	gmacdev->RxDesc      = first_desc;

	gs_debug2("gmacdev->RxDesc = 0x%x \n",(unsigned int)gmacdev->RxDesc );

	for (i = 0; i < gmacdev->RxDescCount; i++) {
		gs_gmac_rx_desc_init_ring(gmacdev->RxDesc + i, (i == gmacdev->RxDescCount-1) ? 1 : 0);
		gs_debug2("%02d %08x \n",i, (unsigned int)(gmacdev->RxDesc + i));
	}

	gmacdev->RxNext = 0;
	gmacdev->RxBusy = 0;
	gmacdev->RxNextDesc = gmacdev->RxDesc;
	gmacdev->RxBusyDesc = gmacdev->RxDesc;

	gmacdev->BusyRxDesc   = 0;

	return 0;
}

static s32 gs_gmac_setup_tx_desc_queue(gs_gmac_device *gmacdev, u32 no_of_desc)
{
	s32 i;
	dma_desc *first_desc = NULL;
	gmacdev->TxDescCount = 0;

	gs_debug2("Total size of memory required for Tx Descriptors in Ring Mode = 0x%08x\n",((sizeof(dma_desc) * no_of_desc)));

    first_desc = (dma_desc *)malloc (sizeof (dma_desc) * no_of_desc + 0x1000);
	if (first_desc == NULL){
		gs_debug0("Error in Tx Descriptors memory allocation\n");
		return -1;
	}

    tx_desc_org = first_desc;
    first_desc = (dma_desc *) MALLOC_ALIGN (first_desc);
	gmacdev->TxDescCount = no_of_desc;
	gmacdev->TxDesc      = first_desc;

	gs_debug2("gmacdev->TxDesc = 0x%x \n",(unsigned int)gmacdev->TxDesc);

	for (i = 0; i < gmacdev->TxDescCount; i++) {
		gs_gmac_tx_desc_init_ring(gmacdev->TxDesc + i, (i == gmacdev->TxDescCount-1) ? 1 : 0);
		gs_debug2("%02d %08x \n",i, (unsigned int)(gmacdev->TxDesc + i));
	}

	gmacdev->TxNext = 0;
	gmacdev->TxBusy = 0;
	gmacdev->TxNextDesc = gmacdev->TxDesc;
	gmacdev->TxBusyDesc = gmacdev->TxDesc;
	gmacdev->BusyTxDesc = 0;

	return 0;
}

static s32 gs_gmac_init_rx_qptr(gs_gmac_device *gmacdev)
{
	unsigned int i = 0;
    u32 dmaaddr = 0;

	rx_buff = (unsigned char *)malloc (RX_BUFFER_LEN * RECEIVE_DESC_SIZE + 0x1000);
	if (rx_buff)
	{
		rx_buff_org = rx_buff;
		rx_buff = MALLOC_ALIGN (rx_buff);
	}
	do 
    {

        dmaaddr = (u32)(rx_buff + RX_BUFFER_LEN * i);
		gs_gmac_set_rx_qptr(gmacdev, dmaaddr, RX_BUFFER_LEN, 0,0,0,0);

		i++;
	} while (i < RECEIVE_DESC_SIZE);

	return 0;
}
//************************************************add*********************************************
void core_dump_regs(void)
{
#if 0
	int i;
	printf("\tDWMAC1000 regs (base addr = 0x%p)\n", g_gsgmacdev->MacBase);

	for (i = 0; i < 55; i++) {
		int offset = i * 4;
		printf("\tReg No. %d (offset 0x%x): 0x%08x\n", i,
			offset, readl(g_gsgmacdev->MacBase + offset));

	}
#endif
}

void dma_dump_regs(void)
{
#if 0
	int i;
	printf(" DMA registers\n");
	for (i = 0; i < 22; i++) {
		if ((i < 9) || (i > 17)) {
			int offset = i * 4;
			printf("\t Reg No. %d (offset 0x%x): 0x%08x\n", i,
			       (DmaBusMode + offset),
			       readl(g_gsgmacdev->DmaBase + DmaBusMode + offset));

		}
	}
#endif
}

//************************************************add down*********************************************
static int gmac_init(struct eth_device *netdev, bd_t *bd)
{
    int ret = -1;
	gs_gmac_device *gmacdev = to_gmac(netdev);
    u8 mac_addr0[6] = DEFAULT_MAC_ADDRESS;

    gmac_clk_ctl(1); 

    gmac_mdio_reset();

    gmac_mdio_init();

    ret = gs_gmac_scan_phy(gmacdev);
    if(ret)
    {
		gs_debug0("No availble PHY was found, please check network connection\n");
		return -1;
    }

	/*Soft reset*/
	gs_gmac_write_phy_reg(gmacdev->MacBase, gmacdev->PhyBase, PHY_CONTROL_REG, 1 << 15);
	udelay(1000);
    
	gs_gmac_dma_bus_mode_init();

    ret = gs_gmac_check_phy_init (gmacdev);
    if(ret)
    {
		gs_debug0("phy init failded\n");
        return -1;
    }

	/* Set up the tx and rx descriptor queue/ring */
	gs_gmac_setup_tx_desc_queue(gmacdev, TRANSMIT_DESC_SIZE);
	gs_gmac_init_tx_desc_base(gmacdev);	//Program the transmit descriptor base address in to DmaTxBase addr
	gs_gmac_setup_rx_desc_queue(gmacdev, RECEIVE_DESC_SIZE);
	gs_gmac_init_rx_desc_base(gmacdev);	//Program the transmit descriptor base address in to DmaTxBase addr
	gs_gmac_init_rx_qptr(gmacdev); //init the buffers of each descripor

    core_init();
    gs_gmac_dma_operation_mode();

   	gs_gmac_tx_enable(gmacdev);
	gs_gmac_rx_enable(gmacdev);

	gs_gmac_clear_interrupt(gmacdev);
	gs_gmac_enable_interrupt(gmacdev, DmaIntEnable);
	gs_gmac_enable_dma_rx(gmacdev);
	gs_gmac_enable_dma_tx(gmacdev);

    eth_getenv_enetaddr("ethaddr", mac_addr0);
    gs_gmac_set_mac_addr(gmacdev, GmacAddr0High, GmacAddr0Low, mac_addr0);
    core_dump_regs();
    dma_dump_regs();
	return 0;
}

static int gmac_send(struct eth_device *netdev, volatile void *packet, int length)
{
	s32 status = 0;
	u32 offload_needed = 0;
	gs_gmac_device *gmacdev = to_gmac(netdev);
	unsigned char *packet_on_sdram = NULL;
	unsigned int timeout = MACG_TX_TIMEOUT;
    unsigned char *tx_buff;
	if (packet == NULL) {
		gs_debug0("Packet is NULL. What happened to u-boot?\n");
		return -1;
	}

	packet_on_sdram = (unsigned char *)malloc (length + 0x1000);
	if (!packet_on_sdram)
	{
		gs_debug0("C2GMAC: warning, cannot malloc tx_buff\n");
		return -1;
	}

	tx_buff = MALLOC_ALIGN (packet_on_sdram);
	memcpy (tx_buff, (const void *) packet, length);
	gs_debug2("packet = 0x%x packet on sdram = 0x%x length = %d \n",
	    (unsigned int)packet, (unsigned int)tx_buff , length);

	status = gs_gmac_set_tx_qptr(gmacdev, (u32)tx_buff, length,
	                                (u32)packet, 0, 0, 0, offload_needed);
	if (status < 0) {
		gs_debug0("No More Free Tx Descriptors\n");
		return -1;
	}

	gs_gmac_resume_dma_tx(gmacdev);

	while (gs_gmac_intr_tx_handler(gmacdev) != 0 && --timeout > 0);

	if (!timeout)
		gs_debug2("Timeout\n");
    if(packet_on_sdram)
    {
        free(packet_on_sdram);
        packet_on_sdram = NULL;
    }

	return 0;
}

static int gmac_recv(struct eth_device *netdev)
{
	gs_gmac_device *gmacdev = to_gmac(netdev);
	u32 interrupt = 0;
    volatile int dma_status_reg = 0;

	/*Read the Dma interrupt status to know whether the interrupt got generated by our device or not*/
	dma_status_reg = gs_gmac_read_reg(gmacdev->DmaBase, DmaStatus);

	if(dma_status_reg == 0)
		return 0;

	gs_debug2("Dma Rx Status Reg: 0x%08x\n",dma_status_reg);

	if(dma_status_reg & GmacMmcIntr){
		gs_debug2(": Interrupt due to MMC module\n");
	}

	if(dma_status_reg & GmacLineIntfIntr){
		gs_debug2(": Interrupt due to GMAC LINE module\n");
	}

	/*Now lets handle the DMA interrupts
	 this is also clear the interrupts ! ! !*/
	interrupt = gs_gmac_get_interrupt_type(gmacdev, DmaIntRxOnly);

	gs_debug2(":Interrupts to be handled: 0x%08x\n",interrupt);

	if (interrupt & GMACDmaError) {
		gs_debug0(":Fatal Bus Error Inetrrupt Seen\n");
	}

	if(interrupt & GMACDmaRxNormal){
		gs_debug2(": -----------------------------Rx Normal \n");
		gs_handle_received_data(gmacdev);
	}

	if(interrupt & GMACDmaRxAbnormal){
		gs_debug0(":Abnormal Rx Interrupt Seen\n");
		gs_gmac_resume_dma_rx(gmacdev);
	}

	if(interrupt & GMACDmaRxStopped){
		gs_debug0(":Receiver stopped seeing Rx interrupts\n");
	}
    core_dump_regs();
    dma_dump_regs();
	return 0;
}
static void gmac_halt(struct eth_device *netdev)
{
	gs_gmac_device *gmacdev = to_gmac(netdev);

	gs_gmac_tx_disable(gmacdev);
	gs_gmac_rx_disable(gmacdev);
	/* Disable TX and RX */
	gs_gmac_disable_dma_tx(gmacdev);
	gs_gmac_disable_dma_rx(gmacdev);

   	/*Free the Rx/Tx Descriptor contents */
	if( rx_buff_org )
		free(rx_buff_org);
	if( tx_desc_org )
		free(tx_desc_org);
	if( rx_desc_org )
		free(rx_desc_org);

	rx_buff_org = NULL;
	tx_desc_org = NULL;
	rx_desc_org = NULL;
}

int gmac_eth_initialize(int id, u32 regs, unsigned int phy_addr)
{
	struct eth_device *netdev = NULL;

	memset(g_gsgmacdev, 0, sizeof(gs_gmac_device));

	netdev = &(g_gsgmacdev->netdev);

	sprintf(netdev->name,"gmac%d", id) ;
	netdev->init = gmac_init;
	netdev->halt = gmac_halt;
	netdev->send = gmac_send;
	netdev->recv = gmac_recv;

    gmac_system_init();

	gs_gmac_reset(g_gsgmacdev);

	gs_gmac_attach(g_gsgmacdev, (u32)(regs + MACBASE),
	                  (u32)(regs + DMABASE));

	eth_register(netdev);

	//miiphy_register("gmac", gmac_mii_phy_read, gmac_mii_phy_write);
	return 0;
}
