/*  sun7i_gmac.c: Version 1.0 2013.06.07

        GMAC Ethernet driver for u-boot.

        This program is free software; you can redistribute it and/or
        modify it under the terms of the GNU General Public License
        as published by the Free Software Foundation; either version 2
        of the License, or (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.


  (C)Copyright 2013 Grandstream. All Rights Reserved.
        xchen@grandstream.com

*/

#include <common.h>
#include <asm/io.h>
#include <miiphy.h>
#include "sun7i_gmac.h"
#include "sun7i_phy.h"
#include <malloc.h>
extern gs_gmac_device* g_gsgmacdev;
/**
 * gmac_mdio_read
 * @bus: points to the mii_bus structure
 * @phyaddr: MII addr reg bits 15-11
 * @phyreg: MII addr reg bits 10-6
 * Description: it reads data from the MII register from within the phy device.
 * For the 7111 GMAC, we must set the bit 0 in the MII address register while
 * accessing the PHY registers.
 * Fortunately, it seems this has no drawback for the 7109 MAC.
 */
int gmac_mdio_read(u32 RegBase, int phyaddr, int phyreg)
{
	u16 value = (((phyaddr << 11) & (0x0000F800)) |
					((phyreg << 6) & (0x000007C0)));
    u16 data = 0x55aa;

	value |= MII_BUSY | ((0x2/*clk_csr*/ & 7) << 2);

	while (((readl(RegBase + GmacGmiiAddr)) & MII_BUSY) == 1);
	writel(value, RegBase + GmacGmiiAddr);
	while (((readl(RegBase + GmacGmiiAddr)) & MII_BUSY) == 1);
    data = (u16)readl(RegBase + GmacGmiiData);
    gs_debug2("gmac_mdio_read 0x%x\n", data);

	return data;
}

/**
 * gmac_mdio_write
 * @bus: points to the mii_bus structure
 * @phyaddr: MII addr reg bits 15-11
 * @phyreg: MII addr reg bits 10-6
 * @phydata: phy data
 * Description: it writes the data into the MII register from within the device.
 */
int gmac_mdio_write(u32 RegBase, int phyaddr,
							int phyreg, u16 data)
{
	u16 value = (((phyaddr << 11) & (0x0000F800)) |
		 ((phyreg << 6) & (0x000007C0))) | MII_WRITE;

	value |= MII_BUSY | ((0x2/*clk_csr*/ & 7) << 2);


	/* Wait until any existing MII operation is complete */
	while (((readl(RegBase + GmacGmiiAddr)) & MII_BUSY) == 1);

	/* Set the MII address register to write */
	writel(data, RegBase + GmacGmiiData);
	writel(value, RegBase + GmacGmiiAddr);

	/* Wait until any existing MII operation is complete */
	while (((readl(RegBase + GmacGmiiAddr)) & MII_BUSY) == 1);

	return 0;
}

uint16_t phy_reg_read(uint32_t phy_addr, uint8_t reg)
{
	return gmac_mdio_read(g_gsgmacdev->MacBase, phy_addr, reg);
}

int phy_reg_write(uint32_t phy_addr, uint8_t reg, uint16_t data)
{
	return gmac_mdio_write(g_gsgmacdev->MacBase, phy_addr, reg, data);
}

s32 gs_gmac_read_phy_reg(u32 RegBase, u32 Phy_addr, u32 reg, u16 * data)
{
    int i, value = 0;
#if defined CONFIG_GVR3550_EP2
    value = gmac_mdio_read(g_gsgmacdev->MacBase, Phy_addr, reg);
#else
    for (i = 1; i <= 2; i++){
        value |= gmac_mdio_read(g_gsgmacdev->MacBase, i, reg);
    }
#endif
    *data = (u16)value;
    return 0;
}

s32 gs_gmac_write_phy_reg(u32 RegBase, u32 Phy_addr, u32 reg, u16 data)
{
    int i, ret;
#if defined CONFIG_GVR3550_EP2
	return gmac_mdio_write(g_gsgmacdev->MacBase, Phy_addr, reg, data);
#else
    for (i = 1; i <= 2; i++){
        ret |= gmac_mdio_write(g_gsgmacdev->MacBase, i, reg, data);
    }
    return ret;
#endif
}

int gmac_system_init(void)
{
	int ret = 0;
	int reg_value;
	/* configure system io */
    writel(0x55555555, GPIO_BASE + PA_CFG0);

    writel(0x55555555, GPIO_BASE + PA_CFG1);

    writel(0x00000015, GPIO_BASE + PA_CFG2);

    /* writel(0x00002222, priv->gpiobase + PA_CFG3); */

    writel(0xFFFFFFFF, GPIO_BASE + PA_DRV0_REG);
    writel(0xF, GPIO_BASE + PA_DRV1_REG);

    /*clear PA17*/
    reg_value = readl(GPIO_BASE + PA_DAT_REG);
    reg_value &= ~(1<<17);
    writel(reg_value,GPIO_BASE + PA_DAT_REG);

    udelay(30000);

    /*set PA17 to output 1*/
    reg_value = readl(GPIO_BASE + PA_DAT_REG);
    reg_value |= (1<<17);
    writel(reg_value, GPIO_BASE + PA_DAT_REG);
	return ret;
}

/**
 * gmac_mdio_reset
 * @bus: points to the mii_bus structure
 * Description: reset the MII bus
 */
void gmac_mdio_reset(void)
{
	writel(0, g_gsgmacdev->MacBase + GmacGmiiAddr);
}

int gmac_mdio_init(void)
{
#if defined CONFIG_GVR3550_EP2
    {
        int regval = 0;
        gmac_mdio_write(g_gsgmacdev->MacBase, 0, 0xd, 0x0007);
        gmac_mdio_write(g_gsgmacdev->MacBase, 0, 0xe, 0x8016);
        gmac_mdio_write(g_gsgmacdev->MacBase, 0, 0xd, 0x4007);
        regval = gmac_mdio_read(g_gsgmacdev->MacBase, 0, 0xe);
        gmac_mdio_write(g_gsgmacdev->MacBase, 0, 0xe, (regval|0x0018));

        gmac_mdio_write(g_gsgmacdev->MacBase, 0, 0x1d, 0x05);
        regval = gmac_mdio_read(g_gsgmacdev->MacBase, 0, 0x1e);
        gmac_mdio_write(g_gsgmacdev->MacBase, 0, 0x1e, (regval|0x0100));
    }
#else
	athrs17_reg_init();
#endif
	return 0;
}

/**
 * gmac_clk_ctl
 * @flag: 0--disable, 1--enable.
 * Description: enable and disable gmac clk.
 */
void gmac_clk_ctl(unsigned int flag)
{
	int phy_interface = PHY_INTERFACE_MODE_RGMII;
	u32  priv_clk_reg;

	int reg_value;
	reg_value = readl(CCMU_BASE + AHB1_GATING);
	flag ? (reg_value |= GMAC_AHB_BIT) : (reg_value &= ~GMAC_AHB_BIT);
	writel(reg_value, CCMU_BASE + AHB1_GATING);

	/* We should set the interface type. */
	priv_clk_reg = readl(CCMU_BASE + GMAC_CLK_REG);

	if (phy_interface == PHY_INTERFACE_MODE_RGMII)
		priv_clk_reg |= 0x00000004;
	else
		priv_clk_reg &= (~0x00000004);

	/* Set gmac transmit clock source. */
	priv_clk_reg &= (~0x00000003);
	if (phy_interface == PHY_INTERFACE_MODE_RGMII
			|| phy_interface == PHY_INTERFACE_MODE_GMII)
		priv_clk_reg |= 0x00000002;

	writel(priv_clk_reg, CCMU_BASE + GMAC_CLK_REG);
}

/**
 * The Low level function to read register contents from Hardware.
 *
 * @param[in] pointer to the base of register map
 * @param[in] Offset from the base
 * \return  Returns the register contents
 */
u32 gs_gmac_read_reg(u32 RegBase, u32 RegOffset)
{
	u32 addr = RegBase + RegOffset;
	u32 data = readl(addr);

	return data;
}

/**
 * The Low level function to write to a register in Hardware.
 *
 * @param[in] pointer to the base of register map
 * @param[in] Offset from the base
 * @param[in] Data to be written
 * \return  void
 */
void gs_gmac_write_reg(u32 RegBase, u32 RegOffset, u32 RegData)
{
	u32 addr = RegBase + RegOffset;
	writel(RegData, addr);
}
/**
 * The Low level function to clear bits of a register in Hardware.
 *
 * @param[in] pointer to the base of register map
 * @param[in] Offset from the base
 * @param[in] Bit mask to clear bits to logical 0
 * \return  void
 */
static void gs_gmac_clear_bits(u32 RegBase, u32 RegOffset, u32 BitPos)
{
	u32 addr = RegBase + RegOffset;
	u32 data = readl(addr);
	data &= (~BitPos);
	writel(data, addr);
}

/**
  * Disable the transmission of frames on GMII/MII.
  * GMAC transmit state machine is disabled after completion of transmission of current frame.
  * \return returns void.
  */
void gs_gmac_tx_disable(gs_gmac_device *gmacdev)
{
	gs_gmac_clear_bits(gmacdev->MacBase, GmacConfig, GmacTx);
}

/**
  * Disable the reception of frames on GMII/MII.
  * GMAC receive state machine is disabled after completion of reception of current frame.
  * \return returns void.
  */
void gs_gmac_rx_disable(gs_gmac_device *gmacdev)
{
	gs_gmac_clear_bits(gmacdev->MacBase, GmacConfig, GmacRx);
}

/**
  * Disable the DMA for Transmission.
  * \return returns void.
  */
void gs_gmac_disable_dma_tx(gs_gmac_device *gmacdev)
{
	u32 data;
	data = gs_gmac_read_reg(gmacdev->DmaBase, DmaControl);
	data &= (~DmaTxStart);
	gs_gmac_write_reg(gmacdev->DmaBase, DmaControl, data);
}

/**
  * Disable the DMA for Reception.
  * \return returns void.
  */
void gs_gmac_disable_dma_rx(gs_gmac_device *gmacdev)
{
	u32 data;
	gs_gmac_clear_bits(gmacdev->DmaBase, DmaControl, DmaRxStart);
	data = gs_gmac_read_reg(gmacdev->DmaBase, DmaControl);
	data &= (~DmaRxStart);
	gs_gmac_write_reg(gmacdev->DmaBase, DmaControl, data);
}

/**
  * Function to reset the GMAC core.
  * This resets the DMA and GMAC core. After reset all the
  * registers holds their respective reset value
  * \return 0 on success else return the error status.
  */
s32 gs_gmac_reset(gs_gmac_device *gmacdev)
{
	u32 data = 0;
	gs_gmac_write_reg(gmacdev->DmaBase, DmaBusMode, DmaResetOn);
	udelay(50000);
	data = gs_gmac_read_reg(gmacdev->DmaBase, DmaBusMode);
	return 0;
}

/**
  * Sets the Mac address in to GMAC register.
  * This function sets the MAC address to the MAC register in question.
  * @param[in] Register offset for Mac address high
  * @param[in] Register offset for Mac address low
  * @param[in] buffer containing mac address to be programmed.
  * \return 0 upon success. Error code upon failure.
  */
s32 gs_gmac_set_mac_addr(gs_gmac_device *gmacdev, u32 MacHigh, u32 MacLow, u8 *MacAddr)
{
	u32 data;

	data = (MacAddr[5] << 8) | MacAddr[4];
	gs_gmac_write_reg(gmacdev->MacBase,MacHigh,data);
	data = (MacAddr[3] << 24) | (MacAddr[2] << 16) | (MacAddr[1] << 8) | MacAddr[0];
	gs_gmac_write_reg(gmacdev->MacBase,MacLow,data);

	return 0;
}

s32 gs_gmac_attach(gs_gmac_device *gmacdev, u32 macBase, u32 dmaBase)
{

	/*Make sure the Device data strucure is cleared before we proceed further*/
	/*Populate the mac and dma base addresses*/
	gmacdev->MacBase = macBase;
	gmacdev->DmaBase = dmaBase;
	gmacdev->PhyBase = 0;

    gs_debug2("gmacdev->MacBase = 0x%x gmacdev->DmaBase = 0x%x gmacdev->PhyBase = 0x%x\n",gmacdev->MacBase , gmacdev->DmaBase , gmacdev->PhyBase);

	return 0;
}

s32 gs_gmac_scan_phy(gs_gmac_device *gmacdev)
{
	u16 phy_id1 = 0;
	u16 phy_id2 = 0;

	s32 status = -1;
	unsigned int tested_phy = 0;
	int rval = -1;

	gmacdev->PhyMap = 0;
	for (tested_phy = 0; tested_phy <= 32; tested_phy++) {
		status = gs_gmac_read_phy_reg(gmacdev->MacBase, tested_phy, PHY_ID_HI_REG, &phy_id1);
		if (status)
			return status;

		status = gs_gmac_read_phy_reg(gmacdev->MacBase, tested_phy, PHY_ID_LOW_REG, &phy_id2);
		if (status)
			return status;

		if ( (phy_id1 != 0xffff) && (phy_id2 != 0xffff) ) {
			gs_debug2("The selected address of the phy is = 0x%x\n", tested_phy);
			gmacdev->PhyBase = tested_phy ;
			rval = 0;
			gmacdev->PhyMap |= 1<<tested_phy;
		}
	}

	return rval;
}


/**
  * Initialize the rx descriptors for ring or chain mode operation.
  *	- Status field is initialized to 0.
  *	- EndOfRing set for the last descriptor.
  *	- buffer1 and buffer2 set to 0 for ring mode of operation. (note)
  *	- data1 and data2 set to 0. (note)
  * @param[in] pointer to dma_desc structure.
  * @param[in] whether end of ring
  * \return void.
  * \note Initialization of the buffer1, buffer2, data1,data2 and status are not done here. This only initializes whether one wants to use this descriptor
  * in chain mode or ring mode. For chain mode of operation the buffer2 and data2 are programmed before calling this function.
  */
void gs_gmac_rx_desc_init_ring(dma_desc *desc, unsigned int last_ring_desc)
{
	desc->status = 0;
	desc->length = last_ring_desc ? RxDescEndOfRing : 0;
	desc->buffer1 = 0;
	desc->buffer2 = 0;
}

void gs_gmac_tx_desc_init_ring(dma_desc *desc, unsigned int last_ring_desc)
{
	desc->length = last_ring_desc? TxDescEndOfRing : 0;
	desc->buffer1 = 0;
	desc->buffer2 = 0;
	desc->status = 0;
}

static s32 gs_gmac_phy_check_link_status(gs_gmac_device *gmacdev)
{
	u16 data;
	s32 status = -1;

	status = gs_gmac_read_phy_reg(gmacdev->MacBase, gmacdev->PhyBase, PHY_STATUS_REG, &data);
	if (status)
		return status;

	gs_debug2("data = 0x%x\n", data);

	if (data == 0xffff)
		return 0;

	data &= PHY_STATUS_LINK_STATUS;

	return (data ? 1 : 0);
}

/**
  * Checks and initialze phy.
  * This function checks whether the phy initialization is complete.
  * \return 0 if success else returns the error number.
  */
s32 gs_gmac_check_phy_init(gs_gmac_device *gmacdev)
{

	s32 status = -1;
	u16 phyReg0Img, phyReg1Img, phyReg4Img, phyReg17Img;
    int retry = 100;
    u32 ctrl = readl(g_gsgmacdev->MacBase + GmacConfig);

	while (gs_gmac_phy_check_link_status(gmacdev) == 0 && retry){
		udelay(1000000);
		retry-- ;
		if (ctrlc()) {
			gs_debug0("\n we get ctrl + c so Abort\n");
			return -1;
		}
	}

    if(retry)
    {
        gmacdev->LinkState = LINKUP;
        gs_debug2("link up\n");
    }
    else
    {
        gmacdev->LinkState = LINKDOWN;
        gs_debug0("No link please check network connection\n");
        return -1;
    }

	status = gs_gmac_read_phy_reg(gmacdev->MacBase, gmacdev->PhyBase, PHY_CONTROL_REG, &phyReg0Img);
	if (status)
		return status;

	if (phyReg0Img == 0xffff) {
		gs_debug0("Error: Unable to start auto negotiation \n");
		return -1;
	}

	phyReg0Img  &= (~PHY_Reg0_AUTONEG);
	status = gs_gmac_write_phy_reg(gmacdev->MacBase, gmacdev->PhyBase, PHY_CONTROL_REG, phyReg0Img);
	if (status)
		return status;
	// Set Local PHY Capabilities for Auto-Negotiation Advertisement
	status =  gs_gmac_read_phy_reg(gmacdev->MacBase,gmacdev->PhyBase, PHY_AN_ADV_REG, &phyReg4Img);
	if (status)
		return status;

	phyReg4Img |= PHY_Reg4_5_PAUSE_OP; // Supports handling RX Pause Frame

	phyReg4Img |= (PHY_Reg4_5_FD_100M | PHY_Reg4_5_HD_100M | PHY_Reg4_5_FD_10M  | PHY_Reg4_5_HD_10M);    // | 0x01E0

	// Speed: 100, Duplex: Full
	phyReg0Img = PHY_Reg0_SPEED | PHY_Reg0_DUPLEX;              // = 0x2100

	// Set Auto-Negotiation Capabilities at PHY
	status = gs_gmac_write_phy_reg(gmacdev->MacBase, gmacdev->PhyBase, PHY_AN_ADV_REG, phyReg4Img);
	if (status)
		return status;
	/* Restart PHY's Auto-Negotiation, with maximal requested setting */
	phyReg0Img |= (PHY_Reg0_AUTONEG | PHY_Reg0_AUTONEGRST);
	status = gs_gmac_write_phy_reg(gmacdev->MacBase, gmacdev->PhyBase, PHY_CONTROL_REG, phyReg0Img);
	if (status)
		return status;
	/* Wait for Auto-Negotiation Results */
	while (1) {
		// Wait a few  mSeconds
		//udelay(DEFAULT_DELAY_VARIABLE);
		udelay(1000000);
		if (ctrlc()) {
			gs_debug0("\n we get ctrl + c so Abort\n");
			return -1;
		}
		// Check for Auto-Negotiation completion
		status =  gs_gmac_read_phy_reg(gmacdev->MacBase, gmacdev->PhyBase, PHY_STATUS_REG, &phyReg1Img);
		if (status){
				return status;
		}

		if (phyReg1Img & PHY_Reg1_AUTONEGDONE) {
				// Auto-Negotiation completed (Successful or Not!)
				gs_debug2("Auto-Negotiation completed\n");//Auto-Negotiation completed (Successful or Not!)
				break;
		}

	} // end_while

    status =  gs_gmac_read_phy_reg(gmacdev->MacBase, gmacdev->PhyBase, PHY_SPECIFIC_STATUS_REG, &phyReg17Img);
    if (status){
        return status;
    }

	gmacdev->DuplexMode = (phyReg17Img & Mii_phy_status_full_duplex) ? FULLDUPLEX : HALFDUPLEX;

		/*if not set to Master configuration in case of Half duplex mode set it manually as Master */
    if (gmacdev->DuplexMode == HALFDUPLEX)
    {
        gs_gmac_read_phy_reg (gmacdev->MacBase, gmacdev->PhyBase, PHY_CONTROL_REG, &phyReg0Img);
        gs_gmac_write_phy_reg (gmacdev->MacBase, gmacdev->PhyBase, PHY_CONTROL_REG, phyReg0Img | Mii_Manual_Master_Config);
    }

    gs_gmac_read_phy_reg (gmacdev->MacBase, gmacdev->PhyBase, PHY_SPECIFIC_STATUS_REG, &phyReg17Img);

    if (phyReg17Img & Mii_phy_status_speed_1000)
    {
        gmacdev->Speed = SPEED1000;
        ctrl &= ~GMAC_CTL_PS;
    }
    else if (phyReg17Img & Mii_phy_status_speed_100)
    {
        gmacdev->Speed = SPEED100;
        ctrl |= GMAC_CTL_PS;
        ctrl |= GMAC_CTL_FES;
    }
    else
    {
        gmacdev->Speed = SPEED10;
        ctrl |= GMAC_CTL_PS;
        ctrl &= ~GMAC_CTL_FES;
    }

#if defined CONFIG_GVR3550_EP2
    writel(ctrl, g_gsgmacdev->MacBase + GmacConfig);
#endif
	gs_debug2("GSGMAC: Link up %sM/%s\n",
			(gmacdev->Speed == SPEED1000) ? "1000" : ((gmacdev->Speed==SPEED100)?"100":"10"),
			(gmacdev->DuplexMode == FULLDUPLEX) ? "FullDuplex" : "HalfDuplex");
	printf("GSGMAC: Link up %sM/%s\n",
			(gmacdev->Speed == SPEED1000) ? "1000" : ((gmacdev->Speed==SPEED100)?"100":"10"),
			(gmacdev->DuplexMode == FULLDUPLEX) ? "FullDuplex" : "HalfDuplex");


	return 0;
}

/**
  * Function to program DMA bus mode register.
  *
  * The Bus Mode register is programmed with the value given. The bits to be set are
  * bit wise or'ed and sent as the second argument to this function.
  * @param[in] the data to be programmed.
  * \return 0 on success else return the error status.
  */
s32 gs_gmac_dma_bus_mode_init(void)
{
    u32 value = readl(g_gsgmacdev->DmaBase + DmaBusMode);
	int limit;
    int pbl = 2;
	/* DMA SW reset */
	value |= SOFT_RESET;
	writel(value, g_gsgmacdev->DmaBase + DmaBusMode);
	limit = 10;
	while (limit--) {
		if (!(readl(g_gsgmacdev->DmaBase + DmaBusMode) & SOFT_RESET))
			break;
		udelay(10000);
	}
	if (limit < 0)
    {
        gs_debug0("gs_gmac_dma_bus_mode_init failed\n");
		return -1;
    }

	value = BUS_MODE_FIXBUST | BUS_MODE_4PBL |
	    ((pbl << BUS_MODE_PBL_SHIFT) |
	     (pbl << BUS_MODE_RPBL_SHIFT));

#ifdef CONFIG_GMAC_DA
	value |= BUS_MODE_DA;	/* Rx has priority over tx */
#endif
	writel(value, g_gsgmacdev->DmaBase + DmaBusMode);

	return 0;
}

/**
  * Programs the DmaTxBaseAddress with the Tx descriptor base address.
  * Tx Descriptor's base address is available in the gmacdev structure. This function progrms the
  * Dma Tx Base address with the starting address of the descriptor ring or chain.
  * \return returns void.
  */
void gs_gmac_init_tx_desc_base(gs_gmac_device *gmacdev)
{
	u32 address = 0;

	address = (u32)gmacdev->TxDesc;

	gs_debug2("0x%x\n",address);
	gs_gmac_write_reg(gmacdev->DmaBase, DmaTxBaseAddr, address);
}

/**
  * Programs the DmaRxBaseAddress with the Rx descriptor base address.
  * Rx Descriptor's base address is available in the gmacdev structure. This function progrms the
  * Dma Rx Base address with the starting address of the descriptor ring or chain.
  * \return returns void.
  */
void gs_gmac_init_rx_desc_base(gs_gmac_device *gmacdev)
{
	u32 address = 0;

	address = ((u32)gmacdev->RxDesc);

	gs_debug2("0x%x\n",address);
	gs_gmac_write_reg(gmacdev->DmaBase,DmaRxBaseAddr,address);
}

/*
 *
 * Sun6i platform gmac core operations
 *
 */
void core_init(void)
{
	u32 value = readl(g_gsgmacdev->MacBase + GmacConfig);
	value |= GMAC_CORE_INIT;
	writel(value, g_gsgmacdev->MacBase + GmacConfig);

	/* Mask GMAC interrupts */
	writel(0x207, g_gsgmacdev->MacBase + GmacInterruptMask);

#ifdef GMAC_VLAN_TAG_USED
	/* Tag detection without filtering */
	//writel(0x0, g_gsgmacdev->MacBase + GMAC_VLAN_TAG);
#endif

}

void dma_oper_mode(int txmode,
				    int rxmode)
{
    u32 op_val = gs_gmac_read_reg(g_gsgmacdev->DmaBase, DmaControl);

	if (txmode == SF_DMA_MODE) {
		gs_debug2("GMAC: enable TX store and forward mode\n");
		/* Transmit COE type 2 cannot be done in cut-through mode. */
		op_val |= OP_MODE_TSF;
		/* Operating on second frame increase the performance
		 * especially when transmit store-and-forward is used.*/
		op_val |= OP_MODE_OSF;
	} else {
		gs_debug2("GMAC: disabling TX store and forward mode"
			      " (threshold = %d)\n", txmode);
		op_val &= ~OP_MODE_TSF;
		op_val &= OP_MODE_TC_TX_MASK;
		/* Set the transmit threshold */
		if (txmode <= 32)
			op_val |= OP_MODE_TTC_32;
		else if (txmode <= 64)
			op_val |= OP_MODE_TTC_64;
		else if (txmode <= 128)
			op_val |= OP_MODE_TTC_128;
		else if (txmode <= 192)
			op_val |= OP_MODE_TTC_192;
		else
			op_val |= OP_MODE_TTC_256;
	}

	if (rxmode == SF_DMA_MODE) {
		gs_debug2("GMAC: enable RX store and forward mode\n");
		op_val |= OP_MODE_RSF;
	} else {
		gs_debug2("GMAC: disabling RX store and forward mode"
			      " (threshold = %d)\n", rxmode);
		op_val &= ~OP_MODE_RSF;
		op_val &= OP_MODE_TC_RX_MASK;
		if (rxmode <= 32)
			op_val |= OP_MODE_RTC_32;
		else if (rxmode <= 64)
			op_val |= OP_MODE_RTC_64;
		else if (rxmode <= 96)
			op_val |= OP_MODE_RTC_96;
		else
			op_val |= OP_MODE_RTC_128;
	}

	gs_gmac_write_reg(g_gsgmacdev->DmaBase, DmaControl, op_val);
}

/**
 *  gmac_dma_operation_mode - HW DMA operation mode
 *  @priv : pointer to the private device structure.
 *  Description: it sets the DMA operation mode: tx/rx DMA thresholds
 *  or Store-And-Forward capability.
 */
void gs_gmac_dma_operation_mode(void)
{
	dma_oper_mode(SF_DMA_MODE, SF_DMA_MODE);
}

/**
 * The Low level function to set bits of a register in Hardware.
 *
 * @param[in] pointer to the base of register map
 * @param[in] Offset from the base
 * @param[in] Bit mask to set bits to logical 1
 * \return  void
 */
static void gs_gmac_set_bits(u32 RegBase, u32 RegOffset, u32 BitPos)
{
	u32 addr = RegBase + RegOffset;
	u32 data = readl(addr);
	data |= BitPos;
	writel(data, addr);
}

/**
  * Enable the reception of frames on GMII/MII.
  * \return returns void.
  */
void gs_gmac_rx_enable(gs_gmac_device *gmacdev)
{
	gs_gmac_set_bits(gmacdev->MacBase, GmacConfig, GmacRx);
}


/**
  * Enable the transmission of frames on GMII/MII.
  * \return returns void.
  */
void gs_gmac_tx_enable(gs_gmac_device *gmacdev)
{
	gs_gmac_set_bits(gmacdev->MacBase, GmacConfig, GmacTx);
}

/**
  * Clears all the pending interrupts.
  * If the Dma status register is read then all the interrupts gets cleared
  * \return returns void.
  */
void gs_gmac_clear_interrupt(gs_gmac_device *gmacdev)
{
	u32 data;
	data = gs_gmac_read_reg(gmacdev->DmaBase, DmaStatus);
	gs_gmac_write_reg(gmacdev->DmaBase, DmaStatus, data);
}

/**
  * Enable all the interrupts.
  * Enables the DMA interrupt as specified by the bit mask.
  * @param[in] bit mask of interrupts to be enabled.
  * \return returns void.
  */
void gs_gmac_enable_interrupt(gs_gmac_device *gmacdev, u32 interrupts)
{
	gs_gmac_write_reg(gmacdev->DmaBase, DmaInterrupt, interrupts);
	//writel(GDMA_DEF_INTR, gmacdev->DmaBase + DmaInterrupt);
}

/**
  * Enable the DMA Reception.
  * \return returns void.
  */
void gs_gmac_enable_dma_rx(gs_gmac_device *gmacdev)
{
	u32 data;
	data = gs_gmac_read_reg(gmacdev->DmaBase, DmaControl);
	data |= DmaRxStart;
	gs_gmac_write_reg(gmacdev->DmaBase, DmaControl, data);
}

/**
  * Enable the DMA Transmission.
  * \return returns void.
  */
void gs_gmac_enable_dma_tx(gs_gmac_device *gmacdev)
{
	u32 data;
	data = gs_gmac_read_reg(gmacdev->DmaBase, DmaControl);
	data |= DmaTxStart;
	gs_gmac_write_reg(gmacdev->DmaBase, DmaControl, data);
}

void gs_gmac_disable_interrupt_all(gs_gmac_device *gmacdev)
{
	gs_gmac_write_reg(gmacdev->DmaBase, DmaInterrupt, DmaIntDisable);
}

static void gs_gmac_tx_desc_init_chain(dma_desc * desc)
{
	desc->length = TxDescChain;
	desc->buffer1 = 0;
}

/**
  * Checks whether the descriptor is owned by DMA.
  * If descriptor is owned by DMA then the OWN bit is set to 1. This API is same for both ring and chain mode.
  * @param[in] pointer to dma_desc structure.
  * \return returns true if Dma owns descriptor and false if not.
  */
static unsigned int gs_gmac_is_desc_owned_by_dma(dma_desc *desc)
{
	return ( ((desc->status & DescOwnByDma) == DescOwnByDma) ? 1 : 0 );
}

static unsigned int gs_gmac_is_desc_empty(dma_desc *desc)
{
	//if both the buffer1 length and buffer2 length are zero desc is empty
	return(( ((desc->length  & DescSize1Mask) == 0) && ((desc->length  & DescSize2Mask) == 0)) ? 1 : 0 );
}

static unsigned int gs_gmac_is_last_tx_desc(gs_gmac_device *gmacdev, dma_desc *desc)
{
	return ( ((desc->length & TxDescEndOfRing) == TxDescEndOfRing) ? 1 : 0 );
}

/**
  * Checks if any Ipv4 header checksum error in the frame just transmitted.
  * This serves as indication that error occureed in the IPv4 header checksum insertion.
  * The sent out frame doesnot carry any ipv4 header checksum inserted by the hardware.
  * @param[in] u32 status field of the corresponding descriptor.
  * \return returns true if error in ipv4 header checksum, else returns false.
  */
static unsigned int gs_gmac_is_tx_ipv4header_checksum_error(gs_gmac_device *gmacdev, u32 status)
{
	return (((status & DescTxIpv4ChkError) == DescTxIpv4ChkError) ? 1 : 0);
}

/**
  * Checks if any payload checksum error in the frame just transmitted.
  * This serves as indication that error occureed in the payload checksum insertion.
  * The sent out frame doesnot carry any payload checksum inserted by the hardware.
  * @param[in] u32 status field of the corresponding descriptor.
  * \return returns true if error in ipv4 header checksum, else returns false.
  */
static unsigned int gs_gmac_is_tx_payload_checksum_error(gs_gmac_device *gmacdev, u32 status)
{
	return (((status & DescTxPayChkError) == DescTxPayChkError) ? 1 : 0);
}

/**
  * Checks whether the descriptor is valid
  * if no errors such as CRC/Receive Error/Watchdog Timeout/Late collision/Giant Frame/Overflow/Descriptor
  * error the descritpor is said to be a valid descriptor.
  * @param[in] pointer to dma_desc structure.
  * \return True if desc valid. false if error.
  */
static unsigned int gs_gmac_is_desc_valid(u32 status)
{
	return (((status & DescError) == 0) ? 1 : 0);
}

static unsigned int gs_gmac_is_tx_desc_chained(dma_desc * desc)
{
	return (((desc->length & TxDescChain) == TxDescChain) ? 1 : 0);
}

static unsigned int gs_gmac_is_last_rx_desc(gs_gmac_device *gmacdev, dma_desc *desc)
{
	return ( ( ((desc->length & RxDescEndOfRing) == RxDescEndOfRing)) ? 1 : 0);
}

static unsigned int gs_gmac_is_rx_desc_chained(dma_desc *desc)
{
	return (((desc->length & RxDescChain) == RxDescChain) ? 1 : 0);
}

static void gs_gmac_rx_desc_init_chain(dma_desc * desc)
{
	desc->status = 0;
	desc->length = RxDescChain;
	desc->buffer1 = 0;
}

s32 gs_gmac_set_tx_qptr(gs_gmac_device *gmacdev, u32 Buffer1, u32 Length1, u32 Data1, u32 Buffer2, u32 Length2, u32 Data2,u32 offload_needed)
{
	u32 txnext = gmacdev->TxNext;
	dma_desc *txdesc = gmacdev->TxNextDesc;

	if (!gs_gmac_is_desc_empty(txdesc)) {
		gs_debug0("TX descriptor is not empty\n");
		return -1;
	}

	(gmacdev->BusyTxDesc)++; //busy tx descriptor is reduced by one as it will be handed over to Processor now

	txdesc->length |= (((Length1 <<DescSize1Shift) & DescSize1Mask) | ((Length2 <<DescSize2Shift) & DescSize2Mask));

	txdesc->length |=  (DescTxFirst | DescTxLast | DescTxIntEnable); //Its always assumed that complete data will fit in to one descriptor

	txdesc->buffer1 = Buffer1;

	txdesc->buffer2 = Buffer2;

	txdesc->status = DescOwnByDma;

	gmacdev->TxNext = gs_gmac_is_last_tx_desc(gmacdev,txdesc) ? 0 : txnext + 1;
	gmacdev->TxNextDesc = gs_gmac_is_last_tx_desc(gmacdev,txdesc) ? gmacdev->TxDesc : (txdesc + 1);

	gs_debug2("%02d %08x %08x %08x %08x %08x %08x %08x\n",txnext,(u32)txdesc,txdesc->status,txdesc->length,txdesc->buffer1,txdesc->buffer2,txdesc->buffer1,txdesc->buffer2);

	return txnext;
}

void gs_gmac_resume_dma_tx(gs_gmac_device *gmacdev)
{
	gs_gmac_write_reg(gmacdev->DmaBase, DmaTxPollDemand, 1);
}

void gs_gmac_resume_dma_rx(gs_gmac_device *gmacdev)
{
	gs_gmac_write_reg(gmacdev->DmaBase, DmaRxPollDemand, 1);
}

u32 gs_gmac_get_interrupt_type(gs_gmac_device *gmacdev, u32 clear_mask)
{
	u32 data;
	u32 interrupts = 0;
	data = gs_gmac_read_reg(gmacdev->DmaBase, DmaStatus);
	gs_gmac_write_reg(gmacdev->DmaBase, DmaStatus , data & 0x1ffff);
	gs_debug2("DMA status reg is  %08x\n",data);

	if (data & DmaIntErrorMask)	interrupts |= GMACDmaError;
	if (data & DmaIntRxNormMask)	interrupts |= GMACDmaRxNormal;
	if (data & DmaIntRxAbnMask)	interrupts |= GMACDmaRxAbnormal;
	if (data & DmaIntRxStoppedMask)	interrupts |= GMACDmaRxStopped;
	if (data & DmaIntTxNormMask)	interrupts |= GMACDmaTxNormal;
	if (data & DmaIntTxAbnMask)	interrupts |= GMACDmaTxAbnormal;
	if (data & DmaIntTxStoppedMask)	interrupts |= GMACDmaTxStopped;

	if (data & (0x1 << 0) )
    {
		gs_debug2("DMA status transmission is finished\n");
    }
	if (data & (0x1 << 1) )
    {
		gs_debug2("DMA status transmission is stopped\n");
    }

	if (data & (0x1 << 2) ) {
		gs_debug2("DMA status Transmit Buffer Unavailable - Transmission is suspended\n");

        if( ((data & (0x00700000)) >> 20)  == 0x0) gs_debug2("Stopped; Reset or Stop Transmit Command issued.\n");
		if( ((data & (0x00700000)) >> 20)  == 0x1) gs_debug2("Running; Fetching Transmit Transfer Descriptor.\n");
		if( ((data & (0x00700000)) >> 20)  == 0x2) gs_debug2("Running; Waiting for status.\n");
		if( ((data & (0x00700000)) >> 20)  == 0x3) gs_debug2("Running; Reading Data from host memory buffer and queuing it to\n");

		if( ((data & (0x00700000)) >> 20)  == 0x4) gs_debug2("TIME_STAMP write state.\n");
		if( ((data & (0x00700000)) >> 20)  == 0x5) gs_debug2("Reserved for future use.\n");
		if( ((data & (0x00700000)) >> 20)  == 0x6) gs_debug2("Suspended; Transmit Descriptor Unavailable or Transmit Buffer Underflow.\n");
		if( ((data & (0x00700000)) >> 20)  == 0x7) gs_debug2("Running; Closing Transmit Descriptor.\n");
	}

	if (data & (0x1 << 3) )	gs_debug2("DMA status Transmit Jabber Timeout - transmitter had been excessively active\n");
	if (data & (0x1 << 4) )	gs_debug2("DMA status Receive Overflow\n");
	if (data & (0x1 << 5) ) gs_debug2("DMA status Transmit Underflow\n");
	if (data & (0x1 << 6) )	gs_debug2("DMA status Receive Interrupt - completion of frame reception\n");
	if (data & (0x1 << 7) )	gs_debug2("DMA status Receive Buffer Unavailable - Receive Process is suspended\n");
	if (data & (0x1 << 8) )	gs_debug2("DMA status Receive Process Stopped\n");
	if (data & (0x1 << 9) )	gs_debug2("DMA status Receive Watchdog Timeout - a frame with a length greater than 2,048 bytes is received\n");
	if (data & (0x1 << 10) )	gs_debug2("DMA status Early Transmit Interrupt - the frame to be transmitted was fully transferred to the MTL transmit FIFO\n");

	if (data & (0x1 << 13) ) {
		gs_debug0("DMA status Fatal Bus Error Interrupt - \n");
		if(data & (0x1 << 23) )
			gs_debug0("Error during data transfer by TxDMA\n");
		else
			gs_debug0("Error during data transfer by RxDMA\n");

		if(data & (0x1 << 24) )
			gs_debug0("Error during read transfer\n");
		else
			gs_debug0("Error during write transfer\n");

		if(data & (0x1 << 25) )
			gs_debug0("Error during descriptor access\n");
		else
			gs_debug0("Error during data buffer access\n");
	}

	if (data & (0x1 << 14) )	gs_debug2("DMA status Early Receive Interrupt - the DMA had filled the first data buffer of the packet\n");

	if ( ((data & (0x000E0000)) >> 17)  == 0x0) gs_debug2("Stopped: Reset or Stop Receive Command issued.\n");
	if ( ((data & (0x000E0000)) >> 17)  == 0x1) gs_debug2("Running: Fetching Receive Transfer Descriptor.\n");
	if ( ((data & (0x000E0000)) >> 17)  == 0x2) gs_debug2("Reserved for future use.\n");
	if ( ((data & (0x000E0000)) >> 17)  == 0x3) gs_debug2("Running: Waiting for receive packet.\n");
	if ( ((data & (0x000E0000)) >> 17)  == 0x4) gs_debug2("Suspended: Receive Descriptor Unavailable.\n");
	if ( ((data & (0x000E0000)) >> 17)  == 0x5) gs_debug2("Running: Closing Receive Descriptor.\n");
	if ( ((data & (0x000E0000)) >> 17)  == 0x6) gs_debug2("TIME_STAMP write state.\n");
	if ( ((data & (0x000E0000)) >> 17)  == 0x7) gs_debug2("Running: Transferring the receive packet data from receive buffer to host memory.\n");
	return interrupts;
}

/**
  * Get the index and address of Tx desc.
  * This api is same for both ring mode and chain mode.
  * This function tracks the tx descriptor the DMA just closed after the transmission of data from this descriptor is
  * over. This returns the descriptor fields to the caller.
  * @param[out] status field of the descriptor.
  * @param[out] Dma-able buffer1 pointer.
  * @param[out] length of buffer1 (Max is 2048).
  * @param[out] virtual pointer for buffer1.
  * @param[out] Dma-able buffer2 pointer.
  * @param[out] length of buffer2 (Max is 2048).
  * @param[out] virtual pointer for buffer2.
  * @param[out] u32 data indicating whether the descriptor is in ring mode or chain mode.
  * \return returns present tx descriptor index on success. Negative value if error.
  */
static s32 gs_gmac_get_tx_qptr(gs_gmac_device *gmacdev, u32 *Status, u32 *Buffer1, u32 *Length1, u32 *Data1, u32 *Buffer2, u32 * Length2, u32 * Data2 )
{
	u32 txover = gmacdev->TxBusy;
	dma_desc *txdesc = gmacdev->TxBusyDesc;

	if (gs_gmac_is_desc_owned_by_dma(txdesc))
    {
        gs_debug2("gs_gmac_is_desc_owned_by_dma++++++++++++++++++++++++++++++++++\n");
		return -1;
    }
	if (gs_gmac_is_desc_empty(txdesc))
    {
        gs_debug2("gs_gmac_is_desc_empty++++++++++++++++++++++++++++++++++\n");
		return -1;
    }

	(gmacdev->BusyTxDesc)--; //busy tx descriptor is reduced by one as it will be handed over to Processor now

	if (Status != 0)
		*Status = txdesc->status;

	if (Buffer1 != 0)
		*Buffer1 = txdesc->buffer1;
	if (Length1 != 0)
		*Length1 = (txdesc->length & DescSize1Mask) >> DescSize1Shift;
	if (Data1 != 0)
		*Data1 = txdesc->buffer1;

	if (Buffer2 != 0)
		*Buffer2 = txdesc->buffer2;
	if (Length2 != 0)
		*Length2 = (txdesc->length & DescSize2Mask) >> DescSize2Shift;
	if (Data1 != 0)
		*Data2 = txdesc->buffer2;

	gmacdev->TxBusy     = gs_gmac_is_last_tx_desc(gmacdev,txdesc) ? 0 : txover + 1;

	if (gs_gmac_is_tx_desc_chained(txdesc)) {
		gs_gmac_tx_desc_init_chain(txdesc);
	} else {
		gmacdev->TxBusyDesc = gs_gmac_is_last_tx_desc(gmacdev,txdesc) ? gmacdev->TxDesc : (txdesc + 1);
		gs_gmac_tx_desc_init_ring(txdesc, gs_gmac_is_last_tx_desc(gmacdev, txdesc));
	}
	gs_debug2("%02d %08x %08x %08x %08x %08x %08x %08x\n", txover, (u32)txdesc, txdesc->status, txdesc->length, txdesc->buffer1,txdesc->buffer2,txdesc->buffer1,txdesc->buffer2);

	return txover;
}

static void gs_handle_transmit_over(gs_gmac_device *gmacdev)
{
	s32 desc_index;
	u32 data1, data2;
	u32 status;
	u32 length1, length2;
	u32 dma_addr1, dma_addr2;

	do {
		desc_index = gs_gmac_get_tx_qptr(gmacdev, &status, &dma_addr1, &length1, &data1, &dma_addr2, &length2, &data2);

		if (desc_index >= 0 && data1 != 0) {
			gs_debug2("Finished Transmit at Tx Descriptor %d for skb 0x%08x and buffer = %08x whose status is %08x \n", desc_index,data1,dma_addr1,status);

			if (gs_gmac_is_tx_ipv4header_checksum_error(gmacdev, status))
            {
				gs_debug0("Harware Failed to Insert IPV4 Header Checksum\n");
            }
			if (gs_gmac_is_tx_payload_checksum_error(gmacdev, status))
            {
				gs_debug0("Harware Failed to Insert Payload Checksum\n");
            }

			if (!gs_gmac_is_desc_valid(status))
            {
				gs_debug0("Error in Status %08x\n",status);
            }
		}
	} while (desc_index >= 0);
}

int gs_gmac_intr_tx_handler(gs_gmac_device *gmacdev)
{
	u32 interrupt,dma_status_reg;
	s32 status = -1;

	dma_status_reg = gs_gmac_read_reg(gmacdev->DmaBase, DmaStatus);

	if (dma_status_reg == 0)
		return 0;

	gs_gmac_disable_interrupt_all(gmacdev);

	gs_debug2("Dma Tx Status Reg: 0x%08x\n",dma_status_reg);

	if (dma_status_reg & GmacMmcIntr) {
		gs_debug2(": Interrupt due to MMC module\n");
	}

	if (dma_status_reg & GmacLineIntfIntr) {
		gs_debug2(": Interrupt due to GMAC LINE module\n");
	}

	/*Now lets handle the DMA interrupts
	 this is also clear the interrupts ! ! !*/
	interrupt = gs_gmac_get_interrupt_type(gmacdev, DmaIntTxOnly);

	gs_debug2(":Interrupts to be handled: 0x%08x\n",interrupt);

	if (interrupt & GMACDmaError) {
		gs_debug0(":Fatal Bus Error Inetrrupt Seen\n");
		status = -1;
	}
	if (interrupt & GMACDmaTxNormal) {
		gs_debug2(":Finished Normal Transmission \n");
		gs_handle_transmit_over(gmacdev);//Do whatever you want after the transmission is over
		status = 0;
	}

	if (interrupt & GMACDmaTxAbnormal) {
		gs_debug0(":Abnormal Tx Interrupt Seen\n");
		gs_handle_transmit_over(gmacdev);
		status = -1;
	}

	if (interrupt & GMACDmaTxStopped) {
		gs_debug0(":Transmitter stopped sending the packets\n");
		gs_debug0(":Transmission Resumed\n");
		status = -1;
	}

	/* Enable the interrrupt before returning from ISR*/
	gs_gmac_enable_interrupt(gmacdev,(u32)DmaIntEnable);
	return status;
}

s32 gs_gmac_set_rx_qptr(gs_gmac_device *gmacdev, u32 Buffer1, u32 Length1, u32 Data1, u32 Buffer2, u32 Length2, u32 Data2)
{
	u32 rxnext = gmacdev->RxNext;
	dma_desc * rxdesc = gmacdev->RxNextDesc;

	if (!gs_gmac_is_desc_empty(rxdesc))
		return -1;

	if (gs_gmac_is_rx_desc_chained(rxdesc)) {
		gs_debug2("Chain \n");
		rxdesc->length |= ((Length1 <<DescSize1Shift) & DescSize1Mask);

		rxdesc->buffer1 = Buffer1;

//		if ((rxnext % MODULO_INTERRUPT) != 0)
//			rxdesc->length |= RxDisIntCompl;

		rxdesc->status = DescOwnByDma;

		gmacdev->RxNext     = gs_gmac_is_last_rx_desc(gmacdev,rxdesc) ? 0 : rxnext + 1;
	} else {
		gs_debug2("Ring \n");
		rxdesc->length |= ((((Length1 - 1) <<DescSize1Shift) & DescSize1Mask) | ((Length2 << DescSize2Shift) & DescSize2Mask));

		rxdesc->buffer1 = Buffer1;

		rxdesc->buffer2 = Buffer2;
		rxdesc->status = DescOwnByDma;

		gmacdev->RxNext     = gs_gmac_is_last_rx_desc(gmacdev,rxdesc) ? 0 : rxnext + 1;
		gmacdev->RxNextDesc = gs_gmac_is_last_rx_desc(gmacdev,rxdesc) ? gmacdev->RxDesc : (rxdesc + 1);
	}
	gs_debug2("%02d %08x %08x %08x %08x %08x %08x %08x\n",rxnext,(u32)rxdesc,rxdesc->status,rxdesc->length,rxdesc->buffer1,rxdesc->buffer2);
	(gmacdev->BusyRxDesc)++; //One descriptor will be given to Hardware. So busy count incremented by one

	return rxnext;
}

static s32 gs_gmac_get_rx_qptr(gs_gmac_device *gmacdev, u32 * Status, u32 * Buffer1, u32 * Length1, u32 * Data1, u32 * Buffer2, u32 * Length2, u32 * Data2)
{
	u32 rxnext = gmacdev->RxBusy;	// index of descriptor the DMA just completed. May be useful when data
					//is spread over multiple buffers/descriptors
	dma_desc *rxdesc = gmacdev->RxBusyDesc;

	if (gs_gmac_is_desc_owned_by_dma(rxdesc))
    {
        gs_debug2("gs_gmac_is_desc_owned_by_dma------------------\n");
		return -1;
    }
	if (gs_gmac_is_desc_empty(rxdesc))
    {
        gs_debug2("gs_gmac_is_desc_empty------------------\n");
		return -1;
    }

	if (Status != 0)
		*Status = rxdesc->status;// send the status of this descriptor

	if (Length1 != 0)
		*Length1 = (rxdesc->length & DescSize1Mask) >> DescSize1Shift;
	if (Buffer1 != 0)
		*Buffer1 = rxdesc->buffer1;
	if (Data1 != 0)
		*Data1 = rxdesc->buffer1;

	if (Length2 != 0)
		*Length2 = (rxdesc->length & DescSize2Mask) >> DescSize2Shift;
	if (Buffer2 != 0)
		*Buffer2 = rxdesc->buffer2;
	if (Data1 != 0)
		*Data2 = rxdesc->buffer2;

	gmacdev->RxBusy = gs_gmac_is_last_rx_desc(gmacdev, rxdesc) ? 0 : rxnext + 1;

	if (gs_gmac_is_rx_desc_chained(rxdesc)){
		gs_gmac_rx_desc_init_chain(rxdesc);
		//gs_gmac_desc_init_chain(rxdesc, gmac_is_last_rx_desc(gmacdev,rxdesc),0,0);
	} else {
		gmacdev->RxBusyDesc = gs_gmac_is_last_rx_desc(gmacdev,rxdesc) ? gmacdev->RxDesc : (rxdesc + 1);
		gs_gmac_rx_desc_init_ring(rxdesc, gs_gmac_is_last_rx_desc(gmacdev,rxdesc));
	}
	gs_debug2("%02d %08x %08x %08x %08x %08x %08x %08x\n",rxnext,(u32)rxdesc,rxdesc->status,rxdesc->length,rxdesc->buffer1,rxdesc->buffer2,rxdesc->buffer1,rxdesc->buffer2);
	(gmacdev->BusyRxDesc)--; //This returns one descriptor to processor. So busy count will be decremented by one

	return rxnext;
}

/**
  * returns the byte length of received frame including CRC.
  * This returns the no of bytes received in the received ethernet frame including CRC(FCS).
  * @param[in] pointer to dma_desc structure.
  * \return returns the length of received frame lengths in bytes.
  */
static u32 gs_gmac_get_rx_desc_frame_length(u32 status)
{
	return ((status & DescFrameLengthMask) >> DescFrameLengthShift);
}

/**
  * Decodes the Rx Descriptor status to various checksum error conditions.
  * @param[in] u32 status field of the corresponding descriptor.
  * \return returns decoded enum (u32) indicating the status.
  */
static u32 gs_gmac_is_rx_checksum_error(gs_gmac_device *gmacdev, u32 status)
{
	if     (((status & DescRxChkBit5) == 0) && ((status & DescRxChkBit7) == 0) && ((status & DescRxChkBit0) == 0))
		return RxLenLT600;
	else if(((status & DescRxChkBit5) == 0) && ((status & DescRxChkBit7) == 0) && ((status & DescRxChkBit0) != 0))
		return RxIpHdrPayLoadChkBypass;
	else if(((status & DescRxChkBit5) == 0) && ((status & DescRxChkBit7) != 0) && ((status & DescRxChkBit0) != 0))
		return RxChkBypass;
	else if(((status & DescRxChkBit5) != 0) && ((status & DescRxChkBit7) == 0) && ((status & DescRxChkBit0) == 0))
		return RxNoChkError;
	else if(((status & DescRxChkBit5) != 0) && ((status & DescRxChkBit7) == 0) && ((status & DescRxChkBit0) != 0))
		return RxPayLoadChkError;
	else if(((status & DescRxChkBit5) != 0) && ((status & DescRxChkBit7) != 0) && ((status & DescRxChkBit0) == 0))
		return RxIpHdrChkError;
	else if(((status & DescRxChkBit5) != 0) && ((status & DescRxChkBit7) != 0) && ((status & DescRxChkBit0) != 0))
		return RxIpHdrPayLoadChkError;
	else
		return RxIpHdrPayLoadRes;
}

/**
  * Checks whether the rx descriptor is valid.
  * if rx descripor is not in error and complete frame is available in the same descriptor
  * @param[in] pointer to dma_desc structure.
  * \return returns true if no error and first and last desc bits are set, otherwise it returns false.
  */
static unsigned int gs_gmac_is_rx_desc_valid(u32 status)
{
	return ( ((status & DescError) == 0) && ((status & DescRxFirst) == DescRxFirst) && ((status & DescRxLast) == DescRxLast) ? 1 : 0);
}

void gs_handle_received_data(gs_gmac_device *gmacdev)
{
	s32 desc_index;

	u32 data1;
	u32 data2;
	u32 len;
	u32 status;
	u32 dma_addr1;
	u32 dma_addr2;
	void *skb; //This is the pointer to hold the received data

	//Handle the Receive Descriptors
	do {
		desc_index = gs_gmac_get_rx_qptr(gmacdev, &status,&dma_addr1,NULL, &data1,&dma_addr2,NULL,&data2);
        gs_debug2("desc_index = 0x%x data1 = 0x%x\n", desc_index, data1);

		if(desc_index >= 0 && data1 != 0){
			gs_debug2("Received Data at Rx Descriptor %d for skb 0x%08x whose status is %08x\n",desc_index,data1,status);
			skb = (void *)data1;
			if(gs_gmac_is_rx_desc_valid(status)){

				len =  gs_gmac_get_rx_desc_frame_length(status) - 4; //Not interested in Ethernet CRC bytes

				NetReceive(skb, len);

				gs_debug2("Checksum Offloading will be done now\n");

				if (gs_gmac_is_rx_checksum_error(gmacdev, status) == RxNoChkError)
					gs_debug2("Ip header and TCP/UDP payload checksum Bypassed <Chk Status = 4>\n");

				if (gs_gmac_is_rx_checksum_error(gmacdev, status) == RxIpHdrChkError)
					gs_debug2(" Error in 16bit IPV4 Header Checksum <Chk Status = 6>\n");

				if (gs_gmac_is_rx_checksum_error(gmacdev, status) == RxLenLT600)
					gs_debug2("IEEE 802.3 type frame with Length field Lesss than 0x0600 <Chk Status = 0>\n");

				if (gs_gmac_is_rx_checksum_error(gmacdev, status) == RxIpHdrPayLoadChkBypass)
					gs_debug2("Ip header and TCP/UDP payload checksum Bypassed <Chk Status = 1>\n");

				if (gs_gmac_is_rx_checksum_error(gmacdev, status) == RxChkBypass)
					gs_debug2("Ip header and TCP/UDP payload checksum Bypassed <Chk Status = 3>\n");

				if (gs_gmac_is_rx_checksum_error(gmacdev, status) == RxPayLoadChkError)
					gs_debug2(" TCP/UDP payload checksum Error <Chk Status = 5>\n");

				if (gs_gmac_is_rx_checksum_error(gmacdev, status) == RxIpHdrChkError)
					gs_debug2(" Both IP header and Payload Checksum Error <Chk Status = 7>\n");
			}
			else {
				gs_debug0("Rx descriptor error. Packet dropped. Status = 0x%08x\n", status);
			}

			desc_index = gs_gmac_set_rx_qptr(gmacdev, dma_addr1, RX_BUFFER_LEN, 0, 0, 0, 0);

			if (desc_index < 0)
				gs_debug0("Cannot set Rx Descriptor for %08x\n", dma_addr1);
		}
	} while (desc_index >= 0);
}
