/*
 * (C) Copyright 2007-2011
 * Allwinner Technology Co., Ltd. <www.allwinnertech.com>
 * Tom Cubie <tangliang@allwinnertech.com>
 *
 * Configuration settings for the Allwinner sun7i-evb board.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef __CONFIG_H
#define __CONFIG_H

/*
 * High Level Configuration Options
 */

//#define DEBUG
//#define DEBUG_TICK_PRINTF

#define CONFIG_ALLWINNER			/* It's a Allwinner chip */
#define	CONFIG_SUNXI				/* which is sunxi family */
#define CONFIG_SUN7I				/* which is sun7i */
//#define CONFIG_FPGA					/* working with fpga board */
#define CONFIG_CMD_BOOTA
#include <asm/arch/cpu.h>			/* get chip and board defs */
#include <asm/sizes.h>

#define BOARD_LATE_INIT				/* init the fastboot partitions */

#define SYS_CONFIG_BASE             0x43000000
#define CONFIG_SYS_TEXT_BASE		0x4A000000
#if 0
#define CONFIG_SKIP_LOWLEVEL_INIT	/* currently u-boot is loaded from ice */
#endif
/*
 * Display CPU and Board information
 */
#define CONFIG_DISPLAY_CPUINFO
#define CONFIG_DISPLAY_BOARDINFO

/* Clock Defines */


/* Serial & console */
#define CONFIG_SYS_NS16550
#define CONFIG_SYS_NS16550_SERIAL
#define CONFIG_SYS_NS16550_REG_SIZE	(-4)		/* ns16550 reg in the low bits of cpu reg */
#define CONFIG_SYS_NS16550_CLK		(24000000)
#define CONFIG_SYS_NS16550_COM1		SUNXI_UART0_BASE
#define CONFIG_SYS_NS16550_COM2		SUNXI_UART1_BASE
#define CONFIG_SYS_NS16550_COM3		SUNXI_UART2_BASE
#define CONFIG_SYS_NS16550_COM4		SUNXI_UART3_BASE

#define CONFIG_CONS_INDEX			1			/* which serial channel for console */

/* DRAM Base */
#define CONFIG_SYS_SDRAM_BASE		0x40000000
#define CONFIG_SYS_INIT_RAM_ADDR	0x0
#define CONFIG_SYS_INIT_RAM_SIZE	0x8000      /* 32K */

#define CONFIG_SYS_INIT_SP_OFFSET \
	(CONFIG_SYS_INIT_RAM_SIZE - GENERATED_GBL_DATA_SIZE)
#define CONFIG_SYS_INIT_SP_ADDR \
	(CONFIG_SYS_INIT_RAM_ADDR + CONFIG_SYS_INIT_SP_OFFSET)

/* A10-EVB has 1 banks of DRAM, we use only one in U-Boot */
#define CONFIG_NR_DRAM_BANKS		1
#define PHYS_SDRAM_1				CONFIG_SYS_SDRAM_BASE	/* SDRAM Bank #1 */
#define PHYS_SDRAM_1_SIZE			(512 << 20)				/* 0x20000000, 512 MB Bank #1 */

//#define CONFIG_SYS_MONITOR_BASE	0x00000000

/* Nand config */
#define CONFIG_NAND
#define CONFIG_NAND_SUNXI
#define CONFIG_CMD_NAND                         /* NAND support */
#define CONFIG_SYS_MAX_NAND_DEVICE      1
#define CONFIG_SYS_NAND_BASE            0x00
#define CONFIG_CMD_MEMORY
#define CONFIG_SUNXI_DMA

#define CONFIG_SETUP_MEMORY_TAGS
#define CONFIG_CMDLINE_TAG
#define CONFIG_INITRD_TAG
#define CONFIG_CMDLINE_EDITING

/* mmc config */
#define CONFIG_MMC
#define CONFIG_GENERIC_MMC
#define CONFIG_CMD_MMC
#define CONFIG_MMC_SUNXI
#define CONFIG_MMC_SUNXI_SLOT		0		/* which mmc slot to use, could be 0,1,2,3 */
#define CONFIG_MMC_SUNXI_USE_DMA
#define CONFIG_ENV_IS_IN_MMC
#define CONFIG_SYS_MMC_ENV_DEV		mmc_card_no
#define CONFIG_STORAGE_EMMC
#define CONFIG_FASTBOOT_MMC_NO		mmc_card_no
#define CONFIG_MMC_LOGICAL_OFFSET   (20 * 1024 * 1024/512)

#define CONFIG_DOS_PARTITION
/*
 * Size of malloc() pool
 * 1MB = 0x100000, 0x100000 = 1024 * 1024
 */
#define CONFIG_SYS_MALLOC_LEN		(CONFIG_ENV_SIZE + (4 << 20))

#define CONFIG_FASTBOOT
#define CONFIG_STORAGE_NAND
#define FASTBOOT_TRANSFER_BUFFER		0x41000000
#define FASTBOOT_TRANSFER_BUFFER_SIZE	(256 << 20)

#define FASTBOOT_ERASE_BUFFER			0x40000000
#define FASTBOOT_ERASE_BUFFER_SIZE      (16 << 20)

/* NET config */
#define CONFIG_CMD_NET
#define CONFIG_CMD_PING
//#define CONFIG_MII		1	/* MII PHY management */
#define CONFIG_NET_MULTI
#define CONFIG_TFTP_TSIZE
#define CONFIG_SUN7I_GMAC

/*
 * Miscellaneous configurable options
 */
#define CONFIG_SYS_LONGHELP				/* undef to save memory */
#define CONFIG_SYS_HUSH_PARSER			/* use "hush" command parser	*/
#define CONFIG_SYS_PROMPT_HUSH_PS2	"> "
#ifdef CONFIG_GVR3550
#define CONFIG_SYS_PROMPT              "GVR # "
#else
#define CONFIG_SYS_PROMPT              "gxv#"
#endif
#define CONFIG_SYS_CBSIZE	256			/* Console I/O Buffer Size */
#define CONFIG_SYS_PBSIZE	384			/* Print Buffer Size */
#define CONFIG_SYS_MAXARGS	16			/* max number of command args */

/* Boot Argument Buffer Size */
#define CONFIG_SYS_BARGSIZE			CONFIG_SYS_CBSIZE

/* memtest works on */
#define CONFIG_SYS_MEMTEST_START	CONFIG_SYS_SDRAM_BASE
#define CONFIG_SYS_MEMTEST_END		(CONFIG_SYS_SDRAM_BASE + 256<<20)	/* 256M */
#define CONFIG_SYS_LOAD_ADDR		0x50000000					/* default load address */

#define CONFIG_SYS_HZ				1000

/*-----------------------------------------------------------------------
 * IIC stuff
 *-----------------------------------------------------------------------
 */
 #define CONFIG_SUN7I_I2C
 
 #ifdef CONFIG_SUN7I_I2C
 #define CONFIG_CMD_I2C
#define  CONFIG_HARD_I2C		/* I2C with hardware support	*/
#undef CONFIG_SOFT_I2C			/* I2C bit-banged		*/

#define I2C_INIT
#define I2C_ACTIVE 0
#define I2C_TRISTATE 0

#define CONFIG_SYS_I2C_SPEED		200000	/* use the standard 100kHz speed */
#define CONFIG_SYS_I2C_SLAVE		0x7F	/* mask valid bits */
#define CONFIG_SYS_I2C_FT5X_ADDR   0x38
#endif

/* valid baudrates */
#define CONFIG_BAUDRATE				115200
#define CONFIG_SYS_BAUDRATE_TABLE	{ 9600, 19200, 38400, 57600, 115200 }

/*-----------------------------------------------------------------------
 * Stack sizes
 *
 * The stack sizes are set up in start.S using the settings below
 */
#define CONFIG_STACKSIZE			(256 << 10)				/* 256 KiB */
#define LOW_LEVEL_SRAM_STACK		0x00003FFC


/*-----------------------------------------------------------------------
 * FLASH and environment organization
 */
#define CONFIG_SYS_NO_FLASH

#define CONFIG_SYS_MONITOR_LEN		(256 << 10)	/* 256 KiB */
#define CONFIG_IDENT_STRING			" Grandstream Networks "

#define CONFIG_ENV_IS_IN_NAND_SUNXI	    /* we store env in one partition of our nand */
#define CONFIG_SUNXI_ENV_PARTITION		"env"	/* the partition name */

/*------------------------------------------------------------------------
 * we save the environment in a nand partition, the partition name is defined
 * in sysconfig.fex, which must be the same as CONFIG_SUNXI_NAND_ENV_PARTITION
 * if not, below CONFIG_ENV_ADDR and CONFIG_ENV_SIZE will be where to store env.
 * */
#define CONFIG_CMD_ENV
#define CONFIG_ENV_OVERWRITE
#define CONFIG_CMD_DEFENV
#define CONFIG_ENV_ADDR				(32 << 20)  /* 16M */
#define CONFIG_ENV_SIZE				(128 << 10)	/* 128KB */
#define CONFIG_CMD_SAVEENV
#define CONFIG_ENV_OVERWRITE

#define XMK_STR(x)	#x
#define MK_STR(x)	XMK_STR(x)
#if defined(CONFIG_GXV3275) || defined(CONFIG_GXV3240)
#define CONFIG_EXTRA_ENV_SETTINGS \
	"target=" MK_STR(CONFIG_TARGET) "\0" \
	"bootdelay=1\0" \
	"bootcmd=run setargs_nand boot_normal\0" \
	"console=ttyS0,115200\0" \
	"init=/init\0" \
	"loglevel=4\0" \
	"setargs_nand=setenv bootargs console=${console} init=${init} loglevel=${loglevel} partitions=${partitions} pn=${pn} maxcpus=1\0" \
	"boot_normal=sunxi_flash read 40007800 boot;boota 40007800\0" \
	"boot_kernel=run setargs_nand && nand read 40007800 boot && bootm 40007800\0" \
	"update_boot=tftp 41000000 uImage && nand erase.part boot && nand write 41000000 boot\0" \
	"boot_recovery=sunxi_flash read 40007800 recovery;boota 40007800\0" \
	"load_factcfg=tftp 0x40008000 ${target}.cfg;gs_nvram write fact 0x40008000 0x0 0x100000\0"
#else
#define CONFIG_EXTRA_ENV_SETTINGS \
	"target=" MK_STR(CONFIG_TARGET) "\0" \
	"bootdelay=1\0" \
	"bootcmd=run setargs_nand boot_normal\0" \
	"console=ttyS0,115200\0" \
	"init=/init\0" \
	"loglevel=4\0" \
	"setargs_nand=setenv bootargs console=${console} init=${init} loglevel=${loglevel} partitions=${partitions} pn=${pn}\0" \
	"boot_normal=sunxi_flash read 40007800 boot;boota 40007800\0" \
	"boot_kernel=run setargs_nand && nand read 40007800 boot && bootm 40007800\0" \
	"update_boot=tftp 41000000 uImage && nand erase.part boot && nand write 41000000 boot\0" \
	"boot_recovery=sunxi_flash read 40007800 recovery;boota 40007800\0" \
	"load_factcfg=tftp 0x40008000 ${target}.cfg;gs_nvram write fact 0x40008000 0x0 0x100000\0"
#endif

#define CONFIG_BOOTDELAY	1
#define CONFIG_BOOTCOMMAND	"nand read 50000000 boot;boota 50000000"
#define CONFIG_SYS_BOOT_GET_CMDLINE
#define CONFIG_AUTO_COMPLETE

#define CONFIG_CMD_FAT			/* with this we can access bootfs in nand */
#define CONFIG_CMD_BOOTA		/* boot android image */
#define CONFIG_CMD_RUN			/* run a command */
#define CONFIG_CMD_BOOTD		/* boot the default command */

#define CONFIG_LZMA 1
/* added by grandstream.lfzhu  begin*/
#define CONFIG_GS_RECOVERY
//#define CONFIG_GS_STORAGE_EMMC
#define CONFIG_GS_STORAGE_FLASH

#ifdef CONFIG_GVR3550
#define GS_PROD_SIGNATURE       0x6c4f06fd

#define GS_FACTORY_CFG_SIZE     SZ_2M
#define MTD_SIZE_RESERVED       SZ_2M
#define MTD_SIZE_BCD   	        SZ_1M

#define SECTORSIZE 512
#define MTD_SIZE_IMGHDR      SZ_128K
#define MTD_SIZE_PROVID      SZ_16K
#define MTD_SIZE_NVRAM       SZ_2M
#define MTD_SIZE_SEGHDR      (SZ_512K + SZ_256K)  //= 6*MTD_SIZE_SEGHDR
#define MTD_SIZE_USERCFG        SZ_2M

#define NAND_OFFSET_NVRAM  0    //offset from RAWDATA partition.
#define NAND_OFFSET_CURRENTID  (NAND_OFFSET_NVRAM+MTD_SIZE_NVRAM)
#define NAND_OFFSET_LASTUPDID  (NAND_OFFSET_CURRENTID+MTD_SIZE_PROVID)
#define NAND_OFFSET_SEGHDR0   (NAND_OFFSET_LASTUPDID+MTD_SIZE_PROVID)
#define NAND_OFFSET_SEGHDR1   (NAND_OFFSET_SEGHDR0+MTD_SIZE_SEGHDR)
#define NAND_OFFSET_SEGHDR2   (NAND_OFFSET_SEGHDR1+MTD_SIZE_SEGHDR)
#define NAND_OFFSET_SEGHDR3   (NAND_OFFSET_SEGHDR2+MTD_SIZE_SEGHDR)
#define GS_FACTORY_CFG_OFFSET   (NAND_OFFSET_SEGHDR3+MTD_SIZE_SEGHDR)
#define GS_OFFSET_RESERVED  (GS_FACTORY_CFG_OFFSET+GS_FACTORY_CFG_SIZE)

#define NAND_OFFSET_IMGHDR0_UBOOT     NAND_OFFSET_SEGHDR0
#define NAND_OFFSET_IMGHDR0_BOOTR    (NAND_OFFSET_SEGHDR0+MTD_SIZE_IMGHDR)
#define NAND_OFFSET_IMGHDR0_BOOTA    (NAND_OFFSET_IMGHDR0_BOOTR+MTD_SIZE_IMGHDR)
#define NAND_OFFSET_IMGHDR0_BOOTB    (NAND_OFFSET_IMGHDR0_BOOTA+MTD_SIZE_IMGHDR)
#define NAND_OFFSET_IMGHDR0_SYSTEMA  (NAND_OFFSET_IMGHDR0_BOOTB+MTD_SIZE_IMGHDR)
#define NAND_OFFSET_IMGHDR0_SYSTEMB  (NAND_OFFSET_IMGHDR0_SYSTEMA+MTD_SIZE_IMGHDR)

#if defined(CONFIG_GS_STORAGE_EMMC)
#define BCD_SECTOR_OFFSET (34817)
#define BCD_SECTOR_NUM (64)
#elif defined(CONFIG_GS_STORAGE_FLASH)
#define BCD_SECTOR_OFFSET      (GS_OFFSET_RESERVED+MTD_SIZE_RESERVED)
#define BCD_SECTOR_NUM (64)
#define CONFIG_VERSIONHEADER_OFFSET  (SZ_2M)
#else
#define BCD_SECTOR_OFFSET      (GS_OFFSET_RESERVED+MTD_SIZE_RESERVED)
#define BCD_SECTOR_NUM (64)
#endif

#define NAND_OFFSET_USERCFG   (BCD_SECTOR_OFFSET+BCD_SECTOR_NUM*512)
#define GS_TEMP_SECTOR_OFFSET  (NAND_OFFSET_USERCFG+MTD_SIZE_USERCFG)

#else
#define GS_PROD_SIGNATURE       0x43a78f39

#define MTD_SIZE_NVRAM       SZ_2M
#define MTD_SIZE_SEGHDR	     SZ_2M

#define GS_FACTORY_CFG_SIZE     SZ_2M
#define MTD_SIZE_RESERVED  SZ_2M
#define MTD_SIZE_BCD   	      SZ_1M
#define MTD_SIZE_TEMP   	      SZ_4M

#define NAND_OFFSET_NVRAM  0    //relative NVRAM part start addr
#define NAND_OFFSET_SEGHDR0   (NAND_OFFSET_NVRAM+MTD_SIZE_NVRAM)
#define NAND_OFFSET_SEGHDR1   (NAND_OFFSET_SEGHDR0+SZ_1M)
//#define NAND_OFFSET_SEGHDR2   (NAND_OFFSET_SEGHDR1+MTD_SIZE_SEGHDR)
//#define NAND_OFFSET_SEGHDR3   (NAND_OFFSET_SEGHDR2+MTD_SIZE_SEGHDR)
#define GS_FACTORY_CFG_OFFSET   (NAND_OFFSET_SEGHDR1+SZ_1M)
#define GS_OFFSET_RESERVED  (GS_FACTORY_CFG_OFFSET+GS_FACTORY_CFG_SIZE)

#if defined(CONFIG_GS_STORAGE_EMMC)
#define BCD_SECTOR_OFFSET (34817)
#elif defined(CONFIG_GS_STORAGE_FLASH)
#define BCD_SECTOR_OFFSET      (GS_OFFSET_RESERVED+MTD_SIZE_RESERVED)
#define BCD_SECTOR_NUM (64)
#define CONFIG_VERSIONHEADER_OFFSET  (SZ_2M)
#else
#define BCD_SECTOR_OFFSET      (GS_OFFSET_RESERVED+MTD_SIZE_RESERVED)
#endif

#define GS_TEMP_SECTOR_OFFSET  (BCD_SECTOR_OFFSET+MTD_SIZE_BCD)

#endif /* GXV32xx */

#define CONFIG_GS_BOOT_TEST
//#if defined (CONFIG_GVR3550) || defined (CONFIG_GXV3275) || defined (CONFIG_GXV3240)
#if defined (CONFIG_GXV3275) || defined (CONFIG_GXV3240)
#define CONFIG_SUNXI_WATCHDOG
#endif
#define DEBUG_BOOTKEYS  1

/* added by grandstream.lfzhu  end*/
#endif /* __CONFIG_H */
